package de.kreawi.trainer.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Collections;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif_lite.component.Factory;

import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.help.HelpRegistry;

@SuppressWarnings("serial")
public class HelpDialog extends KreawiDialogPanel {

	private HelpRegistry helpRegistry;
	private JTextPane textPane;
	private KreawiInternalFrame textInternalFrame;

  private static final Logger logger = LoggerFactory.getLogger(HelpDialog.class); 

	/**
	 * Builds the dialog.
	 */
	public HelpDialog(HelpRegistry helpRegistry) {

		this.helpRegistry = helpRegistry;

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());

		int width = TrainerFrame.MEDIUM_DIALOG_SIZE.width;
		int height = TrainerFrame.MEDIUM_DIALOG_SIZE.height - 270;

		panel.setPreferredSize(new Dimension(width, height));
		panel.setMinimumSize(panel.getPreferredSize());

		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		JSplitPane horizontalSplit = buildHorizontalSplit(
				buildSelectionPanel(), buildTextPanel(), 0.4d);

		panel.add(horizontalSplit, gbc);
		add(panel);
	}

	protected KreawiInternalFrame buildSelectionPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("MENU_ITEM_HELP_CONTENTS"), new BorderLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		Vector<String> listData = new Vector<String>(this.helpRegistry
				.getHelpItemsIds());
		Collections.sort(listData);
		
		final JList<String> list = new JList<String>(listData);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		list.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				if (list.getSelectedValue() instanceof String) {
					String value = (String) list.getSelectedValue();
					textPane.setText(helpRegistry.getHelpItemText(value));
					textPane.setCaretPosition(0);
					textInternalFrame.setTitle(helpRegistry
							.getHelpItemTitle(value));
				}
			}

		});
		list.setOpaque(false);
		list.setCellRenderer(new DefaultListCellRenderer() {
			public Component getListCellRendererComponent(JList<?> list,
			    Object value, int index, boolean isSelected,
					boolean cellHasFocus) {

				boolean isEnabled = true;

				if (value instanceof String) {
					String helpItemId = (String) value;

					value = helpRegistry.getHelpItemTitle(helpItemId);

				}

				JComponent ret = (JComponent) super
						.getListCellRendererComponent(list, value, index,
								isSelected, false);

				ret.setEnabled(isEnabled);

				if (!(isSelected && isEnabled)) {
					ret.setOpaque(false);
				} else {
					ret.setOpaque(true);
				}

				return ret;

			}
		});

		JScrollPane scrollPane = Factory.createStrippedScrollPane((list));
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);

		gradientPanel.add(scrollPane);

		setFocusJComponent(list);

		return kif;
	}

	protected KreawiInternalFrame buildTextPanel() {

		this.textInternalFrame = new KreawiInternalFrame(" ",
				new BorderLayout());

		JPanel gradientPanel = textInternalFrame.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		this.textPane = new JTextPane() {

			@Override
			public void setText(String t) {
				final String css = TrainerHtmlUtil.getDefaultCss(true);
        super.setText(TrainerHtmlUtil.wrapInHtml(t, css));
			}

		};

		textPane.setEditable(false);
		textPane.setOpaque(false);
		textPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));  // border created in HTML
		textPane.setContentType("text/html");
    textPane.setText(TrainerTextResources.getString("HELP_DIALOG_WELCOME_TEXT"));
		textPane.setCaretPosition(0);

		JScrollPane scrollPane = Factory.createStrippedScrollPane(textPane);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		gradientPanel.add(scrollPane);

		return textInternalFrame;
	}

}
