package de.kreawi.trainer.dialogs;

import java.awt.GridBagLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.factories.Borders;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.dialogs.onlineGame.OnlineGameContainer;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.OnlineGameMain.OnlineStatusChangedListener;
import de.kreawi.trainer.onlinegame.OnlineGameModel.OnlineGameStatus;
import de.kreawi.trainer.util.TrainerTextResources;
import de.kreawi.trainer.util.swingx.DesktopApi;
import edu.stanford.ejalbert.BrowserLauncher;
import javafx.application.Platform;
import javafx.embed.swing.JFXPanel;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.AmbientLight;
import javafx.scene.Cursor;
import javafx.scene.Node;
import javafx.scene.PointLight;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.Hyperlink;
import javafx.scene.control.RadioMenuItem;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.Tooltip;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Background;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.RowConstraints;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Sphere;
import javafx.scene.text.Font;
import javafx.scene.text.FontPosture;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;
import javafx.scene.text.TextFlow;

public class MainDialogFx extends JPanel {
 
  private static final long serialVersionUID = -6119261591854596598L;

  public static final int DIALOG_WIDTH = 900;

  public static final int DIALOG_HEIGHT = 600;

  public static final String LOGO_PATH = KreawiGlobals.APP_IMAGES_DIR_NAME + "/logo_nur kreis_transparent_sw.gif";

  public static final double GRID_SPACING = 8d; 

  private final Map<Actions, Action> actionMap;

  private final TrainerModel model;
  
  private BorderPane rootPane;

  private Region mcMenu;

  private Region oralMenu;

  private Region infoBar;

  private final TrainerFrame trainerFrame;

  private static final Logger logger = LoggerFactory.getLogger(MainDialogFx.class);

  public MainDialogFx(final Map<Actions, Action> actionMap, final TrainerModel model, final TrainerFrame trainerFrame) {
    
    this.actionMap = actionMap;
    
    this.model = model;
    
    this.trainerFrame = trainerFrame;
    
    setLayout(new GridBagLayout());
    setOpaque(false);
    setBorder(Borders.DIALOG_BORDER);

    final JFXPanel jfxPanel = new JFXPanel();
    
    this.add(jfxPanel);
    
    Platform.runLater(new Runnable() {
      @Override
      public void run() {
          initFX(jfxPanel);
      }
    });
    
  }

  protected void initFX(final JFXPanel jfxPanel) {
    
    this.mcMenu = buildWrittenExamsPanel();
    
    this.oralMenu = buildOralExamsPanel();
    
    this.infoBar = buildLowerPanel();
    
    this.rootPane = new BorderPane();  // for centering contents
    rootPane.setBackground(Background.EMPTY);
    initRootPaneContent();
    
    final Scene scene = new Scene(this.rootPane);
    scene.setFill(Color.TRANSPARENT);
    
    jfxPanel.setScene(scene);
    jfxPanel.setOpaque(false);
    
  }

  public void initRootPaneContent() {
    final GridPane gridpane = new GridPane();
    gridpane.setMinSize(DIALOG_WIDTH, DIALOG_HEIGHT);
    gridpane.setPrefSize(DIALOG_WIDTH, DIALOG_HEIGHT);
    gridpane.setMaxSize(DIALOG_WIDTH, DIALOG_HEIGHT);
    gridpane.setBackground(Background.EMPTY);

    RowConstraints row1 = new RowConstraints();
    row1.setMinHeight(300d);
    row1.setVgrow(Priority.ALWAYS);
    RowConstraints row2 = new RowConstraints();
    row2.setMinHeight(120d);
    gridpane.getRowConstraints().addAll(row1, row2);

    GridPane.setMargin(mcMenu, new Insets(GRID_SPACING));
    GridPane.setMargin(oralMenu, new Insets(GRID_SPACING, GRID_SPACING, GRID_SPACING, 0));
    GridPane.setMargin(infoBar, new Insets(0, GRID_SPACING, GRID_SPACING, GRID_SPACING));

    final Region allContentsPane;
    final boolean showOnlineGame = this.model.getSettings()
        .getBooleanValue(TrainerSettingsConstants.KEY_SHOW_ONLINE_GAME);
    if (showOnlineGame) {

      final OnlineGameContainer ogc = new OnlineGameContainer(this.model.getOnlineGame(), this.trainerFrame);
      final Node onlineGameNode = new KreawiInternalFrameFx("Quiz-Duell", ogc);
      
      GridPane.setMargin(onlineGameNode, new Insets(GRID_SPACING, GRID_SPACING, GRID_SPACING, 0));

      ColumnConstraints col1 = new ColumnConstraints();
      ColumnConstraints col2 = new ColumnConstraints();
      ColumnConstraints col3 = new ColumnConstraints();
      col1.setPercentWidth(34);
      col2.setPercentWidth(33);
      col3.setPercentWidth(33);
      
      gridpane.getColumnConstraints().addAll(col1,col2,col3);

      gridpane.addRow(0, mcMenu, oralMenu, onlineGameNode);
      gridpane.add(infoBar, 0, 1, 3, 1);

      final StackPane stackpane = new StackPane();

      final Runnable updateOnlineStatusButton = () -> {
        stackpane.getChildren().clear();
        stackpane.getChildren().addAll(gridpane, createOnlineStatusButtonLayer());
      };
      
      this.model.getOnlineGame().addStatusChangedListener(new OnlineStatusChangedListener() {
        @Override
        public void onStatusChanged(OnlineGameStatus oldStatus, OnlineGameStatus newStatus) {
          Platform.runLater(() -> {
            updateOnlineStatusButton.run();
          });
        }
      });
      
      updateOnlineStatusButton.run();
      
      allContentsPane = stackpane;
    } else {

      ColumnConstraints col1 = new ColumnConstraints();
      ColumnConstraints col2 = new ColumnConstraints();
      col1.setPercentWidth(50d);
      col2.setPercentWidth(50d);
      gridpane.getColumnConstraints().addAll(col1,col2);
      
      gridpane.addRow(0, mcMenu, oralMenu);
      gridpane.add(infoBar, 0, 1, 2, 1);
      
      allContentsPane = gridpane;
    }
    
    this.rootPane.setCenter(allContentsPane);
  }

  private Pane createOnlineStatusButtonLayer() {
    final double onlineStatusButtonRadius = 30d;
    final Sphere onlineStatusButton = new Sphere(onlineStatusButtonRadius);
    final Color color;
    final OnlineGameStatus currentOnlineStatus = this.model.getOnlineGame().getStatus();
    final int lightDistance;
    if (OnlineGameStatus.ONLINE.equals(currentOnlineStatus)) {
      color = Color.GREEN;
      lightDistance = -700;
    } else if (OnlineGameStatus.BUSY.equals(currentOnlineStatus)) {
      color = Color.YELLOW;
      lightDistance = -500;
    } else {
      color = Color.RED;
      lightDistance = -500;
    }
    PhongMaterial statusButtonMaterial = new PhongMaterial(color);
//    statusButtonMaterial.setSpecularColor(color);
    onlineStatusButton.setMaterial(statusButtonMaterial);
    onlineStatusButton.translateXProperty().bind(this.mcMenu.widthProperty().add(1.5 * GRID_SPACING));
    onlineStatusButton.translateYProperty().bind(this.mcMenu.heightProperty().add(1.5 * GRID_SPACING));
    onlineStatusButton.setCursor(Cursor.HAND);
    onlineStatusButton.setOnMouseClicked(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        if (MainDialogFx.this.model.getOnlineGame().isLoggedIn()) {
          final OnlineGameStatus status = MainDialogFx.this.model.getOnlineGame().getStatus();
          final ContextMenu contextMenu = createOnlineStatusContextMenu(status);
          contextMenu.show(onlineStatusButton, event.getScreenX(), event.getScreenY());
        } else {
          final String message = "Bitte schließen Sie erst die Registrierung des Quiz-Duell-Benutzers ab!";
          final Alert alert = new Alert(AlertType.WARNING, message);
          alert.setTitle(TrainerTextResources.getString("APPLICATION_TITLE"));
          alert.show();
        }
      }
    });
    Tooltip tootip = new Tooltip("Online-Status des Quiz-Duells.\nKlicken Sie hier um den Status zu ändern!");
    Tooltip.install(onlineStatusButton, tootip);
    final AmbientLight ambientLight = new AmbientLight();
    ambientLight.setColor(Color.GRAY);
    final PointLight pointLight = new PointLight();
    pointLight.setColor(Color.WHITE);
    pointLight.setLayoutX(500);
    pointLight.setLayoutY(300);
    pointLight.setTranslateZ(lightDistance);
    pointLight.getScope().add(onlineStatusButton);
    final Pane onlineStatusButtonLayer = new Pane(onlineStatusButton, ambientLight, pointLight);
    onlineStatusButtonLayer.setPickOnBounds(false);
    return onlineStatusButtonLayer;
  }
  
  protected ContextMenu createOnlineStatusContextMenu(final OnlineGameStatus currentStatus) {

    final Runnable onSuccess = () -> {
      logger.info("Successfully changed online status!");
    };
    final Runnable onFailure = () -> {
      logger.error("Failed to change online status!");
    };
    
    final OnlineGameMain onlineGame = this.model.getOnlineGame();
    
    ToggleGroup group = new ToggleGroup();
    
    final RadioMenuItem radioSetOnline = new RadioMenuItem("Offen für Einladungen");
    radioSetOnline.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        final OnlineGameStatus newStatus = OnlineGameStatus.ONLINE;
        onlineGame.setStatus(newStatus, onSuccess, onFailure);
      }
    });
    radioSetOnline.setToggleGroup(group);
    radioSetOnline.setSelected(OnlineGameStatus.ONLINE.equals(currentStatus));
    
    final RadioMenuItem radioSetBusy = new RadioMenuItem("Keine neuen Einladungen");
    radioSetBusy.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        final OnlineGameStatus newStatus = OnlineGameStatus.BUSY;
        onlineGame.setStatus(newStatus, onSuccess, onFailure);
      }
    });
    radioSetBusy.setToggleGroup(group);
    radioSetBusy.setSelected(OnlineGameStatus.BUSY.equals(currentStatus));
    
    final RadioMenuItem radioSetOffline = new RadioMenuItem("Alle Spiele pausieren");
    radioSetOffline.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        final OnlineGameStatus newStatus = OnlineGameStatus.OFFLINE;
        onlineGame.setStatus(newStatus, onSuccess, onFailure);
      }
    });
    radioSetOffline.setToggleGroup(group);
    radioSetOffline.setSelected(OnlineGameStatus.OFFLINE.equals(currentStatus));
    
    final ContextMenu cm = new ContextMenu();
    cm.getItems().addAll(radioSetOnline, radioSetBusy, radioSetOffline);
    
    return cm;
  }

  private Region buildWrittenExamsPanel() {
    final boolean mc = true;
    return buildSelectionPanel(mc, this.actionMap.get(Actions.MC_RANDOM_QUIZ),
        this.actionMap.get(Actions.WRITTEN_BY_CATEGORY), this.actionMap.get(Actions.WRITTEN_BY_EXAM),
        this.actionMap.get(Actions.MC_INTENSIVE_CARE), this.actionMap.get(Actions.MC_HISTORY));
  }

  private Region buildOralExamsPanel() {
    final boolean mc = false;
    return buildSelectionPanel(mc, this.actionMap.get(Actions.TEXT_RANDOM_QUIZ),
        this.actionMap.get(Actions.ORAL_BY_CATEGORY), this.actionMap.get(Actions.ORAL_BY_EXAM),
        this.actionMap.get(Actions.TEXT_INTENSIVE_CARE), this.actionMap.get(Actions.TEXT_HISTORY));
  }

  private Region buildSelectionPanel(final boolean mc, final Action randomQuizAction,
      final Action categoryAction, final Action examAction, final Action icuAction, final Action historyAction) {

    final Text captionSmall = new Text("Training für die");
    captionSmall.setTextAlignment(TextAlignment.CENTER);
    final Text captionBig = new Text((mc?"Schriftliche":"Mündliche") + " Prüfung");
    captionBig.setTextAlignment(TextAlignment.CENTER);
    captionBig.setFont(Font.font("Arial", FontWeight.BOLD, 16));
    final String expl;
    if (mc) {
      expl = "Die schriftliche Prüfung besteht aus\nFragen nach dem\nMultiple-Choice-Prinzip.";
    } else {
      expl = "Die mündliche Prüfung besteht\naus reinen Textfragen\nohne eindeutige Antwortmöglichkeit.";
    }
    final Text explanation = new Text(expl);
    explanation.setTextAlignment(TextAlignment.CENTER);
    
    final Region gapBeforeExplanation = new Region();
    gapBeforeExplanation.setPrefHeight(30);
    final VBox top = new VBox(5, captionSmall, captionBig, gapBeforeExplanation, explanation);
    top.setFillWidth(true);
    top.setAlignment(Pos.CENTER);
    
    
    final Text selectionLabel = new Text("Auswählen:");
    final Button randomQuizButton = getButtonFromSwingAction(randomQuizAction);
    final Button categoryButton = getButtonFromSwingAction(categoryAction);
    final Button examButton = getButtonFromSwingAction(examAction);
    final Button icuButton = getButtonFromSwingAction(icuAction);
    final Button historyButton = getButtonFromSwingAction(historyAction);
    
    randomQuizButton.setMaxWidth(Double.MAX_VALUE);
    categoryButton.setMaxWidth(Double.MAX_VALUE);
    examButton.setMaxWidth(Double.MAX_VALUE);
    icuButton.setMaxWidth(Double.MAX_VALUE);
    historyButton.setMaxWidth(Double.MAX_VALUE);
    final Region groupGap1 = new Region();
    final Region groupGap2 = new Region();
    final int groupGapHeight = 5;
    groupGap1.setPrefHeight(groupGapHeight);
    groupGap2.setPrefHeight(groupGapHeight);
    final VBox buttonPane = new VBox(10, selectionLabel, 
        randomQuizButton, groupGap1, categoryButton, 
        examButton, groupGap2, icuButton, historyButton);
    buttonPane.setBackground(Background.EMPTY);
    buttonPane.setFillWidth(true);
    
    final BorderPane borderPane = new BorderPane();
    borderPane.setCenter(buttonPane);
    buttonPane.maxWidthProperty().bind(borderPane.widthProperty().multiply(.6d));
//    BorderPane.setMargin(buttonPane, new Insets(20, 65, 0, 65));
    BorderPane.setAlignment(buttonPane, Pos.CENTER);
    
    
    borderPane.setTop(top);
    BorderPane.setMargin(top, new Insets(20));
    
    borderPane.setBackground(Background.EMPTY);

    final String title = "Training " + (mc?"Schriftliche":"Mündliche") + " Prüfung";
    return new KreawiInternalFrameFx(title, borderPane);
  }

  private Button getButtonFromSwingAction(Action swingAction) {
    final Button button = new Button((String) swingAction.getValue(Action.NAME));
    button.setTooltip(new Tooltip((String) swingAction.getValue(Action.SHORT_DESCRIPTION)));
    button.setDisable(!swingAction.isEnabled());
    swingAction.addPropertyChangeListener(new PropertyChangeListener() {
      @Override
      public void propertyChange(PropertyChangeEvent evt) {
        switch (evt.getPropertyName()) {
        case Action.SHORT_DESCRIPTION:
          button.setTooltip(new Tooltip((String) swingAction.getValue(Action.SHORT_DESCRIPTION)));
          break;
        case "enabled":
          button.setDisable(!swingAction.isEnabled());
          break;
        }
      }
    });
    button.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        SwingUtilities.invokeLater(() -> {
          swingAction.actionPerformed(null);
        });
      }
    });
    return button;
  }

  private Region buildLowerPanel() {
    return buildAppTitlePanel();
  }

  public static Region buildAppTitlePanel() {

    final String buyUrl = "https://www.kreawiverlag.de/software/";
    final double trainerFontSize = 25;
    final Text kreawi = new Text("kreawi ");
    kreawi.setFont(Font.font("Arial", FontWeight.BOLD, FontPosture.REGULAR, trainerFontSize));
    final Text trainer = new Text("Prüfungstrainer");
    trainer.setFont(Font.font("Arial", FontWeight.BOLD, FontPosture.ITALIC, trainerFontSize));
    final Text year = new Text(" " + KreawiGlobals.CURRENT_YEAR);
    year.setFont(Font.font("Arial", FontWeight.BOLD, FontPosture.REGULAR, trainerFontSize));
    final TextFlow trainerComplete = new TextFlow(kreawi, trainer, year);
    trainerComplete.setTextAlignment(TextAlignment.RIGHT);
    
    final Text copyright = new Text("\u00A9 2007-" + KreawiGlobals.CURRENT_YEAR + " Kreativität & Wissen Verlag");
    copyright.setFont(Font.font("Arial", 10));
    final VBox titleText = new VBox(3, trainerComplete, copyright);
    
    final BorderPane borderPane = new BorderPane();
    titleText.setAlignment(Pos.CENTER_LEFT);
    titleText.setFillWidth(false);
    titleText.maxWidthProperty().bind(trainerComplete.prefWidthProperty());
    borderPane.setLeft(titleText);
    BorderPane.setAlignment(  titleText, Pos.CENTER_LEFT);
    BorderPane.setMargin(titleText, new Insets(20d));
    
    if (KreawiGlobals.isDemo()) {
      
      final Text demo = new Text("Hier können Sie die Vollversion kaufen: ");
      demo.setFill(Color.RED);
      
      final Hyperlink demoLink = new Hyperlink(buyUrl);
      demoLink.setStyle("-fx-border-color: transparent;-fx-padding: 0;");
      demoLink.setOnAction(new EventHandler<ActionEvent>() {
        @Override
        public void handle(ActionEvent event) {
          SwingUtilities.invokeLater(() -> {
            try {
              logger.info("Buy-Full-Version-URL clicked.");
              final URI uri = new URI(buyUrl);
              DesktopApi.browse(uri);
            } catch (Exception ex) {
              logger.error("Desktop.browse hat nicht funktioniert! (url: " + buyUrl + ")", ex);
              logger.info("Versuche BrowserLauncher...");
              try {
                (new BrowserLauncher()).openURLinBrowser(buyUrl);
              } catch (Exception ex2) {
                logger.error("Konnte URL nicht öffnen! (url: " + buyUrl + ")", ex2);
              }
            }
          });
        }
      });
      final VBox lines = new VBox(2, demo, demoLink);
      lines.setAlignment(Pos.CENTER_LEFT);
      final HBox demoText = new HBox(lines);
      demoText.setAlignment(Pos.CENTER);
      borderPane.setCenter(demoText);
      BorderPane.setAlignment(demoText, Pos.CENTER);
      BorderPane.setMargin(demoText, new Insets(20d));
    }
    
    final ImageView logo;
    try (
        final InputStream is = TrainerMain.class.getResourceAsStream(LOGO_PATH);
        ) {
      logo = new ImageView(new Image(is));
    } catch (IOException e) {
      logger.error("Logo-Datei nicht gefunden! (LOGO_PATH: " + LOGO_PATH + ")", e);
      return new KreawiInternalFrameFx(new Pane());
    }
    
    borderPane.setRight(logo);
    BorderPane.setAlignment(logo, Pos.CENTER);
    BorderPane.setMargin(logo, new Insets(0, 20, 0, 50));

    return new KreawiInternalFrameFx(borderPane);
  }

}
