package de.kreawi.trainer.dialogs.etc;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.RenderingHints;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.factories.Borders;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.dialogs.KreawiDialogPanel;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.martinknauer.util.Util;

@SuppressWarnings("serial")
public class DmgBackgroundDialog extends KreawiDialogPanel {

	public static final String LOGO_PATH = KreawiGlobals.APP_IMAGES_DIR_NAME + "/logo_nur kreis_transparent_sw.gif";

	@SuppressWarnings("unused")
	private Map<Actions, Action> actionMap;

  private static final Logger logger = LoggerFactory.getLogger(DmgBackgroundDialog.class); 

	/**
	 * Builds the dialog.
	 */
	public DmgBackgroundDialog(Map<Actions, Action> actionMap) {

		this.actionMap = actionMap;

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());

		int width = TrainerFrame.SMALL_DIALOG_SIZE.width - 200;
		int height = TrainerFrame.SMALL_DIALOG_SIZE.height - 150;

		panel.setPreferredSize(new Dimension(width, height));
		panel.setMinimumSize(panel.getPreferredSize());

		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		
		panel.add(buildLowerPanel(), gbc);
		add(panel);
	}

	private JComponent buildLowerPanel() {
//		UIManager.put("control", UIManager.getColor("control").darker());

		KreawiInternalFrame sif = new KreawiInternalFrame(null,
				new BorderLayout());

		JPanel gradientPanel = sif.getContent();
//		sif.setContent(gradientPanel);
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(16, 16, 4, 16));
//		gradientPanel.setBackground(Color.WHITE);
		
//		String text = "<html><p align=\"center\">Ziehen Sie das 'kreawiTrainer'-Symbol auf das 'Programme'-Symbol, um den kreawi Prüfungstrainer zu installieren.</p></html>";
//		String text = "<html><p align=\"center\">Bitte klicken Sie doppelt auf das <nobr>'kreawiTrainer Update'-Symbol</nobr>, um den <nobr>kreawi Prüfungstrainer</nobr> zu aktualisieren.</p></html>";
//		String text = "<html><p align=\"center\">Bitte klicken Sie doppelt auf das <nobr>'kreawiTrainer Upgrade'-Symbol</nobr>, um das Upgrade für den <nobr>kreawi Prüfungstrainer</nobr> einzuspielen.</p></html>";
		String text = "<html><p align=\"center\">Ziehen Sie das <nobr>'kreawiTrainer Pro'-Symbol</nobr> auf das <nobr>'Programme'-Symbol</nobr>, um den <nobr>kreawi Prüfungstrainer</nobr> zu installieren.</p></html>";
		
		JLabel expl = new JLabel(text) {
			public void paint(Graphics g) {

				Graphics2D g2d = (Graphics2D) g;

				try {
					g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
							Util.getAATextRenderingHintValue());
				} catch (ClassNotFoundException e) {
				  logger.info("Text-Anti-Aliasing is not supported!");
				}

				super.paint(g2d);
			}
		};
		
		expl.setFont(new Font(expl.getFont().getFontName(), expl.getFont()
				.getStyle(), 18));
		
		gradientPanel.add(expl, BorderLayout.NORTH);

		JPanel labelPanel = new JPanel(new BorderLayout());
		labelPanel.setOpaque(false);
		JPanel innerPanel = new JPanel(new FlowLayout());
		innerPanel.setOpaque(false);
		JLabel imageLabel = new JLabel(new ImageIcon(TrainerMain.class
				.getResource(LOGO_PATH)));
		JLabel textLabel = new JLabel("<html><font size=\"6\"><b>kreawi <i>Prüfungstrainer  </i></b></font><br>\u00A9 Kreativität & Wissen Verlag und Buchhandel GmbH <br></html>") {
			public void paint(Graphics g) {

				Graphics2D g2d = (Graphics2D) g;

				try {
					g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
							Util.getAATextRenderingHintValue());
				} catch (ClassNotFoundException e) {
				  logger.info("Text-Anti-Aliasing is not supported!");
				}

				super.paint(g2d);
			}
		};

		textLabel.setOpaque(false);

		innerPanel.add(textLabel);
		innerPanel.add(imageLabel);
		labelPanel.add(innerPanel, BorderLayout.EAST);
		JPanel gluePanel = new JPanel();
		gluePanel.setOpaque(false);
		labelPanel.add(gluePanel, BorderLayout.SOUTH);

		gradientPanel.add(labelPanel, BorderLayout.SOUTH);

		return sif;
	}

}
