package de.kreawi.trainer.dialogs.history;

import java.awt.Color;
import java.awt.Component;
import java.text.DateFormat;
import java.text.NumberFormat;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;

import org.jdesktop.swingx.JXTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jquizengine.quiz.Quiz;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.quiz.KreawiQuizHelper;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.awt.ColorUtil;
import de.martinknauer.util.Util;

public class HistoryCellRenderer extends DefaultTableCellRenderer {

	private static final long serialVersionUID = -1305351930240399543L;

  private static final Logger logger = LoggerFactory.getLogger(HistoryCellRenderer.class); 

	public HistoryCellRenderer() {
		setHorizontalAlignment(JLabel.CENTER);
	}

	@Override
	protected void setValue(Object value) {
		if (value == null) {
			setText("");
			setIcon(null);
		} else if (value instanceof String) {
			setText((String) value);
			setIcon(null);
		} else if (value instanceof Integer) {
			setText(value.toString());
			setIcon(null);
		} else if (value instanceof Double) {
			setText(NumberFormat.getPercentInstance().format((Double) value));
			setIcon(null);
		} else if (value instanceof Long) {
			setText(getDateTimeString((Long) value));
			setIcon(null);
		} else if (value instanceof Quiz) {
      setText(getDateTimeString(((Quiz) value).getStartTime()));
      setIcon(null);
		} else if (value instanceof ImageIcon) {
			setText("");
			setIcon((ImageIcon) value);
		} else {
      super.setValue(TrainerTextResources.getString("HISTORY_TABLE_ERROR"));
		}
	}

	private String getDateTimeString(Long timestamp) {
		return DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
				DateFormat.SHORT).format(timestamp);
	}

  @Override
  public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
      int row, int column) {
    Component comp = super.getTableCellRendererComponent(table, value, isSelected, false, row, column);
    
    Color color = null;
    
    if (table instanceof JXTable) {
      JXTable jxtable = (JXTable) table;
      
      if (jxtable.getModel() instanceof McQuizHistoryTableModel) {
        McQuizHistoryTableModel model = (McQuizHistoryTableModel) jxtable.getModel();
        
        int convertedRow = jxtable.convertRowIndexToModel(row);
        
        final KreawiQuiz quiz = model.getHistoryItems().get(convertedRow).getQuiz();
        final Double correct = KreawiQuizHelper.getCorrect(quiz);
        final Double progress = KreawiQuizHelper.getProgress(quiz);
        
        if (correct != null && (double) correct >= KreawiGlobals.SUCCESS_LIMIT) {
          color = KreawiGlobals.CORRECT_COLOR;
        } else if (correct != null && progress != null
            && ((double) progress - (double) correct) > (1f - KreawiGlobals.SUCCESS_LIMIT)) {
          color = KreawiGlobals.WRONG_COLOR;
        }
        
        if (color != null) {
          
          if (!isSelected) {
            comp.setForeground(color);
            comp.setBackground(UIManager.getColor("Table.background"));
          } else {
            comp.setForeground(Color.WHITE);
            // comp.setForeground(UIManager
            // .getColor("Table.selectionForeground"));
            comp.setBackground(color);
          }
          
        } else {
          
          if (!isSelected) {
            comp.setForeground(UIManager.getColor("Table.foreground"));
            comp.setBackground(UIManager.getColor("Table.background"));
          } else {
            
            comp.setForeground(UIManager.getColor("Table.selectionForeground"));
            if (!Util.isMacOS()) {
              comp.setBackground(UIManager.getColor("Table.selectionBackground"));
            } else {
              Color background = ColorUtil.convertToGrayscale(UIManager.getColor("Table.selectionBackground"));
              comp.setBackground(background);
            }
          }
          
        }
        
      }
    }
    
    return comp;
  }
	
}
