package de.kreawi.trainer.dialogs.history;

import java.util.Comparator;
import java.util.List;

import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerStorage.TextHistoryChangedListener;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.quiz.KreawiQuizHelper;
import de.kreawi.trainer.util.TrainerTextResources;

public class TextQuizHistoryTableModel extends AbstractHistoryTableModel {

	private static final long serialVersionUID = 6309368417630323714L;

	private final static String[] COLUMNS = {
			TrainerTextResources.getString("HISTORY_COLUMN_START"),
			TrainerTextResources.getString("HISTORY_COLUMN_MODIFIED"),
			TrainerTextResources.getString("HISTORY_COLUMN_CATEGORY_EXAM"),
			TrainerTextResources.getString("HISTORY_COLUMN_QUESTIONS"),
			TrainerTextResources.getString("HISTORY_COLUMN_DONE"),
			TrainerTextResources.getString("HISTORY_COLUMN_TIME") };

	public TextQuizHistoryTableModel(final TrainerModel model) {
		super(model);
		model.getStorage().addTextHistoryChangedListener(new TextHistoryChangedListener() {
      @Override
      public void historyChanged() {
        fireTableDataChanged();
      }
    });;
	}

	public int getColumnCount() {
		return COLUMNS.length;
	}

	@Override
	public String getColumnName(int column) {
		return COLUMNS[column];
	}

	List<HistoryItem> getHistoryItems() {
     return model.getHistoryItemsText();
  }

	@SuppressWarnings("rawtypes")
  @Override
	public Comparator getComparator(final int columnIndex) {
	  switch (columnIndex) {
	  case 0:
	    return new QuizComparator();
	  default:
	    return new Comparator<Object>() {
	      public int compare(Object o1, Object o2) {
	        return o1.toString().compareTo(o2.toString());
	      }
	    };
	  }
	}

	@Override
	protected Object getValue(int rowIndex, int columnIndex) {
	  
    final KreawiQuiz quiz = getHistoryItems().get(rowIndex).getQuiz();
    
    switch (columnIndex) {
    case 0:
      return quiz;
    case 1:
      return quiz.getLastChangedTime();
    case 2:
      return KreawiQuizHelper.getTitle(quiz);
    case 3:
      return new Integer(quiz.getQuestionCount());
    case 4:
      return KreawiQuizHelper.getProgress(quiz);
    case 5:
      return quiz.getStopwatch().getOverallTimeTakenString();
    }

    return TrainerTextResources.getString("HISTORY_TABLE_ERROR");
	}

}
