package de.kreawi.trainer.dialogs.onlineGame.custom_nodes;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.dialogs.onlineGame.OnlineGameContainer;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.OnlineGameSettings;
import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Cursor;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.text.Text;

public class ProfileHeader extends BorderPane {
  
  @SuppressWarnings("unused")
  private static final Logger logger = LoggerFactory.getLogger(ProfileHeader.class);
  private final OnlineGameMain onlineGame;
  private final OnlineGameContainer onlineGameContainer; 

  public ProfileHeader(final OnlineGameMain onlineGame, final OnlineGameContainer onlineGameContainer) {
    
    this.onlineGame = onlineGame;
    this.onlineGameContainer = onlineGameContainer;
    
    getStylesheets().add("/de/kreawi/trainer/dialogs/onlineGame/online_game.css");
    
    getStyleClass().add("profile-header");
    
    final OnlineGameUser profileUser = loadProfileImage();
    
    final Text caption = new Text(profileUser.getUserName());
    BorderPane.setAlignment(caption, Pos.CENTER_LEFT);
    BorderPane.setMargin(caption, new Insets(15));
    setCenter(caption);
    
  }

  public OnlineGameUser loadProfileImage() {
    final OnlineGameSettings settings = this.onlineGame.getModel().getSettings();
    final OnlineGameUser profileUser = settings.getProfileUser();
    
    final ProfileImage profileImage = new ProfileImage(profileUser, true);
    profileImage.setCursor(Cursor.HAND);
    profileImage.setOnMouseClicked(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        if (!event.getButton().equals(MouseButton.PRIMARY)) {
          return;
        }
        ProfileHeader.this.onlineGameContainer.updateProfileImage();
      }
    });
    setLeft(profileImage);
    return profileUser;
  }
  
}
