package de.kreawi.trainer.dialogs.onlineGame.custom_nodes;

import javafx.event.EventHandler;
import javafx.scene.Cursor;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;

public class TopicSelector extends StackPane {
  
  private final double BORDER_WIDTH = 3d;
  
  public TopicSelector(final String topicName, final Color backColor, final Runnable action) {
    
    final double r = backColor.getRed();
    final double g = backColor.getGreen();
    final double b = backColor.getBlue();

    final Circle lowestCircle = new Circle();
    lowestCircle.radiusProperty().bind(widthProperty().divide(2d));
    lowestCircle.setFill(new Color(r, g, b, .3));
    
    final Circle middleCircle = new Circle();
    middleCircle.radiusProperty().bind(widthProperty().divide(2d).subtract(BORDER_WIDTH / 2));
    middleCircle.setFill(new Color(r, g, b, .3));
    
    final Circle circle = new Circle();
    circle.radiusProperty().bind(widthProperty().divide(2d).subtract(BORDER_WIDTH));
    circle.setFill(backColor);
    circle.setOnMouseClicked(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        action.run();
        event.consume();
      }
    });
    circle.setCursor(Cursor.HAND);
    
    final Text caption = new Text(topicName.replace("/", " / ").replace("(A / P)", "(A/P)"));
    caption.setTextAlignment(TextAlignment.CENTER);
    final double captionWidth = .9;
    caption.wrappingWidthProperty().bind(maxWidthProperty().multiply(captionWidth));
    caption.setFill(Color.web("#eee"));
    caption.setFont(Font.font("Arial", FontWeight.BOLD, 11d));
    caption.setOnMouseClicked(new EventHandler<MouseEvent>() {
      @Override
      public void handle(MouseEvent event) {
        action.run();
        event.consume();
      }
    });
    caption.setCursor(Cursor.HAND);
    
    
    getChildren().addAll(lowestCircle, middleCircle, circle, caption);
  }
}
