package de.kreawi.trainer.dialogs.onlineGame.custom_nodes;

import java.util.ArrayList;

import de.kreawi.trainer.dialogs.onlineGame.OnlineGameContainer;
import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;

public class UserListItemView extends BorderPane {
 
  private static final double SPACING = 10d;
  private final OnlineGameUser user;
  private final OnlineGameContainer ogc;

  public UserListItemView(final OnlineGameUser user, final OnlineGameContainer ogc) {
    super();
    
//    setStyle(" -fx-border-color: green; -fx-border-style: solid; -fx-border-width: 1px;");
    this.user = user;
    this.ogc = ogc;
    
    final ProfileImage image = new ProfileImage(this.user);
    setLeft(image);
    BorderPane.setMargin(image, new Insets(3d, 0, 3d, 3d));
    
    final Text username = new Text(this.user.getUserName());
    username.setFont(Font.font("Arial", FontWeight.BOLD, 13));
    setCenter(username);
    BorderPane.setAlignment(username, Pos.CENTER_LEFT);
    BorderPane.setMargin(username, new Insets(SPACING, SPACING, SPACING, SPACING));
    
    final HBox buttonsBox = new HBox(SPACING, generateButtons());
    buttonsBox.setAlignment(Pos.CENTER);
    setRight(buttonsBox);
    BorderPane.setMargin(buttonsBox, new Insets(SPACING, SPACING, SPACING, 0));
    
  }

  private OGButton[] generateButtons() {
    final ArrayList<OGButton> buttons = new ArrayList<OGButton>();
    
    if (this.user.isOpen()) {
      // users in friend list may not be open (busy status / not logged in)
      final Runnable inviteAction = () -> {
        this.ogc.inviteUser(this.user);
      };
      final OGButton invite = new OGButton("Einl.", "Diesen Benutzer zu einem Spiel einladen.", inviteAction);
      buttons.add(invite);
    }
    
    if (user.isFriend()) {
      final Runnable removeAction = () -> {
        this.ogc.removeFriend(this.user);
      };
      final OGButton removeFriend = OGButton.generateRejectButton("Entfernen aus der Freundesliste",
          removeAction);
      buttons.add(removeFriend);
    }
    OGButton[] a = new OGButton[buttons.size()];
    return buttons.toArray(a);
  }

  public OnlineGameUser getUser() {
    return this.user;
  }
}
