package de.kreawi.trainer.dialogs.quiz;

import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.db.QuestionAttributesModel.CardBoxChangedListener;
import de.kreawi.trainer.db.TrainerSettingsModel.SettingsChangedListener;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;

public class CardBoxPanel extends JPanel {
  
  private final TrainerModel model;
  private final TrainerFrame mainframe;
  private final Action startCardBoxQuizAction;
  
  private final JCheckBox checkBoxCardBox;
  private final DiagramPanel diagramPanel;
  private final JPanel diagramContainer;
  private final CardLayout cardLayout;
  private final JButton questionMark;

  private static final Logger logger = LoggerFactory.getLogger(CardBoxPanel.class); 

  public CardBoxPanel(final TrainerModel model, final TrainerFrame mainframe, final Action startCardBoxQuizAction) {
    this.model = model;
    this.mainframe = mainframe;
    this.startCardBoxQuizAction = startCardBoxQuizAction;
    
    this.model.getSettings().addSettingChangedListener(new SettingsChangedListener() {
      @Override
      public void settingChanged(final String key) {
        if (key.equals(TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE)) {
          setCheckbox();
        }
        if (key.equals(TrainerSettingsConstants.KEY_USE_UNANSWERED_FOR_CARD_BOX)) {
          repaint();
        }
      }
    });
    
    model.getQuestionAttributes().addCardBoxChangedListener(new CardBoxChangedListener() {
      @Override
      public void cardBoxLevelChanged() {
        diagramContainer.repaint();
      }
    });
    setOpaque(false);
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    final String labelCheckbox = TrainerTextResources.getString("CARD_BOX_CHECKBOX_ACTIVATE");
    checkBoxCardBox = new JCheckBox(labelCheckbox);;
    checkBoxCardBox.setOpaque(false);
    checkBoxCardBox.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
    checkBoxCardBox.setFocusPainted(false);
    checkBoxCardBox.setFont(getFont());
    checkBoxCardBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        final boolean checked = checkBoxCardBox.isSelected();
        model.getSettings().setBooleanValue(TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE, checked);
        if (checked) {
          showDiagram();
        } else {
          hideDiagram();
        }
      }
    });
    questionMark = new JButton();

    questionMark.setPreferredSize(new Dimension(20, 20));
    questionMark.setText("?");
    questionMark.setFont(getFont());
    
    questionMark.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        showCardBoxHelp();
      }
    });
    final JPanel checkBoxWithQuestionMark = new JPanel();
    checkBoxWithQuestionMark.setLayout(new BoxLayout(checkBoxWithQuestionMark, BoxLayout.X_AXIS));
    checkBoxWithQuestionMark.setOpaque(false);
    checkBoxWithQuestionMark.add(checkBoxCardBox);
    checkBoxWithQuestionMark.add(questionMark);
    
    
    this.add(checkBoxWithQuestionMark);
    checkBoxWithQuestionMark.setAlignmentX(LEFT_ALIGNMENT);
    
    cardLayout = new CardLayout();
    diagramContainer = new JPanel(cardLayout);
    diagramContainer.setOpaque(false);
    diagramPanel = new DiagramPanel(model);
    diagramContainer.add(diagramPanel, "show");
    final JPanel emptyPanel = new JPanel();
    emptyPanel.setOpaque(false);
    diagramContainer.add(emptyPanel, "hide");
    this.add(diagramContainer);
    
    setCheckbox();
  }
  
  public void setFont(Font font) {
    super.setFont(font);
    if (diagramPanel != null) {
      diagramPanel.setFont(font);
    }
    if (checkBoxCardBox != null) {
      checkBoxCardBox.setFont(getFont());
    }
    if (questionMark != null) {
      questionMark.setFont(font);
    }
  }
  
  private void setCheckbox() {
    final boolean cardBoxActive = this.model.getSettings()
        .getBooleanValue(TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE);
    checkBoxCardBox.setSelected(cardBoxActive);
    if (cardBoxActive) {
      showDiagram();
    } else {
      hideDiagram();
    }
  }
  
  private void showDiagram() {
    cardLayout.show(diagramContainer, "show");
  }

  private void hideDiagram() {
    cardLayout.show(diagramContainer, "hide");
  }

  private class DiagramPanel extends JPanel implements MouseListener {
    
    private TrainerModel model; 
    
    int maxCardBoxLevel = 0;
    private int[] boxStart = new int[0]; 
    private int[] boxEnd = new int[0];
    
    private final int m_activeCardBoxLevel;

    public DiagramPanel(final TrainerModel model) {
      this.model = model;
      final KreawiQuiz selectedQuestions = model.getSelectedQuestions();
      if (selectedQuestions != null) {
        this.m_activeCardBoxLevel = selectedQuestions.cardBoxLevel();
      } else {
        this.m_activeCardBoxLevel = -1;
      }
      setOpaque(false);
      addMouseListener(this);
    }
    
    public void paintComponent(final Graphics g) {
      super.paintComponent(g);
      if (getHeight() < 60) {
        return;
      }
      // Hinweis: maxLevel wird als Index gespeichert. (echterWert-1)
      maxCardBoxLevel = this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL);
      final int numCardBoxes = maxCardBoxLevel + 1;
      final int[] numQuestionForCardBox = new int[numCardBoxes];
      final int[] maxQuestionForCardBox = new int[numCardBoxes];
      int maxQuestionsTotal = 0;
      for (int cardBoxLevel=0; cardBoxLevel<numCardBoxes; cardBoxLevel++) {
        numQuestionForCardBox[cardBoxLevel] = model.getQuestionsForCardBoxLevel(cardBoxLevel).getQuestionCount();
        if (cardBoxLevel == maxCardBoxLevel) {
          // Archiv wird mit gleicher Größe wie Fach 2 angezeigt
          maxQuestionForCardBox[cardBoxLevel] = KreawiGlobals.getCardBoxWarnLimit(1);
        } else {
          maxQuestionForCardBox[cardBoxLevel] = KreawiGlobals.getCardBoxWarnLimit(cardBoxLevel);  
        }
        maxQuestionsTotal += maxQuestionForCardBox[cardBoxLevel];
      }
      Graphics2D graphics2D = (Graphics2D)g;
      graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      graphics2D.setColor(Color.BLACK);
      graphics2D.setFont(getFont());
      final int padding = 10;
      final int height = getHeight() - 2*padding - 1;
      int top3dOffset = (int) ((double) height / (double) 3);
      final double ratio3d = 3.0d/4.0d;
      int left3dOffset = (int) ((double) top3dOffset * ratio3d);
      final int totalWidth = getWidth() - 2 * padding - left3dOffset;
      final double averageWidth = ((double) totalWidth) / ((double) numCardBoxes);
      int left = padding;
      final int top = padding;
      final int startGray = 215;
      final int endGray = 40;
      boxStart = new int[numCardBoxes];
      boxEnd = new int[numCardBoxes];
      for (int i=0; i<numCardBoxes; i++) {
        boxStart[i] = left;
        final int boxWidth = (int) (averageWidth / 3d + ((double) totalWidth) * (2d/3d) * (double) maxQuestionForCardBox[i]
            / (double) maxQuestionsTotal);
        final int fillWidth;
        final int gray;
        if (numQuestionForCardBox[i] >= maxQuestionForCardBox[i]) {
          gray = endGray; 
          fillWidth = boxWidth;
        } else {
          gray = startGray - (int) ((double) (startGray - endGray) * (double) numQuestionForCardBox[i] / (double) maxQuestionForCardBox[i]);
          fillWidth = (int) ((double) boxWidth * (double) numQuestionForCardBox[i] / (double) maxQuestionForCardBox[i]);
        }
        final Color fillColor;
        if (i == m_activeCardBoxLevel) {
          fillColor = new Color(49, 102, 153);
        } else {
          fillColor = new Color(gray, gray, gray);
        }
        final int top_front = top + top3dOffset;
        int height_front = height - top3dOffset;
        graphics2D.drawRect(left, top_front, boxWidth - 1, height_front - 1);
        graphics2D.setColor(fillColor);
        graphics2D.fillRect(left + 1, top_front + 1, boxWidth - 2, height_front - 2);
        graphics2D.setColor(Color.BLACK);
        
        final int topsideBottomLeftX = left;
        final int topsideBottomLeftY = top_front;
        final int topsideTopLeftX = left + left3dOffset;
        final int topsideTopLeftY = top;
        final int topsideTopRightX = left + left3dOffset + boxWidth - 1;
        final int topsideTopRightY = top;
        final int topsideBottomRightX = left + boxWidth - 1;
        final int topsideBottomRightY = top_front;
        if (i==0) {
          graphics2D.drawLine(topsideBottomLeftX, topsideBottomLeftY, topsideTopLeftX, topsideTopLeftY);
        }
        graphics2D.drawLine(topsideTopLeftX, topsideTopLeftY, topsideTopRightX, topsideTopRightY);
        graphics2D.drawLine(topsideTopRightX, topsideTopRightY, topsideBottomRightX, topsideBottomRightY );
        Polygon p = new Polygon(new int[] { topsideBottomLeftX + 1, topsideTopLeftX + 1, topsideTopRightX - 1, topsideBottomRightX},
            new int[] { topsideBottomLeftY, topsideTopLeftY + 1, topsideTopRightY + 1, topsideBottomRightY }, 4);
        graphics2D.setColor(Color.LIGHT_GRAY);
        graphics2D.fillPolygon(p);
        graphics2D.setColor(Color.BLACK);
        if (fillWidth > 1) {
          final int yCorrection = 4;
          final int xCorrection = (int) ((double) yCorrection * ratio3d);
          for (int offset = 3; offset < fillWidth; offset = offset+3) {
            final int startX = left + offset + xCorrection;
            final int startY = top_front - yCorrection;
            final int endX = left + left3dOffset + offset - xCorrection;
            final int endY = top + yCorrection;
            graphics2D.drawLine(startX, startY, endX, endY);
          }
        }
        
        final int cardBoxNumber = i + 1;
        final boolean isArchive = (i == maxCardBoxLevel);
        final int numQuestions = numQuestionForCardBox[i];
        drawLabel(graphics2D, left, top_front, boxWidth, height_front, fillColor, cardBoxNumber, numQuestions,
            isArchive);
        left += boxWidth - 1;
        boxEnd[i] = left;
      }
      final int top_front = top + top3dOffset;
      graphics2D.drawLine(left, top + height - 1, left + left3dOffset, top + height - 1 - top3dOffset);
      graphics2D.drawLine(left + left3dOffset, top + height - 1 - top3dOffset, left + left3dOffset, top);
      graphics2D.setColor(new Color(startGray, startGray, startGray));
      graphics2D.fillPolygon(new int[] { left + 1, left + left3dOffset, left + left3dOffset, left + 1}, new int[] {
          top + height - 2, top + height - top3dOffset - 1, top + 1, top_front + 1}, 4);
    }

    private void drawLabel(final Graphics2D graphics2d, final int left, final int top, final int boxWidth,
        final int height, final Color fillColor, final int cardBoxNumber, final int numQuestions,
        final boolean isArchive) {
      if (fillColor.getRed() < 100) {
        graphics2d.setColor(Color.WHITE);
      } else {
        graphics2d.setColor(Color.BLACK);
      }
      final String compartmentArchive = TrainerTextResources.getString("CARD_BOX_COMPARTMENT_ARCHIVE_LABEL");
      final String compartmentNumber = TrainerTextResources.getStringWithSubstitution("CARD_BOX_COMPARTMENT_NUMBER", "" + cardBoxNumber);
      final String labelLong1 = (isArchive?compartmentArchive:compartmentNumber) + ":";
      final String labelLong2 = "" + numQuestions;
      final FontMetrics fontMetrics = graphics2d.getFontMetrics();
      final int lineHeight = fontMetrics.getAscent();
      if (fontMetrics.stringWidth(labelLong1) > boxWidth || fontMetrics.stringWidth(labelLong2) > boxWidth) {
        final String compartmentArchiveShort = TrainerTextResources.getString("CARD_BOX_COMPARTMENT_ARCHIVE_SHORT");
        final String labelShort1 = (isArchive?compartmentArchiveShort:"#" + cardBoxNumber) + ":";
        final String labelShort2 = "" + numQuestions;
        final int topText1 = top - 1 + (int) ((double) height / 2.0d);
        final int leftText1 = left + 1 + (int) ((double) boxWidth / 2.0d - (double) fontMetrics.stringWidth(labelShort1) / 2.0d);
        final int topText2 = top - 1 + (int) ((double) height / 2.0d + (double) lineHeight);
        final int leftText2 = left + 1 + (int) ((double) boxWidth / 2.0d - (double) fontMetrics.stringWidth(labelShort2) / 2.0d);
        graphics2d.drawString(labelShort1, leftText1, topText1);
        graphics2d.drawString(labelShort2, leftText2, topText2);
      } else {
        final int topText1 = top - 1 + (int) ((double) height / 2.0d);
        final int leftText1 = left + 1 + (int) ((double) boxWidth / 2.0d - (double) fontMetrics.stringWidth(labelLong1) / 2.0d);
        final int topText2 = top - 1 + (int) ((double) height / 2.0d + (double) lineHeight);
        final int leftText2 = left + 1 + (int) ((double) boxWidth / 2.0d - (double) fontMetrics.stringWidth(labelLong2) / 2.0d);
        graphics2d.drawString(labelLong1, leftText1, topText1);
        graphics2d.drawString(labelLong2, leftText2, topText2);
      }
      
      graphics2d.setColor(Color.BLACK);
    }

    public void mouseClicked(MouseEvent e) {
      final int x = e.getPoint().x;
      for (int i=0; i<boxStart.length; i++) {
        if (x >= boxStart[i] && x <= boxEnd[i]) {
          startCardBoxQuizAction.putValue(TrainerFrame.ACTION_CARD_BOX_LEVEL_KEY, new Integer(i));
          ActionEvent event = new ActionEvent(e.getSource(), e.getID(), "Mausklick");
          startCardBoxQuizAction.actionPerformed(event);
          return;
        }
      }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

  }
  
  private void showCardBoxHelp() {
    final String helpItemText = model.getHelpRegistry().getHelpItemText(
        TrainerSettingsConstants.KEY_SHOW_CARD_BOX_FILTER_EXPLANATION_MESSAGE);
    JScrollPane helpText = TrainerHtmlUtil.createHTMLMessageScrollPane(helpItemText);
    helpText.setOpaque(false);
    helpText.setBorder(BorderFactory.createEmptyBorder());
    final String title = model.getHelpRegistry().getHelpItemTitle(
        TrainerSettingsConstants.KEY_SHOW_CARD_BOX_FILTER_EXPLANATION_MESSAGE);
    mainframe.showMessageDialog(title, helpText, JOptionPane.INFORMATION_MESSAGE);
  }

}
