package de.kreawi.trainer.dialogs.quiz;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.datatransfer.FlavorEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.URL;
import java.text.NumberFormat;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.ButtonBarBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.Question;
import com.jquizengine.quiz.Response;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.db.QuestionAttributesModel.IcuChangedListener;
import de.kreawi.trainer.db.QuestionAttributesModel.NoteChangedListener;
import de.kreawi.trainer.db.TrainerSettingsModel.SettingsChangedListener;
import de.kreawi.trainer.dialogs.KreawiDialogPanel;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.quiz.KreawiQuizHelper;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerHyperlinkListener;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.swing.JAntialiasedTextButton;
import de.martinknauer.util.HtmlUtil;
import de.martinknauer.util.NoSelectionCaret;
import de.martinknauer.util.Util;
import de.martinknauer.util.text.CopyAction;
import de.martinknauer.util.text.CutAction;
import de.martinknauer.util.text.PasteAction;
import de.martinknauer.util.undo.UndoManager;

@SuppressWarnings("serial")
public class QuizDialog extends KreawiDialogPanel implements SettingsChangedListener {

	private Map<Actions, Action> actionMap;

	private JTextPane questionPane;

	private KreawiQuiz selectedQuestions;

	private JTextPane commentedAnswerPane;

	private KreawiInternalFrame questionInternalFrame;

	private KreawiInternalFrame answersInternalFrame;

	private MultipleChoicesPanel answersPanel;

	private AbstractAction answerAction;

	private AbstractAction nextUnansweredAction;
	
	private AbstractAction previousUnansweredAction;

	private AbstractAction previousAction;
	
	private AbstractAction nextAction;

	private TrainerModel model;

	private QuizTitleClock clock;

	protected boolean showTime;

	private Timer timeLimitChecker;

	private TrainerFrame mainframe;

	private JCheckBox icuCheckBox;

	private JTextArea notesArea;

	private UndoManager notesUndo;

	private TrainerHyperlinkListener hyperLinkListener;

	private JScrollPane commentedAnswerScrollPane;

	private JScrollPane questionScrollPane;
	
	private Set<Runnable> configChangeRunnables;
	
	private JLabel questionIdLabel;

	private static final Logger logger = LoggerFactory.getLogger(QuizDialog.class); 

	public QuizDialog(TrainerFrame mainframe, TrainerModel model,
			Map<Actions, Action> actionMap) {

		this.mainframe = mainframe;
		this.model = model;
		this.selectedQuestions = this.model.getSelectedQuestions();
		this.actionMap = actionMap;
		
		this.selectedQuestions.addObserver(new Observer() {
			public void update(Observable o, Object arg) {
				if (selectedQuestions.getCurrentResponse().isAnswered(false)) {
					revealCurrentAnswer();
				}
			}
		});
		
		this.configChangeRunnables = new HashSet<Runnable>();
		this.model.getSettings().addSettingChangedListener(this);
		this.model.getQuestionAttributes().addNoteChangedListener(new NoteChangedListener() {
      @Override
      public void noteChanged() {
        logger.info("----------------------------------Notes changed!!!");
        final Question currentQuestion = QuizDialog.this.selectedQuestions.getCurrentQuestion();
        final String note = QuizDialog.this.model.getNote(currentQuestion);
        final String currentText = QuizDialog.this.notesArea.getText();
        if ((note == null && currentText != null) || (note != null && !note.equals(currentText))) {
          SwingUtilities.invokeLater(() -> {
            QuizDialog.this.notesArea.setText(note);
          });
        }
      }
    });
    this.model.getQuestionAttributes().addIcuChangedListener(new IcuChangedListener() {
      @Override
      public void icuChanged() {
        logger.info("----------------------------------ICU changed!!!");
        final Question currentQuestion = QuizDialog.this.selectedQuestions.getCurrentQuestion();
        final boolean icu = QuizDialog.this.model.isInIntensiveCare(currentQuestion);
        if (QuizDialog.this.icuCheckBox.isSelected() == icu) {
          SwingUtilities.invokeLater(() -> {
            QuizDialog.this.icuCheckBox.setSelected(icu);
          });
        }
      }
    });

		initHyperLinkListener();
		
		setLayout(new GridBagLayout());

		if (model.getSelectedQuestions().isCardBoxSelection()) {
		  setOpaque(true);
		  setBackground(new Color(49, 102, 153));
		} else {
		  setOpaque(false);
		}

		/*
		 * BEGIN registering the action which is used to jump directly to a
		 * question
		 */
		Action newQuestionSelected = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				displayCurrentQuestion();
			}
		};
		newQuestionSelected.putValue(Action.NAME, TrainerTextResources
				.getString("QUIZ_BUTTON_NEXT_QUESTION"));
		this.actionMap.put(TrainerFrame.Actions.NEW_QUESTION_SELECTED,
				newQuestionSelected);
		/* END */

		this.actionMap.get(Actions.JUSTIFY_TEXT).addPropertyChangeListener(
				new PropertyChangeListener() {

					public void propertyChange(PropertyChangeEvent evt) {
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								refreshTextAlignment();
							}
						});
					}

				});

		JPanel panel = new JPanel(new BorderLayout());
		panel.setOpaque(false);

		panel.setBorder(Borders.DIALOG_BORDER);

		if (this.selectedQuestions.containsOnlyTextQuestions()) {
			panel.add(buildStaticVerticalSplit(buildHorizontalSplit(
					buildVerticalSplit(buildQuestionPanel(),
							buildNotesPanel(150), 0.9f), buildVerticalSplit(
							buildCommentedAnswerPanel(), buildEvaluationPanel(
									null, 150, true, false), 0.9f), 0.5f),
					buildButtonsPanel(), 95));

			buildAnswersPanel(); // avoid null pointer

		} else {
			

			JPanel evalPanel = null;

			if (this.selectedQuestions.isExamMode()) {
				evalPanel = buildEvaluationNotesPanel(260, true);
			} else {
				evalPanel = buildEvaluationNotesPanel(260, false);
			}
			
			if (!this.model.getConfig().getBool("useAltQuizSplitDisposition")) {
				panel.add(buildStaticVerticalSplit(buildHorizontalSplit(
						buildVerticalSplit(buildQuestionPanel(),
								buildAnswersPanel(), 0.9f), buildVerticalSplit(
								buildCommentedAnswerPanel(),
								evalPanel, 0.9f), 0.5f),
						buildButtonsPanel(), 93, 1));
			} else {
				panel.add(buildStaticVerticalSplit(buildVerticalSplit(
						buildHorizontalSplit(buildQuestionPanel(),
								buildCommentedAnswerPanel(), 0.5f),
						buildHorizontalSplit(buildAnswersPanel(),
								evalPanel, 0.5f), 0.9f),
						buildButtonsPanel(), 93, 1));
			}
		}

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		panel.setMaximumSize(TrainerFrame.FRAME_SIZE);
		add(panel, gbc);

		displayCurrentQuestion();
		
		deployQuizTimeLimitChecker();

	}

	private void deployQuizTimeLimitChecker() {
		
		this.timeLimitChecker = new Timer(1000, null);
		
		final long questionLimit;
		if (this.selectedQuestions.containsOnlyTextQuestions()) {
			questionLimit = this.model.getConfig().getLong("questionTimeLimitText");
		} else {
			questionLimit = this.model.getConfig().getLong("questionTimeLimit");
		}
		
		timeLimitChecker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (selectedQuestions.isQuizTimeLimitExceeded(questionLimit)) {

					timeLimitChecker.stop();

					if (selectedQuestions.isExamMode()) {
						commitTentativeAnswers();
					}

					String message;
					String title = TrainerTextResources
							.getString("QUIZ_TIME_LIMIT_EXCEEDED_TITLE");

					if (selectedQuestions.isExamMode()) {
						message = TrainerTextResources
								.getString("QUIZ_TIME_LIMIT_EXCEEDED_EXAM_MESSAGE");
					} else {
						message = TrainerTextResources
								.getString("QUIZ_TIME_LIMIT_EXCEEDED_MESSAGE");
					}

					mainframe.showMessageDialog(title, message, JOptionPane.WARNING_MESSAGE);

					if (selectedQuestions.isExamMode()) {
						actionMap.get(Actions.TO_EVALUATION).actionPerformed(e);
					}

				}
			}
		});
		timeLimitChecker.setInitialDelay((int) this.selectedQuestions.getQuizTimeLimit(questionLimit) - 9800);

		timeLimitChecker.start();
	}
	
	private KreawiInternalFrame buildQuestionPanel() {
		
		JToolBar toolBar = new JToolBar();
		this.icuCheckBox = new JCheckBox();
		this.icuCheckBox.setBorder(BorderFactory.createEmptyBorder(0, 4, 1, 4));
		this.icuCheckBox.setOpaque(false);
		this.icuCheckBox.setToolTipText(TrainerTextResources
				.getString("QUIZ_CHECKBOX_INTENSIVE_CARE_EXPLANATION"));
		this.icuCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Question question = model.getSelectedQuestions()
						.getCurrentQuestion();
				if (model.isInIntensiveCare(question)) {
					model.removeQuestionFromIcu(question);
				} else {
					model.addQuestionToIcu(question);
					showIcuExplanationMessage();
				}
			}
		});
		this.icuCheckBox.setFocusPainted(false);
		
		toolBar.add(new JLabel(TrainerTextResources
				.getString("QUIZ_CHECKBOX_INTENSIVE_CARE")));
		toolBar.add(this.icuCheckBox);
		toolBar.setToolTipText(TrainerTextResources
				.getString("QUIZ_CHECKBOX_INTENSIVE_CARE_EXPLANATION"));
		toolBar.setOpaque(false);
		toolBar.setBorder(BorderFactory.createEmptyBorder());
		toolBar.setFloatable(false);

		this.questionInternalFrame = new KreawiInternalFrame(TrainerTextResources
				.getString("QUIZ_TITLE_QUESTION"), new BorderLayout(), toolBar);

		this.questionInternalFrame.setPreferredSize(new Dimension(300, 150));

		JPanel gradientPanel = this.questionInternalFrame.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		this.questionPane = new JTextPane();
		this.questionPane.setOpaque(false);
		this.questionPane.setEditable(false);
		if (!KreawiGlobals.isDevMode()) {
			this.questionPane.setCaret(new NoSelectionCaret());
		}
		this.questionPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 2,
				10));
		this.questionPane.setContentType("text/html");
		
		this.questionPane.addHyperlinkListener(this.hyperLinkListener);

		refreshProgress();

		this.questionScrollPane = Factory
				.createStrippedScrollPane(this.questionPane);
		questionScrollPane.setOpaque(false);
		questionScrollPane.getViewport().setOpaque(false);

		gradientPanel.add(questionScrollPane);

		return this.questionInternalFrame;
	}
	
	private void initHyperLinkListener() {
		this.hyperLinkListener = new TrainerHyperlinkListener(mainframe);
		configureHyperLinkListener();
	}

	private void configureHyperLinkListener() {
    hyperLinkListener.setImageViewerEnabled(
        !this.model.getSettings().getBooleanValue(TrainerSettingsConstants.KEY_USE_EXTERNAL_IMAGE_VIEWER));
    hyperLinkListener.setAudioViewerEnabled(
        !this.model.getSettings().getBooleanValue(TrainerSettingsConstants.KEY_USE_EXTERNAL_AUDIO_PLAYER));
	}

	private void showIcuExplanationMessage() {

		this.model.getHelpRegistry().showHelpMessage(this.mainframe,
				TrainerSettingsConstants.KEY_SHOW_ICU_EXPLANATION_MESSAGE);

	}

	private KreawiInternalFrame buildCommentedAnswerPanel() {

		questionIdLabel = new JLabel();
		questionIdLabel.setFont(getFont().deriveFont(10.0f));
		
		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("QUIZ_TITLE_COMMENTED_ANSWER"), new BorderLayout(), questionIdLabel);

		kif.setPreferredSize(new Dimension(300, 150));

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		this.commentedAnswerPane = new JTextPane() {
			public void paint(Graphics g) {
				Graphics2D g2d = (Graphics2D) g;
				if (selectedQuestions.isExamMode()
						&& !selectedQuestions.getCurrentResponse()
								.isAnswered()) {
					g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
							RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
				}
				super.paint(g2d);
			}
		};
		this.commentedAnswerPane.setOpaque(false);
		this.commentedAnswerPane.setEditable(false);
		if (!KreawiGlobals.isDevMode()) {
			this.commentedAnswerPane.setCaret(new NoSelectionCaret());
		}
		this.commentedAnswerPane.setBorder(BorderFactory.createEmptyBorder(10,
				10, 2, 10));
		this.commentedAnswerPane.setContentType("text/html");
		this.commentedAnswerPane
				.addHyperlinkListener(this.hyperLinkListener);
		
		this.commentedAnswerScrollPane = Factory
				.createStrippedScrollPane(this.commentedAnswerPane);
		commentedAnswerScrollPane.setOpaque(false);
		commentedAnswerScrollPane.getViewport().setOpaque(false);

		gradientPanel.add(commentedAnswerScrollPane);

		return kif;
	}

	private KreawiInternalFrame buildAnswersPanel() {

		this.answersInternalFrame = new KreawiInternalFrame(TrainerTextResources
				.getString("QUIZ_TITLE_ANSWERS"), new GridLayout(1, 1));

		if (this.selectedQuestions.containsOnlyTextQuestions()) {
			this.answersInternalFrame = new KreawiInternalFrame(null,
					new GridLayout(1, 1));
		}

		this.answersInternalFrame.setPreferredSize(new Dimension(300, 260));

		this.answersInternalFrame.getContent().setBorder(
				BorderFactory.createEmptyBorder());

		return this.answersInternalFrame;
	}
	
	private JPanel buildEvaluationNotesPanel(int height, boolean textQuestionMode) {
		final CardLayout cards = new CardLayout();
		final JPanel ret = new JPanel(cards);
		ret.setPreferredSize(new Dimension(300, height));
		ret.setBorder(BorderFactory.createEmptyBorder());

		String icon = KreawiGlobals.APP_IMAGES_DIR_NAME + "/down12_grey.gif";
		String iconRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/down12_grey_light.gif";
		String iconPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/down12_grey_dark.gif";

		URL url = KreawiGlobals.class.getResource(icon);
		URL urlRollover = KreawiGlobals.class.getResource(iconRollover);
		URL urlPressed = KreawiGlobals.class.getResource(iconPressed);

		ActionListener listener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cards.show(ret, "" + true);
				showNotesExplanationMessage();
			}
		};

		String text = TrainerTextResources.getString("QUIZ_SHOW_NOTES");
		String toolTip = TrainerTextResources.getString("QUIZ_SHOW_NOTES_TOOL_TIP");

		ret.add(buildEvaluationPanel(KreawiInternalFrame
				.createLabeledIconButtonToolBar(new ImageIcon(url),
						new ImageIcon(urlRollover), new ImageIcon(urlPressed),
						text, toolTip, listener), height, textQuestionMode,
				false), "" + false);

		icon = KreawiGlobals.APP_IMAGES_DIR_NAME + "/up12_grey.gif";
		iconRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/up12_grey_light.gif";
		iconPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/up12_grey_dark.gif";

		url = KreawiGlobals.class.getResource(icon);
		urlRollover = KreawiGlobals.class.getResource(iconRollover);
		urlPressed = KreawiGlobals.class.getResource(iconPressed);

		listener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cards.show(ret, "" + false);
			}
		};

		text = TrainerTextResources.getString("QUIZ_HIDE_NOTES");
		toolTip = TrainerTextResources.getString("QUIZ_HIDE_NOTES_TOOL_TIP");

		ret.add(buildStaticVerticalSplit(buildEvaluationPanel(
				KreawiInternalFrame.createLabeledIconButtonToolBar(
						new ImageIcon(url), new ImageIcon(urlRollover),
						new ImageIcon(urlPressed), text, toolTip, listener),
				(height - 10) / 2, textQuestionMode, true),
				buildNotesPanel((height - 10) / 2)), "" + true);
		return ret;
	}
	
	private void showNotesExplanationMessage() {

		this.model.getHelpRegistry().showHelpMessage(this.mainframe,
				TrainerSettingsConstants.KEY_SHOW_NOTES_EXPLANATION_MESSAGE);

	}

  private KreawiInternalFrame buildEvaluationPanel(final JToolBar toolBar, int height, final boolean textQuestionMode,
      final boolean shortVersion) {
		
		final KreawiInternalFrame kif = new KreawiInternalFrame("",
				new BorderLayout(), toolBar);

		kif.setPreferredSize(new Dimension(300, height));

		if (this.selectedQuestions.isExamMode()) {
			toolBar.setEnabled(false);
			this.selectedQuestions.addObserver(new Observer() {
				public void update(Observable o, Object arg) {
					if (!selectedQuestions.hasUnanswered(false)) {
						toolBar.setEnabled(true);
					}
				}
			});
		}
		
		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		JPanel evaluationPanel = new JPanel();
		evaluationPanel.setLayout(new BoxLayout(evaluationPanel, BoxLayout.Y_AXIS));
		evaluationPanel.setOpaque(false);
		
		final JLabel evaluationLabel = new JLabel(generateEvaluationHtml(textQuestionMode, true));
    evaluationLabel.setFont(getEvaluationFont());
    evaluationLabel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		evaluationPanel.add(evaluationLabel);
		evaluationLabel.setHorizontalAlignment(SwingConstants.CENTER);
		
		final CardBoxPanel cardBoxDiagramPanel;
		if (!shortVersion && !textQuestionMode) {
      cardBoxDiagramPanel = new CardBoxPanel(model, mainframe, actionMap.get(Actions.START_CARD_BOX_QUIZ));
      cardBoxDiagramPanel.setFont(getEvaluationFont());
      evaluationPanel.add(cardBoxDiagramPanel);
    } else {
      cardBoxDiagramPanel = null;
    }
		
		this.selectedQuestions.addObserver(new Observer() {
			public void update(Observable o, Object arg) {
				String text = generateEvaluationHtml(textQuestionMode,
						true);
				evaluationLabel.setText(text);
				if (cardBoxDiagramPanel != null) {
				  cardBoxDiagramPanel.repaint();
				}
			}
		});

		this.configChangeRunnables.add(new Runnable() {
			public void run() {
				evaluationLabel.setFont(getEvaluationFont());
        if (cardBoxDiagramPanel != null) {
          cardBoxDiagramPanel.setFont(getEvaluationFont());
          cardBoxDiagramPanel.repaint();
        }
			}
		});

		JScrollPane scrollPane = Factory
				.createStrippedScrollPane(evaluationPanel);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

		gradientPanel.add(scrollPane);

		final long questionLimit;
		if (this.selectedQuestions.containsOnlyTextQuestions()) {
			questionLimit = this.model.getConfig().getLong("questionTimeLimitText");
		} else {
			questionLimit = this.model.getConfig().getLong("questionTimeLimit");
		}
		
		if (this.clock == null) {
			this.clock = new QuizTitleClock(500, this.selectedQuestions, kif
					.getTitleLabel(), TrainerTextResources
					.getString("QUIZ_TITLE_COMMENTED_EVALUATION")
					+ " - ", "", questionLimit, selectedQuestions.isExamMode());
			this.clock.setInitialDelay(0);

			this.clock.start();
		} else {
			this.clock.addLabel(kif.getTitleLabel(), TrainerTextResources
					.getString("QUIZ_TITLE_COMMENTED_EVALUATION")
					+ " - ", "");
		}
		
		return kif;
	}
	
	private KreawiInternalFrame buildNotesPanel(int height) {
		
		String icon = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2.gif";
		String iconRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_light.gif";
		String iconPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_dark.gif";

		URL url = KreawiGlobals.class.getResource(icon);
		URL urlRollover = KreawiGlobals.class.getResource(iconRollover);
		URL urlPressed = KreawiGlobals.class.getResource(iconPressed);

		ActionListener listener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				notesArea.setText(null);
			}
		};

		String text = TrainerTextResources.getString("QUIZ_ERASE_NOTE");
		String toolTip = TrainerTextResources.getString("QUIZ_ERASE_NOTE_TOOL_TIP");
		
		final JToolBar eraseNoteToolBar = KreawiInternalFrame
				.createLabeledIconButtonToolBar(new ImageIcon(url),
						new ImageIcon(urlRollover), new ImageIcon(urlPressed),
						text, toolTip, listener);
		eraseNoteToolBar.setEnabled(false);
		final KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("QUIZ_TITLE_NOTES"), new BorderLayout(),
				eraseNoteToolBar);

			kif.setPreferredSize(new Dimension(300, height));

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());
		
		this.notesArea = new JTextArea();
		notesArea.setOpaque(false);
		notesArea.setFont(getEvaluationFont());
		notesArea.setLineWrap(true);
		notesArea.setWrapStyleWord(true);
		notesArea.setMargin(new Insets(0, 2, 2, 2));
		notesArea.getDocument().addDocumentListener(new DocumentListener() {

			public void changedUpdate(DocumentEvent e) {
				applyNoteChange();
			}

			public void insertUpdate(DocumentEvent e) {
				applyNoteChange();
			}

			public void removeUpdate(DocumentEvent e) {
				applyNoteChange();
			}
			
			private void applyNoteChange() {
				String note = notesArea.getText();
				if (note != null && note.length() > 0) {
					model.putNote(selectedQuestions.getCurrentQuestion(), note);
					eraseNoteToolBar.setEnabled(true);
				} else {
					model.eraseNote(selectedQuestions.getCurrentQuestion());
					eraseNoteToolBar.setEnabled(false);
				}
			}
			
		});
		
		this.notesUndo = new UndoManager();

		this.notesArea.getDocument().addUndoableEditListener(notesUndo);

		final JPopupMenu popup = new JPopupMenu();
		
		
		InputMap inputMap = this.notesArea.getInputMap();

		Action action = notesUndo.getUndoAction();
		JMenuItem menuItem = new JMenuItem(action);
		KeyStroke key = (KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		inputMap.put(key, action);
		popup.add(menuItem);
		
		action = notesUndo.getRedoAction();
		menuItem = new JMenuItem(action);
		key = (KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		inputMap.put(key, action);
		popup.add(menuItem);
		
		popup.addSeparator();
		
		CutAction cutAction = new CutAction();
		menuItem = new JMenuItem(cutAction);
		key = (KeyStroke) cutAction.getValue(Action.ACCELERATOR_KEY);
		inputMap.put(key, cutAction);
		this.notesArea.addCaretListener(cutAction);
		popup.add(menuItem);

		CopyAction copyAction = new CopyAction();
		menuItem = new JMenuItem(copyAction);
		key = (KeyStroke) copyAction.getValue(Action.ACCELERATOR_KEY);
		inputMap.put(key, copyAction);
		this.notesArea.addCaretListener(copyAction);
		popup.add(menuItem);

		PasteAction pasteAction = new PasteAction();
		menuItem = new JMenuItem(pasteAction);
		key = (KeyStroke) pasteAction.getValue(Action.ACCELERATOR_KEY);
		inputMap.put(key, pasteAction);
		Toolkit.getDefaultToolkit().getSystemClipboard().addFlavorListener(
				pasteAction);
		
		// init the actions enabled state
		pasteAction.flavorsChanged(new FlavorEvent(Toolkit.getDefaultToolkit()
				.getSystemClipboard()));
		
		popup.add(menuItem);

		this.notesArea.addMouseListener(new MouseAdapter() {
			
			public void mousePressed(MouseEvent e) {
				handlePopup(e);
				e.consume();
			}

			public void mouseReleased(MouseEvent e) {
				handlePopup(e);
				e.consume();
			}

			void handlePopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}

		});

		JScrollPane scrollPane = Factory
				.createStrippedScrollPane(this.notesArea);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);

		gradientPanel.add(scrollPane);

		return kif;
	}

	private KreawiInternalFrame buildButtonsPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(null,
				new GridBagLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 11, 10));
		gradientPanel.setLayout(new GridBagLayout());

		/* answer */
		JButton answerButton = new JButton();
		this.answerAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				answerCurrentQuestion();
			}
		};

		String answerActionName = TrainerTextResources
				.getString("QUIZ_BUTTON_ANSWER_QUESTION");

		if (this.selectedQuestions.containsOnlyTextQuestions()
				|| this.selectedQuestions.isExamMode()) {
			answerActionName = TrainerTextResources
					.getString("QUIZ_BUTTON_SOLVE_QUESTION");
		}
		
		if (Util.isMacOS()) {
			answerActionName = answerActionName.trim();
		}

		this.answerAction.putValue(Action.NAME, answerActionName);

		answerButton.setAction(this.answerAction);
		
		/* next unanswered */
		JButton nextUnansweredButton = new JAntialiasedTextButton();
		
		this.nextUnansweredAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				nextUnansweredQuestion();
			}
		};
		this.nextUnansweredAction.putValue(Action.NAME, TrainerTextResources
				.getString("QUIZ_BUTTON_NEXT_UNANSWERED"));
		this.nextUnansweredAction.putValue(Action.SHORT_DESCRIPTION, TrainerTextResources
				.getString("QUIZ_BUTTON_NEXT_UNANSWERED_SHORT_DESC"));
		this.nextUnansweredAction.setEnabled(this.selectedQuestions.hasUnanswered());
		
		nextUnansweredButton.setAction(this.nextUnansweredAction);

		/* previous unanswered */
		JButton previousUnansweredButton = new JAntialiasedTextButton();

		this.previousUnansweredAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				previousUnansweredQuestion();
			}
		};
		this.previousUnansweredAction.putValue(Action.NAME,
				TrainerTextResources.getString("QUIZ_BUTTON_PREVIOUS_UNANSWERED"));
		this.previousUnansweredAction.putValue(Action.SHORT_DESCRIPTION,
				TrainerTextResources
						.getString("QUIZ_BUTTON_PREVIOUS_UNANSWERED_SHORT_DESC"));
		this.previousUnansweredAction.setEnabled(this.selectedQuestions
				.hasUnanswered());

		previousUnansweredButton.setAction(this.previousUnansweredAction);

		/* next */
		JButton nextButton = new JAntialiasedTextButton();
		this.nextAction = new AbstractAction(TrainerTextResources
				.getString("QUIZ_BUTTON_NEXT")) {
			public void actionPerformed(ActionEvent e) {
				next();
			}
		};
		this.nextAction.putValue(Action.SHORT_DESCRIPTION, TrainerTextResources
				.getString("QUIZ_BUTTON_NEXT_SHORT_DESC"));
		this.nextAction.setEnabled(this.selectedQuestions.getQuestionCount() > 1);
		nextButton.setAction(this.nextAction);
		
		nextButton.setPreferredSize(nextUnansweredButton.getPreferredSize());
		
		/* previous */
		JButton previousButton = new JAntialiasedTextButton();
		this.previousAction = new AbstractAction(TrainerTextResources
				.getString("QUIZ_BUTTON_PREVIOUS")) {
			public void actionPerformed(ActionEvent e) {
				back();
			}
		};
		this.previousAction.putValue(Action.SHORT_DESCRIPTION,
				TrainerTextResources
						.getString("QUIZ_BUTTON_PREVIOUS_SHORT_DESC"));
		this.previousAction.setEnabled(this.selectedQuestions.getQuestionCount() > 1);
		previousButton.setAction(this.previousAction);
		
		previousButton.setPreferredSize(previousUnansweredButton.getPreferredSize());

		
		final Action originalAction = this.actionMap.get(Actions.TO_EVALUATION);

		final Action evalAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				if (selectedQuestions.isExamMode()) {
					if (selectedQuestions.hasUnanswered(false)) {
						if (reallyCommitTentativeAnswers()) {
							commitTentativeAnswers();
						} else {
							return;
						}
					}
				} else {
					selectedQuestions.refreshTimeTaken();
				}
				originalAction.actionPerformed(e);
			}
		};
		evalAction.putValue(Action.NAME, originalAction.getValue(Action.NAME));

		JButton evalButton = new JButton(evalAction);

		JButton mainMenuButton = new JButton(this.actionMap
				.get(Actions.BACK_TO_MAIN_DIALOG));

		ButtonBarBuilder builder = new ButtonBarBuilder();
		
		if (builder.isLeftToRightButtonOrder()) {
			builder.addGlue();
			builder.addGridded(evalButton);
			builder.addUnrelatedGap();
			builder.addGlue();
			builder.addFixed(previousUnansweredButton);
			builder.addUnrelatedGap();
			builder.addFixed(previousButton);
			builder.addUnrelatedGap();
//			builder.addGlue();
			builder.addGridded(answerButton);
//			builder.addGlue();
			builder.addUnrelatedGap();
			builder.addFixed(nextButton);
			builder.addUnrelatedGap();
			builder.addFixed(nextUnansweredButton);
			builder.addUnrelatedGap();
			builder.addGlue();
			builder.addGridded(mainMenuButton);
			if (this.selectedQuestions.containsOnlyTextQuestions()) {
				evalButton.setVisible(false);
			}
			builder.addGlue();
		} else {
			builder.addGlue();
			builder.addGridded(mainMenuButton);
			builder.addUnrelatedGap();
			builder.addGlue();
			builder.addFixed(previousUnansweredButton);
			builder.addUnrelatedGap();
			builder.addFixed(previousButton);
			builder.addUnrelatedGap();
//			builder.addGlue();
			builder.addGridded(answerButton);
//			builder.addGlue();
			builder.addUnrelatedGap();
			builder.addFixed(nextButton);
			builder.addUnrelatedGap();
			builder.addFixed(nextUnansweredButton);
			builder.addUnrelatedGap();
			builder.addGlue();
			builder.addGridded(evalButton);
			if (this.selectedQuestions.containsOnlyTextQuestions()) {
				evalButton.setVisible(false);
			}
			builder.addGlue();
		}
		
		JPanel buttonsPanel = builder.getPanel();
		
		buttonsPanel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		gradientPanel.add(buttonsPanel, gbc);

		return kif;
	}

	private void answerCurrentQuestion() {

		this.selectedQuestions.setAnswersForCurrentQuestion(this.answersPanel
				.getSelected());
		
		updateCardBoxLevel(this.selectedQuestions.getCurrentQuestion(),
				this.selectedQuestions.getCurrentResponse());
		
		updateProposeEvaluationFlag();

    logger.info("Answer was " + (this.selectedQuestions.getCurrentResponse().isCorrect() ? "" : "in") + "correct!");
    
		/* The following code is now handled in an observer object! */
//		if (!this.selectedQuestions.hasUnanswered()) {
//			this.clock.stop();
//			if (!this.selectedQuestions.isTextQuestionCollection()) {
//				this.actionMap.get(Actions.BACK_TO_MAIN_DIALOG).putValue(
//						TrainerFrame.ACTION_PROPOSE_EVALUATION_KEY,
//						Boolean.TRUE);
//			}
//		}
//
//		this.answersPanel.setEnabled(false);
//
//		showCurrentAnswer();
//
//		refreshButtonStates();

	}
	
	private void updateProposeEvaluationFlag() {
		if (!this.selectedQuestions.hasUnanswered()) {
			if (!this.selectedQuestions.containsOnlyTextQuestions()) {
				this.actionMap.get(Actions.BACK_TO_MAIN_DIALOG).putValue(
						TrainerFrame.ACTION_PROPOSE_EVALUATION_KEY,
						Boolean.TRUE);
			}
		}
	}
	
	private void updateCardBoxLevel(Question question, Response reply) {
	  
		if (question.getType() == Question.Type.TEXT) {
			/*
			 * As for text questions, there is no wrong or right answer, card
			 * box level must not be changed; however, it must be set, so the
			 * question is counted as an answered question
			 */
      this.model.setCardBoxLevel(question, this.model.getCardBoxLevel(question));
		}
		
		if (reply.isCorrect()) {
			this.model.increaseCardBoxLevel(question);
		} else {
			this.model.setCardBoxLevel(question, 0);
		}

	}
	
	private void revealCurrentAnswer() {
		
		if (!this.selectedQuestions.hasUnanswered()) {
			this.clock.stop();
		}
		
		this.answersPanel.setEnabled(false);

		showCurrentAnswer();

		refreshButtonStates();
	}

	private void showCurrentAnswer() {
		String commentText = this.selectedQuestions.getCurrentQuestion()
				.getComment();
		
		logger.info("Commented answer text: " + commentText);

		setTextPaneText(this.commentedAnswerPane, commentText);
		this.commentedAnswerScrollPane.getVerticalScrollBar().setValue(0);
		this.commentedAnswerScrollPane.getHorizontalScrollBar().setValue(0);
		
		if (KreawiGlobals.isDevMode()) {
		  logger.info(wrapInHtmlAndSetFont(commentText));
		}

//		this.answersPanel.setSelected(this.selectedQuestions
//				.getCurrentQuestionReply().getReplies());
		this.answersPanel.highlightCorrectAnswers(this.selectedQuestions
				.getCurrentQuestion().getCorrectAnswerBits());

		// DocumentRenderer docRenderer = new DocumentRenderer();
		// docRenderer.print((HTMLDocument)this.commentedAnswerPane.getStyledDocument());
	}

	private void clearAnswer() {
		/* clear the text in the commented answer panel */
		clearTextPane(this.commentedAnswerPane);
	}
	
	private boolean reallyCommitTentativeAnswers() {
		String message = TrainerTextResources.getString("QUIZ_HAND_IN_EXAM_MESSAGE");
		String title = TrainerTextResources.getString("QUIZ_HAND_IN_EXAM_TITLE");
    return JOptionPane.showConfirmDialog(this, message, title, JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
	}
	
	private void commitTentativeAnswers() {
		this.selectedQuestions.commitAllTentativeResponses();

		for (Question question : this.selectedQuestions.getQuestions()) {
			updateCardBoxLevel(question, this.selectedQuestions
					.getResponse(question));
		}
		
		updateProposeEvaluationFlag();
	}

	private void nextUnansweredQuestion() {
		switchQuestion(this.selectedQuestions.nextUnanswered());
	}
	
	private void previousUnansweredQuestion() {
		switchQuestion(this.selectedQuestions.previousUnanswered());
	}

	private void next() {

		if (this.selectedQuestions.hasNext()) {
			switchQuestion(this.selectedQuestions.next());
		} else {
			switchQuestion(this.selectedQuestions.first());
		}

	}
	
	private void back() {

		if (this.selectedQuestions.hasPrevious()) {
			switchQuestion(this.selectedQuestions.previous());
		} else {
			switchQuestion(this.selectedQuestions.last());
		}

	}
	
	private void switchQuestion(Question question) {

		clearAnswer();

		displayQuestion(question);

		if (this.selectedQuestions.getCurrentResponse().isAnswered()) {

			showCurrentAnswer();

			this.answersPanel.setEnabled(false);

		} else {

			this.selectedQuestions.getStopwatch()
					.startQuestionTimeTracking();

			if (this.selectedQuestions.isExamMode()) {
				setTextPaneText(this.commentedAnswerPane, TrainerTextResources
						.getString("QUIZ_TEXT_EXAM_MODE_NOTE"));
			}

		}

		refreshProgress();
//		
//		evaluationLabel.setFont(getEvaluationFont());
//		evaluationLabel.revalidate();
//		evaluationLabel.repaint();

		refreshButtonStates();
	}

	private void displayQuestion(final Question question) {

	  logger.info("Displaying question " + question.getId() + ", type "
				+ question.getType());
	  logger.info("Card box level for question " + question.getId() + " is "
				+ this.model.getCardBoxLevel(question));

		questionIdLabel.setText("(ID " + question.getId() + ") ");
		
		String text = attachCombiAnswers(question);

		/* display the question text */
		clearTextPane(this.questionPane);
		setTextPaneText(this.questionPane, text);
		this.questionScrollPane.getVerticalScrollBar().setValue(0);
		this.questionScrollPane.getHorizontalScrollBar().setValue(0);
		
		if (KreawiGlobals.isDevMode()) {
		  logger.info(wrapInHtmlAndSetFont(text));
		}

		this.answersInternalFrame.getContent().removeAll();
		this.answersPanel = new MultipleChoicesPanel(question.getAnswers(),
				question.getType(),
				getAnswersFont(), model.getConfig().getBool(
					"wrapChoiceLines"));
		Response reply = this.selectedQuestions.getResponse(question);

		this.answersPanel.setSelected(reply.getResponse());
		this.answersPanel.addActionListener(new ActionListener() {
			/* listener to update the tentative responses */
			public void actionPerformed(ActionEvent e) {
				selectedQuestions.setResponse(question, answersPanel
						.getSelected(), true);
			}
		});

		JScrollPane scrollPane = Factory
				.createStrippedScrollPane(this.answersPanel);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);

		this.answersInternalFrame.getContent().add(scrollPane);

		this.answersInternalFrame.revalidate();
		this.answersInternalFrame.repaint();

		if(this.model.isInIntensiveCare(question)) {
			this.icuCheckBox.setSelected(true);
		} else {
			this.icuCheckBox.setSelected(false);
		}
		
		this.notesArea.setText(this.model.getNote(question));
		
		this.notesUndo.discardAllEdits();
		
		if (KreawiGlobals.isDevMode()) {
			this.answersPanel.highlightCorrectAnswers(question.getCorrectAnswerBits());
			logger.info(question.getId() + " - \"" + question.getTitle() + "\"");
			logger.info("Correct answer bits: " + question.getCorrectAnswerBits());
		}

	}

	private void displayCurrentQuestion() {

		switchQuestion(this.selectedQuestions.getCurrentQuestion());

	}
	
	private String attachCombiAnswers(Question question) {
		
		String text = question.getText();
		
		if (question.getType() == Question.Type.COMBI) {
			/*
			 * as combi answers are currently not supported, they are just
			 * attached to the text of the questions
			 */

			List<String> combiAnswers = question.getCombiAnswers();

			if (combiAnswers.size() > 0) {

				text += "<br ><br ><ol>";

				for (String currentCombiAnswer : combiAnswers) {
					text += "<li>" + currentCombiAnswer + "</li>";
				}

				text += "</ol>";

			}
		}
		
		return text;
		
	}

	private void refreshButtonStates() {

		this.answerAction.setEnabled(!this.selectedQuestions
				.getCurrentResponse().isAnswered()
				&& !this.selectedQuestions.isExamMode());

		if (this.selectedQuestions.isExamMode()) {
			this.nextUnansweredAction.setEnabled(this.selectedQuestions
					.getUnansweredQuestionCount() > 1);
		} else {
			this.nextUnansweredAction.setEnabled(this.selectedQuestions
					.getCurrentResponse().isAnswered()
					&& this.selectedQuestions.hasUnanswered());
		}

		this.nextUnansweredAction.setEnabled(this.selectedQuestions
				.hasUnanswered());

		this.previousUnansweredAction.setEnabled(this.selectedQuestions
				.hasUnanswered());
	}

	private void clearTextPane(JTextPane textPane) {
		JTextPane dummy = new JTextPane();
		dummy.setContentType("text/html");
		textPane.setStyledDocument(dummy.getStyledDocument());

	}

	private void setTextPaneText(JTextPane textPane, String text) {
		String htmlText = replaceRelativeImagesPaths(wrapInHtmlAndSetFont(text));
		textPane.setText(htmlText);
		textPane.setCaretPosition(0);
	}

	private String replaceRelativeImagesPaths(String text) {
	  final File workDir = new File(System.getProperty("user.dir"));
    final File baseDir = new File(workDir, KreawiGlobals.RESOURCES_DIR_NAME + File.separator);
    
	  return TrainerHtmlUtil.replaceRelativePathOfImages(text, baseDir);
  }

  private String wrapInHtmlAndSetFont(final String text) {
    final boolean justify = this.actionMap.get(Actions.JUSTIFY_TEXT).getValue(TrainerFrame.ACTION_SELECTED_KEY).equals(Boolean.TRUE);
    final String css = TrainerHtmlUtil.getCssWithFont(justify, getQuestionCommentFont());
    return TrainerHtmlUtil.wrapInHtml(HtmlUtil.cleanupHtml(text), css);
	}

	private void refreshProgress() {
    
	  final KreawiQuiz quiz = model.getSelectedQuestions();

    final String title = KreawiQuizHelper.getTitle(quiz) + " - "
      + TrainerTextResources.getString("QUIZ_TITLE_QUESTION") + " " + (this.selectedQuestions.getIndex() + 1) + " "
      + TrainerTextResources.getString("QUIZ_TITLE_QUESTION_OF") + " " + this.selectedQuestions.getQuestionCount();

    this.questionInternalFrame.setTitle(title);
	}

	private void refreshTextAlignment() {
		refreshTextAlignment(this.questionPane, attachCombiAnswers(this.selectedQuestions
				.getCurrentQuestion()));

		if (this.selectedQuestions.getCurrentResponse().isAnswered()) {
			refreshTextAlignment(this.commentedAnswerPane,
					this.selectedQuestions.getCurrentQuestion().getComment());
		}
	}

	private void refreshTextAlignment(JTextPane textPane, String text) {
		clearTextPane(textPane);
		setTextPaneText(textPane, text);
	}

	private String generateEvaluationHtml(boolean textQuestionMode, boolean shortVersion) {

		int all = this.selectedQuestions.getQuestionCount();
		int answered = this.selectedQuestions.getAnsweredQuestionCount();
		int correct = this.selectedQuestions.getCorrectAnswersCount();
		int incorrect = this.selectedQuestions.getIncorrectAnswersCount();

		double percentCorrectAnswered = answered > 0 ? (double) correct
				/ (double) answered : 0d;
		double percentAllAnswered = all > 0 ? (double) correct / (double) all
				: 0;

		String ret = "";

		if (!textQuestionMode) {
			String[] subsitutes = {
					answered + "",
					all + "",
					correct + "",
					incorrect + "",
					(all - answered) + "",
					NumberFormat.getPercentInstance().format(
							percentCorrectAnswered),
					NumberFormat.getPercentInstance()
							.format(percentAllAnswered) };

			ret = TrainerTextResources.getStringWithSubstitution(
					shortVersion ? "QUIZ_EVALUATION_TEXT_FOR_MC_SHORT"
							: "QUIZ_EVALUATION_TEXT_FOR_MC", subsitutes);
		} else {

			long questionLimit;
			if (!shortVersion) {
				questionLimit = this.model.getConfig().getLong(
						"questionTimeLimitText");
			} else {
				questionLimit = this.model.getConfig().getLong(
						"questionTimeLimit");
			}

			String[] subsitutes = {
					answered + "",
					all + "",
					(all - answered) + "",
					this.selectedQuestions.getStopwatch()
							.getOverallTimeTakenMinutesString(),
					this.selectedQuestions.getAnsweredQuestionCount() > 0
							&& !shortVersion ? TrainerTextResources
							.getString(this.selectedQuestions
									.isQuestionsTimeLimitExceeded(
											questionLimit, false) ? "QUIZ_EVALUATION_TEXT_FOR_TEXT_SUPPLEMENT_SLOW"
									: "QUIZ_EVALUATION_TEXT_FOR_TEXT_SUPPLEMENT")
							: shortVersion ? "" : "&nbsp;" };

			ret = TrainerTextResources.getStringWithSubstitution(
					"QUIZ_EVALUATION_TEXT_FOR_TEXT", subsitutes);
		}

		return ret;
	}

	private Font getQuizFont(String fontSizeSettingsKey) {

		int size = this.model.getSettings().getIntValue(fontSizeSettingsKey);

		size = size < 1 ? model.getConfig().getInt("questionDialogFontSize")
				: size;

    return new Font(model.getConfig().getString("questionDialogFont"), Font.PLAIN, size);
	}

	private Font getQuestionCommentFont() {
    return getQuizFont(TrainerSettingsConstants.KEY_QUESTION_COMMENT_FONT_SIZE);
	}

	private Font getEvaluationFont() {
		return getQuizFont(TrainerSettingsConstants.KEY_EVALUATION_FONT_SIZE);
	}

	private Font getAnswersFont() {
		return getQuizFont(TrainerSettingsConstants.KEY_ANSWERS_FONT_SIZE);
	}

	public void closeDialog() {
		this.clock.stop();
		this.clock = null;

		this.timeLimitChecker.stop();
		this.timeLimitChecker = null;

		this.model.getSettings().removeSettingChangedListener(this);

	}

  @Override
  public void settingChanged(String key) {
    if (key.equals(
        TrainerSettingsConstants.KEY_USE_EXTERNAL_IMAGE_VIEWER)
        || key.equals(
            TrainerSettingsConstants.KEY_USE_EXTERNAL_AUDIO_PLAYER)) {
      configureHyperLinkListener();
    } else if (key.equals(
        TrainerSettingsConstants.KEY_QUESTION_COMMENT_FONT_SIZE)
        || key.equals(TrainerSettingsConstants.KEY_ANSWERS_FONT_SIZE)
        || key.equals(TrainerSettingsConstants.KEY_EVALUATION_FONT_SIZE)
        || key.equals(TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE)) {
      switchQuestion(selectedQuestions.getCurrentQuestion());
      for (Runnable run : configChangeRunnables) {
        SwingUtilities.invokeLater(run);
      }
    }
  }

}
