package de.kreawi.trainer.dialogs.selection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.ButtonBarBuilder;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.Category;
import com.jquizengine.questions.Exam;
import com.jquizengine.questions.Question;
import com.jquizengine.questions.collection.QuestionPool;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.db.QuestionAttributesModel;
import de.kreawi.trainer.db.QuestionAttributesModel.CardBoxChangedListener;
import de.kreawi.trainer.db.QuestionAttributesModel.IcuChangedListener;
import de.kreawi.trainer.db.TrainerSettingsModel.SettingsChangedListener;
import de.kreawi.trainer.dialogs.KreawiDialogPanel;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.dialogs.quiz.CardBoxPanel;
import de.kreawi.trainer.exceptions.TrainerStorageException;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.util.TrainerTextResources;
import de.kreawi.trainer.util.TrainerUtil;
import de.martinknauer.awt.ColorUtil;
import de.martinknauer.util.Util;
import de.martinknauer.util.config.Config;

@SuppressWarnings("serial")
public abstract class AbstractSelectionDialog extends KreawiDialogPanel {

  private TrainerFrame mainframe;
  private TrainerModel model;
  private Map<Actions, Action> actionMap;
  private final QuestionPool allQuestions;
  
  private QuestionPool choosableQuestions;
  

  private JButton startButton;

  private JLabel numberOfTotalQuestionsLabel;
  private JLabel numberOfSelectedQuestionsLabel;
  private JLabel scheduledTimeLabel;
  private JCheckBox unansweredCheckBox;
  private JCheckBox icuFilterCheckBox;
  private JCheckBox cardBoxCheckBox;
  private JComboBox<String> cardBoxComboBox;
  private JRadioButton completeCatRadioButton;
  private JRadioButton numberOfQuestionsRadioButton;
  private JSpinner numberOfQuestionsField;
  private JCheckBox examModeCheckBox;
  private JCheckBox shuffleCheckBox;
  private JTextField searchStringTextField;
  private JButton searchButton;
  
  private int questionTimeLimit;
  private String storedSearchString = "";
  
  private HashMap<Category, Float> averageCategoryLevels = new HashMap<Category, Float>();
  
  protected abstract KreawiInternalFrame buildSelectionPanel();
  
  protected abstract boolean showCardBoxPanel();
  
  protected abstract boolean showCardBoxFilterOption();
  
  protected abstract boolean showExamMode();
  
  protected abstract boolean showICU();
  
  protected abstract boolean isTotalDependsOnFilter();
  
  protected abstract boolean showCategoryColors();
  
  protected abstract QuestionPool getSelectedQuestions();
  
  protected abstract void choosableQuestionsChanged();
  
  protected abstract int getSelectedQuestionsCount();
  
  protected abstract String getExplanationMessageKey();

  private static final Logger logger = LoggerFactory.getLogger(AbstractSelectionDialog.class); 

  public AbstractSelectionDialog(final TrainerFrame mainframe, final TrainerModel model,
      final Map<Actions, Action> actionMap, final QuestionPool questionPool) {

		this.mainframe = mainframe;
		this.model = model;
		this.actionMap = actionMap;
		this.allQuestions = questionPool;

		this.choosableQuestions = this.allQuestions.getClone();

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);
		
		Dimension defaultDialogSize = Util.isLargeFontSize() ? TrainerFrame.MEDIUM_DIALOG_SIZE
				: TrainerFrame.SMALL_DIALOG_SIZE;

		int width = defaultDialogSize.width - 40;
		final int height;
		if (showCardBoxPanel()) {
		  height = defaultDialogSize.height + 40;
		} else {
		  height = defaultDialogSize.height - 90;
		}

		panel.setPreferredSize(new Dimension(width, height));
		panel.setMinimumSize(panel.getPreferredSize());
		
		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		if (showCardBoxPanel()) {
    panel.add(buildStaticVerticalSplit(buildStaticHorizontalSplit(
        buildSelectionPanel(), buildParamsPanel()),
        buildStaticVerticalSplit(buildCardBoxPanel(), buildButtonsPanel(), 100), 100), 
        gbc);
    } else {
      panel.add(buildStaticVerticalSplit(
          buildStaticHorizontalSplit(buildSelectionPanel(), buildParamsPanel()),
          buildButtonsPanel(), 100), gbc);
    }
		add(panel);
		
    /*
     * this.numberOfQuestionsField is initialized in template method
     * buildParamsPanel()
     */
    if (this.allQuestions.isTextQuestionPool()) {
      this.numberOfQuestionsField.setValue(this.model.getConfig().getInt("defaultNumberOfQuestionsText"));
      this.questionTimeLimit = this.model.getConfig().getInt("questionTimeLimitText");
    } else {
      this.numberOfQuestionsField.setValue(this.model.getConfig().getInt("defaultNumberOfQuestions"));
      this.questionTimeLimit = this.model.getConfig().getInt("questionTimeLimit");
    }

    // numberOfTotalQuestionsLabel is updated in applyFilters() -> selectionChanged(), if total depends on the filter
    this.numberOfTotalQuestionsLabel.setText(Integer.toString(this.allQuestions.getQuestionCount()));
    
    addComponentListener(new ComponentAdapter() {
      public void componentShown(ComponentEvent e) {
        refreshIcuFilterState();
        refreshQuestionAttributesFilterState();
        if (showCategoryColors() && !KreawiGlobals.isDemo()) {
          calculateAverageCardBoxLevels();
        }
        showExplanationMessage();
        if (showCardBoxFilterOption()) {
          final boolean showCardBoxWarnings = model.getSettings()
              .getBooleanValue(TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE);
          if (showCardBoxWarnings && model.existsFullCardBox()) {
            showCardBoxWarnings();
          }
        }
      }
    });
    
    this.model.getSettings().addSettingChangedListener(new SettingsChangedListener() {
      @Override
      public void settingChanged(final String key) {
        if (key.equals(TrainerSettingsConstants.KEY_USE_UNANSWERED_FOR_CARD_BOX)) {
          
          applyFilters();
          
        } else if (key.equals(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL)
            || key.equals(TrainerSettingsConstants.KEY_DEFAULT_CARD_BOX_LEVEL)) {
          
          refreshCardBoxLevels();
          if (showCategoryColors() && !KreawiGlobals.isDemo()) {
            calculateAverageCardBoxLevels();
          }
          applyFilters();
          
        }
      }
    });
	}
  
  final protected KreawiInternalFrame buildParamsPanel() {

    final KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
        .getString("CATEGORY_SELECTION_TITLE_PARAMS"),
        new GridBagLayout());

    this.numberOfTotalQuestionsLabel = new JLabel("0");
    
    this.numberOfSelectedQuestionsLabel = new JLabel("0");
    
    
    this.completeCatRadioButton = new JRadioButton(TrainerTextResources
        .getString("SELECTION_LABEL_COMPLETE_CATEGORIES"));
    this.completeCatRadioButton.setOpaque(false);
    this.completeCatRadioButton.setSelected(true);
    this.completeCatRadioButton.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        selectionChanged();
      }
    });
    
    this.numberOfQuestionsRadioButton = new JRadioButton(TrainerTextResources
        .getString("SELECTION_LABEL_NUMBER_OF_QUESTION_MAX"));
    this.numberOfQuestionsRadioButton.setOpaque(false);
    this.numberOfQuestionsRadioButton.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        numberOfQuestionsField.setEnabled(numberOfQuestionsRadioButton.isSelected());
        selectionChanged();
      }
    });
    
    this.numberOfQuestionsField = new JSpinner(new SpinnerNumberModel(
        1, 1,
        999, 1));
    this.numberOfQuestionsField.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        selectionChanged();
      }
    });
    this.numberOfQuestionsField.setEnabled(false);
    
    final ButtonGroup limitQuestionsGroup = new ButtonGroup();
    limitQuestionsGroup.add(this.completeCatRadioButton);
    limitQuestionsGroup.add(this.numberOfQuestionsRadioButton);

    this.scheduledTimeLabel = new JLabel("0 min");

    this.unansweredCheckBox = new JCheckBox(TrainerTextResources
        .getString("SELECTION_LABEL_UNANSWERED_FILTER"));
    this.unansweredCheckBox.setToolTipText(TrainerTextResources
        .getStringWithSubstitution(
            "SELECTION_LABEL_UNANSWERED_FILTER_EXPLANATION", "0"));
    this.unansweredCheckBox.setOpaque(false);
    this.unansweredCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (unansweredCheckBox.isSelected()) {
          showUnansweredFilterExplanationMessage();
        }
        applyFilters();
      }
    });
        
    this.icuFilterCheckBox = new JCheckBox(TrainerTextResources
        .getString("SELECTION_LABEL_ICU_FILTER"));
    this.icuFilterCheckBox.setToolTipText(TrainerTextResources
        .getStringWithSubstitution(
            "SELECTION_LABEL_ICU_FILTER_EXPLANATION", "0"));
    this.icuFilterCheckBox.setOpaque(false);
    this.icuFilterCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (icuFilterCheckBox.isSelected()) {
          showIcuFilterExplanationMessage();
        }
        applyFilters();
      }
    });

    this.cardBoxCheckBox = new JCheckBox();
    this.cardBoxCheckBox.setToolTipText(TrainerTextResources
        .getString(
            "SELECTION_LABEL_CARD_BOX_FILTER_EXPLANATION"));
    this.cardBoxCheckBox.setOpaque(false);
    this.cardBoxCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (cardBoxCheckBox.isSelected()) {
          showCardBoxFilterExplanationMessage();
          cardBoxComboBox.setEnabled(true);
        } else {
          cardBoxComboBox.setEnabled(false);
        }
        applyFilters();
      }
    });

    this.cardBoxComboBox = new JComboBox<String>();
    
    refreshCardBoxLevels();
    
    this.cardBoxComboBox.setEnabled(false);
    this.cardBoxComboBox.setBorder(BorderFactory.createEmptyBorder(0, 2, 0,
        0));
    this.cardBoxComboBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        applyFilters();
      }
    });

    FlowLayout fl = new FlowLayout();
    fl.setHgap(0);
    JPanel cardBoxPanel = new JPanel(fl);
    cardBoxPanel.setOpaque(false);
    cardBoxPanel.add(this.cardBoxCheckBox);
    cardBoxPanel.add(this.cardBoxComboBox);
    
    this.searchStringTextField = new JTextField();
    this.searchStringTextField.setBorder(
        BorderFactory.createCompoundBorder(
            searchStringTextField.getBorder(), 
            BorderFactory.createEmptyBorder(1, 1, 1, 1)));
    this.searchStringTextField.setMargin(new Insets(1, 1, 1, 1));
    this.searchStringTextField.getDocument().addDocumentListener(new DocumentListener() {
      public void removeUpdate(DocumentEvent e) {
        textChanged();
      }
      
      public void insertUpdate(DocumentEvent e) {
        textChanged();
      }
      
      public void changedUpdate(DocumentEvent e) {
        textChanged();
      }
      private void textChanged() {
        searchButton.setEnabled(!searchStringTextField.getText().equals(storedSearchString));
      }
    });
    final String searchButtonLabel = TrainerTextResources.getString("SELECTION_LABEL_SEARCH_BUTTON");
    
    final AbstractAction searchAction = new AbstractAction(searchButtonLabel) {
      @Override
      public void actionPerformed(ActionEvent e) {
        if (!searchButton.isEnabled()) {
          return;
        }
        searchButton.setEnabled(false);
        applyFilters();
        showTextSearchExplanationMessage();
      }
    };
    // SHORT_DESCRIPTION is used for the Tooltip text (for both text field and button)
    final String toolTipSearch = TrainerTextResources.getString("SELECTION_LABEL_SEARCH_BUTTON_TOOL_TIP");
    searchAction.putValue(Action.SHORT_DESCRIPTION, toolTipSearch);
    this.searchButton = new JButton(searchAction);
    this.searchButton.setAction(searchAction);
    this.searchButton.setEnabled(false);
    this.searchStringTextField.setAction(searchAction);
    
    this.model.getQuestionAttributes().addCardBoxChangedListener(new CardBoxChangedListener() {
      @Override
      public void cardBoxLevelChanged() {
        if (!isVisible()) {
          return;
        }
        refreshQuestionAttributesFilterState();
      }
    });
    this.model.getQuestionAttributes().addIcuChangedListener(new IcuChangedListener() {
      @Override
      public void icuChanged() {
        if (!isVisible()) {
          return;
        }
        refreshIcuFilterState();
      }
    });
    
    this.examModeCheckBox = new JCheckBox(TrainerTextResources
      .getString("SELECTION_LABEL_EXAM_MODE"));
    this.examModeCheckBox.setToolTipText(TrainerTextResources
        .getString("SELECTION_LABEL_EXAM_MODE_EXPLANATION"));
    this.examModeCheckBox.setOpaque(false);
    this.examModeCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (examModeCheckBox.isSelected()) {
          showExamModeExplanationMessage();
        }
      }
    });
    
    this.shuffleCheckBox = new JCheckBox(TrainerTextResources
        .getString("SELECTION_LABEL_SHUFFLE"));
    this.shuffleCheckBox.setToolTipText(TrainerTextResources.getString("SELECTION_LABEL_SHUFFLE_EXPLANATION"));
    this.shuffleCheckBox.setSelected(false);
    this.shuffleCheckBox.setOpaque(false);
    
    FormLayout layout = new FormLayout(
    /* columns */
    "1dlu:g(1), right:pref, 3dlu, right:30dlu, 1dlu:g(1)",
    /* rows */
    "f:7dlu:g(7), p," // 1
        + "f:2dlu:g(2), p," // 3
        + "f:4dlu:g(4), p," // 5
        + "f:5dlu:g(5), p," // 7
        + "f:5dlu:g(5), c:20dlu," // 9
        + "p, c:20dlu," // 11
        + "p, c:20dlu," // 13
        + "f:6dlu:g(6), p," // 15
        + "f:4dlu:g(4), p," // 17
        + "f:2dlu:g(2), p," // 19
        + "f:5dlu:g(5), p," // 21
        + "f:5dlu:g(5), p," // 23
        + "f:3dlu:g(3), p," // 25
        + "f:5dlu:g(5), p," // 27
        + "f:5dlu:g(5), c:p," // 29
        + "f:9dlu:g(9)"); // 31

    PanelBuilder builder = new PanelBuilder(layout);

    CellConstraints cc = new CellConstraints();

    builder.addLabel(TrainerTextResources
        .getString("SELECTION_LABEL_TOTAL_NUMBER_OF_QUESTIONS"), cc.xy(
        2, 2));
    builder.add(this.numberOfTotalQuestionsLabel, cc.xy(4, 2));
    builder.addLabel(TrainerTextResources
        .getString("SELECTION_LABEL_NUMBER_OF_QUESTIONS_IN_SELECTION"),
        cc.xy(2, 4));
    builder.add(this.numberOfSelectedQuestionsLabel, cc.xy(4, 4));
    builder.addLabel(TrainerTextResources
        .getString("SELECTION_LABEL_SCHEDULED_TIME"),
        cc.xy(2, 6));
    builder.add(this.scheduledTimeLabel, cc.xy(4, 6));
    
    builder.addSeparator("", cc.xyw(2, 8, 3));
    
    builder.add(this.unansweredCheckBox, cc.xy(2, 10, "l, c"));
    
    if (showICU()) {
      builder.add(this.icuFilterCheckBox, cc.xy(2, 12, "l, c"));
    }
    
    if (showCardBoxFilterOption()) {
      builder.add(cardBoxPanel, cc.xyw(2, 14, 3, "l, c"));
    }
    
    builder.addLabel(TrainerTextResources
        .getString("SELECTION_LABEL_NUMBER_OF_QUESTIONS_TO_DO"), cc.xy(
        2, 16, "l, c"));
    builder.add(this.completeCatRadioButton, cc.xy(
        2, 18, "l, c"));
    builder.add(this.numberOfQuestionsRadioButton, cc.xy(
        2, 20, "l, c"));
    builder.add(this.numberOfQuestionsField, cc.xy(4, 20, "f, c"));
    
    builder.addSeparator("", cc.xyw(2, 22, 3));
    if (showExamMode()) {
      builder.add(this.examModeCheckBox, cc.xy(2, 24, "l, c"));
      builder.add(this.shuffleCheckBox, cc.xy(2, 26, "l, c"));
    }
    else
    {
      builder.add(this.shuffleCheckBox, cc.xy(2, 24, "l, c"));
    }
    builder.addSeparator("", cc.xyw(2, 28, 3));
    final JPanel searchItemsPanel = new JPanel();
    BoxLayout boxLayout = new BoxLayout(searchItemsPanel, BoxLayout.X_AXIS);
    searchItemsPanel.setLayout(boxLayout);
    searchItemsPanel.setOpaque(false);
    searchItemsPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
    searchItemsPanel.add(this.searchStringTextField);
    searchItemsPanel.add(new JLabel(" "));
    searchItemsPanel.add(this.searchButton);
    this.searchStringTextField.setAlignmentY(CENTER_ALIGNMENT);
    builder.add(searchItemsPanel, cc.xyw(2, 30, 3));

    JPanel numberPanel = builder.getPanel();
    numberPanel.setOpaque(false);

    JScrollPane scrollPane = Factory.createStrippedScrollPane(numberPanel);
    scrollPane.setOpaque(false);
    scrollPane.getViewport().setOpaque(false);
    
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridy = 0;
    gbc.gridx = 0;
    gbc.gridheight = 1;
    gbc.gridwidth = 1;
    gbc.weightx = 1;
    gbc.weighty = 1;

    kif.getContent().setBorder(BorderFactory.createEmptyBorder());
    kif.getContent().add(scrollPane, gbc);


    return kif;
  }

  private Component buildCardBoxPanel() {
    final KreawiInternalFrame kif = new KreawiInternalFrame(null, new BorderLayout());
    kif.setPreferredSize(new Dimension(800, 105));
    kif.setMaximumSize(new Dimension(800, 105));
    kif.setMinimumSize(new Dimension(600, 105));
    CardBoxPanel cardBoxPanel = new CardBoxPanel(model, mainframe, actionMap.get(Actions.START_CARD_BOX_QUIZ));
    JPanel gradientPanel = kif.getContent();
    gradientPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    gradientPanel.setMinimumSize(new Dimension(400, 220));
    gradientPanel.add(cardBoxPanel);
    return kif;
  }

  protected KreawiInternalFrame buildButtonsPanel() {
	
		KreawiInternalFrame kif = new KreawiInternalFrame(null,
				new GridBagLayout());
	
		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 11, 10));
	
		this.startButton = new JButton(TrainerTextResources
				.getString("SELECTION_BUTTON_START"));
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				selectionConfirmed();
			}
		});
		startButton.setEnabled(false);
		startButton.setFont(new Font(startButton.getFont().getName(),
				Font.BOLD, startButton.getFont().getSize()));
	
		JButton backButton = new JButton(this.actionMap
				.get(Actions.BACK_TO_MAIN_DIALOG));
	
		ButtonBarBuilder builder = new ButtonBarBuilder();
		
		if (builder.isLeftToRightButtonOrder()) {
			builder.addGlue();
			builder.addGridded(startButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(backButton);
			builder.addGlue();
		} else {
			builder.addGlue();
			builder.addGridded(backButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(startButton);
			builder.addGlue();

		}
		
		JPanel buttonsPanel = builder.getPanel();
		buttonsPanel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		
		gradientPanel.add(buttonsPanel, gbc);
	
		return kif;
	}

	private void applyFilters() {
    long start = System.currentTimeMillis();

    this.choosableQuestions = this.allQuestions.getClone();

    if (useUnansweredQuestionsOnly()) {
      this.choosableQuestions.intersect(this.model
          .getUnansweredQuestions());
    }
    
    if (useICUQuestionsOnly()) {
      this.choosableQuestions.intersect(this.model.getIcuQuestions());
    }
    
    if (useCardBoxQuestionsOnly()) {
      
      final int cardBoxLevel = getSelectedCardBoxLevel();
      final QuestionPool cardBox = this.model.getQuestionsForCardBoxLevel(cardBoxLevel);
      
      this.choosableQuestions.intersect(cardBox);
    }
    
    final String searchString = getSearchString();
    if (!"".equals(searchString)) {
      try {
        this.choosableQuestions.intersect(this.model.searchQuestions(searchString));
      } catch (TrainerStorageException e) {
        mainframe.showFatalErrorDialogWithoutQuit();
      }
    }
    storedSearchString = searchString;
    
    if (KreawiGlobals.isDevMode()) {
      logger.info("Time for applyFilters: " + Long.toString((System.currentTimeMillis() - start)) + "ms");
    }

    choosableQuestionsChanged();
    selectionChanged();

  }
	
  private void refreshCardBoxLevels() {
	  
	  this.cardBoxComboBox.removeAllItems();
	  
    int maxCardBoxLevel = this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL);
	  
	  for (int i = 0; i <= maxCardBoxLevel; i++) {
	    final String level = (i==maxCardBoxLevel?"'Archiv'":"" + (i + 1));
      this.cardBoxComboBox.addItem(TrainerTextResources.getStringWithSubstitution("SELECTION_LABEL_CARD_BOX_FILTER",
          level));
	  }
	  
	  
    this.cardBoxComboBox.setSelectedIndex(Math.min(
        this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_DEFAULT_CARD_BOX_LEVEL),
        this.cardBoxComboBox.getItemCount() - 1));
	}
	
	private void refreshIcuFilterState() {
	  int icuCatCount = this.allQuestions.getIntersection(
	      this.model.getIcuQuestions()).getQuestionCount();
	  this.icuFilterCheckBox.setEnabled(icuCatCount > 0);
	  this.icuFilterCheckBox.setToolTipText(TrainerTextResources
	      .getStringWithSubstitution(
	          "SELECTION_LABEL_ICU_FILTER_EXPLANATION", ""
	          + icuCatCount));
	  if (icuCatCount < 1) {
	    this.icuFilterCheckBox.setSelected(false);
	  }
	  applyFilters();
	}
	
	private void refreshQuestionAttributesFilterState() {
	  QuestionPool qp = this.model.getUnansweredQuestions();
	  qp.intersect(this.allQuestions);
	  //	    QuestionPool qp = this.model.getUnansweredQuestions().getIntersection(
	  //	        this.allQuestions);
	  
	  int unansweredCatCount = qp.getQuestionCount();
	  this.unansweredCheckBox.setEnabled(unansweredCatCount > 0);
	  this.unansweredCheckBox.setToolTipText(TrainerTextResources
	      .getStringWithSubstitution(
	          "SELECTION_LABEL_UNANSWERED_FILTER_EXPLANATION", ""
	          + unansweredCatCount));
	  if (unansweredCatCount < 1) {
	    this.unansweredCheckBox.setSelected(false);
	  }
	  applyFilters();
	}
	
	final protected void selectionChanged() {
    
    final int selectedCount = getSelectedQuestionsCount();
    
    if (this.choosableQuestions != null && isTotalDependsOnFilter()) {
        this.numberOfTotalQuestionsLabel.setText(this.choosableQuestions.getQuestionCount() + "");
    }
    
    this.numberOfSelectedQuestionsLabel.setText(selectedCount + "");
    
    this.startButton.setEnabled(selectedCount > 0);
    
    final int numberOfQuestionsToDo;
    if (this.completeCatRadioButton.isSelected()) {
      numberOfQuestionsToDo = selectedCount;
    } else {
      numberOfQuestionsToDo = (Integer) (this.numberOfQuestionsField.getValue());
    }
    
    long minutes = (long) (Math.min(selectedCount, numberOfQuestionsToDo) * ((double) this.questionTimeLimit / (double) 60000));
    this.scheduledTimeLabel.setText(minutes + " min");
    
  }
	
	protected void selectionConfirmed() {

    final QuestionPool selectedQuestions = getSelectedQuestions();
    
    if (selectedQuestions == null || selectedQuestions.getQuestionCount() == 0) {
      return;
    }

	  final KreawiQuiz quiz;
    if (useCardBoxQuestionsOnly()) {
      final int cardBoxLevel = getSelectedCardBoxLevel();
      quiz = new KreawiQuiz(selectedQuestions, isExamMode(), cardBoxLevel);
      Comparator<Question> comp = new Comparator<Question>() {
        public int compare(Question o1, Question o2) {
          Long timestamp1 = Long.valueOf(model.getCardBoxTimestamp(o1));
          Long timestamp2 = Long.valueOf(model.getCardBoxTimestamp(o2));
          
          return timestamp1.compareTo(timestamp2);
        }
      };
      
      quiz.sortQuestions(comp);
    } else {
      quiz = new KreawiQuiz(selectedQuestions, isExamMode());
      if ((selectedQuestions.getCategories().size() + selectedQuestions.getExams().size()) == 1) {
        quiz.sortQuestionsById();
      }
    }
    
    if (shuffleCheckBox.isSelected()) {
      quiz.shuffleQuestions();
    }
    
    model.setSelectedQuestions(quiz);

  }
	
  private void calculateAverageCardBoxLevels() {
    final int userMaxCardBoxLevel = this.model.getSettings()
        .getIntValue(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL);
    
    int defaultCardBoxLevel = this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_DEFAULT_CARD_BOX_LEVEL);
    
    /*
     * if default card box level is 0, we have to push it up to 1 for this
     * calculation, or the color red would never be visible
     */
    if (defaultCardBoxLevel == 0) {
      defaultCardBoxLevel = 1;
    }
    
    for (Category category : this.allQuestions.getCategories()) {
      this.averageCategoryLevels.put(category, calculateAverageCardBoxLevel(
          category, userMaxCardBoxLevel, defaultCardBoxLevel));
    }
  }

  protected Color calculateCategoryColor(Category category, Color baseColor) {

    final int userMaxCardBoxLevel = this.model.getSettings()
        .getIntValue(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL);
    
    int defaultCardBoxLevel = this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_DEFAULT_CARD_BOX_LEVEL);
    
    /*
     * if default card box level is 0, we have to push it up to 1 for this
     * calculation, or the color red would never be visible
     */
    if (defaultCardBoxLevel == 0) {
      defaultCardBoxLevel = 1;
    }

    float averageLevel = this.averageCategoryLevels.get(category) != null ? this.averageCategoryLevels.get(category)
        : defaultCardBoxLevel;
        
    final Color color;
    if (averageLevel < defaultCardBoxLevel) {
      float factor = averageLevel / defaultCardBoxLevel;
      color = ColorUtil.mixColorsRGB(KreawiGlobals.WRONG_COLOR,
          baseColor, factor);
    } else {
      float factor = (averageLevel - defaultCardBoxLevel)
          / (userMaxCardBoxLevel - defaultCardBoxLevel);
      color = ColorUtil.mixColorsRGB(baseColor,
          KreawiGlobals.CORRECT_COLOR, factor);
    }

    return color;
  }
  
  private float calculateAverageCardBoxLevel(Category category,
      int maxCardBoxLevel, int defaultCardBoxLevel) {

    QuestionPool qp = this.allQuestions
        .getQuestionPoolForCategory(category);
    float levelSum = 0;

    for (Question question : qp.getQuestions()) {
      int currentLevel = this.model.getTrueCardBoxLevel(question);

      /*
       * unanswered questions have no level yet. They have to count for
       * the default level / the threshold.
       */
      if (currentLevel == QuestionAttributesModel.CARD_BOX_LEVEL_UNDEFINED) {
        currentLevel = defaultCardBoxLevel;
      }

      levelSum += Math.min(maxCardBoxLevel, currentLevel);
    }

    float averageLevel = levelSum / qp.getQuestionCount();
    
    if (averageLevel != defaultCardBoxLevel) {
      logger.info("Average card box level for category '" + category + "' is " + averageLevel);
    }
    
    return averageLevel;

  }
  
  final protected boolean isTextQuestionPool() {
    return this.allQuestions.isTextQuestionPool();
  }
  
  final protected Set<Category> getAllCategories() {
    return this.allQuestions.getCategories();
  }
  
  final protected Set<Exam> getAllExams() {
    return this.allQuestions.getExams();
  }
  
  final protected Set<Category> getCategoriesForExam(final Exam exam) {
    return this.allQuestions.getQuestionPoolForExam(exam).getCategories();
  }
  
  final protected QuestionPool getChoosableQuestions() {
    return this.choosableQuestions;
  }

	protected int getSelectedCardBoxLevel() {
	  return this.cardBoxComboBox.getSelectedIndex();
	}
	
	protected boolean useUnansweredQuestionsOnly() {
	  return this.unansweredCheckBox.isSelected();
	}
	
	protected boolean useCardBoxQuestionsOnly() {
	  return this.cardBoxCheckBox.isSelected();
	}
	
	protected boolean useICUQuestionsOnly() {
	  return this.icuFilterCheckBox.isSelected();
	}
	
	protected boolean isMaxNumberOfQuestionsSelected() {
	  return this.numberOfQuestionsRadioButton.isSelected();
	}
	
	protected int getMaxNumberOfQuestions() {
	  return (Integer) (this.numberOfQuestionsField.getValue());
	}
	
	protected boolean isExamMode() {
	  /*
     * Set the exam mode if selected. ATTENTION: Exam mode makes no sense at all
     * for text questions!
     */
    if (this.allQuestions.isTextQuestionPool()) {
      return false;
    } else {
      return examModeCheckBox.isSelected();
    }
	}
	
	protected String getSearchString() {
	  return this.searchStringTextField.getText();
	}
	
	
  protected void showExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe, getExplanationMessageKey());
  }
	  
	private void showIcuFilterExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_ICU_FILTER_EXPLANATION_MESSAGE);
	}
	
	private void showUnansweredFilterExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_UNANSWERED_FILTER_EXPLANATION_MESSAGE);
	}
	
	private void showCardBoxFilterExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_CARD_BOX_FILTER_EXPLANATION_MESSAGE);
	}
	
	private void showTextSearchExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_TEXT_SEARCH_EXPLANATION_MESSAGE);
	}

	protected void showExamModeExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_EXAM_MODE_EXPLANATION_MESSAGE);
	}
	
	protected void showShuffleExplanationMessage() {
    this.model.getHelpRegistry().showHelpMessage(this.mainframe,
        TrainerSettingsConstants.KEY_SHOW_EXAM_MODE_EXPLANATION_MESSAGE);// FIXME
	}

  private void showCardBoxWarnings() {
    final Component parent = this;
    final int maxLevel = this.model.getSettings().getIntValue(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL) + 1;
    // Jeder Kasten außer letzter (Archiv) hat 2 Werte: 
    // Fragen enthalten und Menge ab der gewarnt wird
    // Archiv hat nur die enthaltenen Fragen, aber keinen Maximalwert!
    final String[] substitutesCardBoxWarnings = new String[maxLevel * 3 - 2];  
    
    
    for (int i=0; i<substitutesCardBoxWarnings.length; i=i+3) {
      final int cardBoxLevel = (i)/3;
      final int numberQuestionsCurrent = model.getQuestionsForCardBoxLevel(cardBoxLevel).getQuestionCount();
      substitutesCardBoxWarnings[i] = "" + numberQuestionsCurrent;
      if (i != substitutesCardBoxWarnings.length - 1) {
        // Archiv hat nur die enthaltenen Fragen, aber keinen Maximalwert und keine Warnmeldung!
        final int numberQuestionsTotal = KreawiGlobals.getCardBoxWarnLimit(cardBoxLevel);
        substitutesCardBoxWarnings[i+1] = "" + numberQuestionsTotal;
        final String warning = TrainerTextResources.getString("SELECTION_LABEL_CARD_BOX_WARNINGS_FULL");
        final String hint = TrainerTextResources.getString("SELECTION_LABEL_CARD_BOX_WARNINGS_NOT_FULL");
        substitutesCardBoxWarnings[i+2] = (numberQuestionsCurrent >= numberQuestionsTotal?warning:hint);
      }
    }
    
    final String key_message = getTextResourceCardBoxWarnings(maxLevel);
    final String warnings = TrainerTextResources.getStringWithSubstitution(key_message, substitutesCardBoxWarnings);
    final String title = TrainerTextResources.getString("SELECTION_LABEL_CARD_BOX_WARNINGS_TITLE");
    final String settingsKey = TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE;
    TrainerUtil.showDeactivateableDialog(title, warnings, settingsKey,
        new Config(this.model.getSettings().getSettingProperties()), parent);
  }

  private static String getTextResourceCardBoxWarnings(final int maxLevel) {
    return "SELECTION_LABEL_CARD_BOX_WARNINGS_MESSAGE_FOR_MAX_LEVEL_" + maxLevel;
  }

}