package de.kreawi.trainer.dialogs.selection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.Category;
import com.jquizengine.questions.Exam;
import com.jquizengine.questions.TitledQuestionGroup;
import com.jquizengine.questions.collection.QuestionPool;

import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerStorage;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.awt.ColorUtil;
import de.martinknauer.util.Util;

@SuppressWarnings("serial")
abstract public class ListSelectionDialog extends AbstractSelectionDialog {

	private JList<TitledQuestionGroup> list;

  protected abstract boolean showExams();
  
  private static final Logger logger = LoggerFactory.getLogger(ListSelectionDialog.class); 

  public ListSelectionDialog(final TrainerFrame mainframe, final TrainerModel model,
	    final Map<Actions, Action> actionMap, final QuestionPool questionPool) {
		super(mainframe, model, actionMap, questionPool);
	}
	
	@Override
	protected KreawiInternalFrame buildSelectionPanel() {

		String title = TrainerTextResources
				.getString("CATEGORY_SELECTION_TITLE_CATEGORIES");

		if (showExams()) {
			title += " / "
					+ TrainerTextResources
							.getString("CATEGORY_SELECTION_TITLE_EXAMS");
		}
		KreawiInternalFrame kif = new KreawiInternalFrame(title,
				new BorderLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		/* sort category strings */
		ArrayList<TitledQuestionGroup> sortedCategories = new ArrayList<TitledQuestionGroup>(
				getAllCategories());
		Collections.sort(sortedCategories);

		//FIXME: Hier sollten auch die Prüfungen als Kategorien auftauchen???
		//       Aber getExams() liefert ein leeres Array zurück! (außer in Demo!?)
//		ArrayList<TitledQuestionGroup> sortedExams = new ArrayList<TitledQuestionGroup>(
//				this.allQuestions.getExams());
//		Collections.sort(sortedExams);
//		Collections.reverse(sortedExams);

		ArrayList<TitledQuestionGroup> sortedGroups = new ArrayList<TitledQuestionGroup>(
				sortedCategories);
//		sortedGroups.addAll(sortedExams);

		this.list = new JList<TitledQuestionGroup>(new Vector<TitledQuestionGroup>(sortedGroups));
		this.list.setToolTipText(getSelectionToolTipText());
		this.list.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				selectionChanged();
			}

		});

		this.list.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				if (me.getClickCount() == 2) {
          if (list.isSelectedIndex(list.locationToIndex(me.getPoint()))) {
						selectionConfirmed();
					}
				}
			}
		});

		if (Boolean.TRUE.equals(UIManager.getDefaults().get(
				KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY))) {
			this.list.setOpaque(false);
		}

		this.list.setCellRenderer(new DefaultListCellRenderer() {

			@Override
      public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected,
          boolean cellHasFocus) {
				JComponent comp = (JComponent) super
						.getListCellRendererComponent(list, value, index,
								isSelected, false);

				Insets insets = comp.getBorder().getBorderInsets(comp);

				comp.setBorder(BorderFactory.createEmptyBorder(insets.top,
						insets.left + 2, insets.bottom, insets.right));

				boolean isEnabled = true;

				if (value instanceof TitledQuestionGroup) {
					final TitledQuestionGroup group = (TitledQuestionGroup) value;

          final int questionCount = getChoosableQuestions().getQuestionCountForGroup(group);

					if (questionCount < 1) {
						isEnabled = false;
					}

					String valueString = "";

					if (value instanceof Category) {
						Category category = (Category) value;

						valueString = category.getTitle();

						if (showCategoryColors()) {

							if (isEnabled) {

								Color color = null;

								if (isSelected) {

									Color bg = comp.getBackground();

									if (Util.isMacOS()) {
										// on mac os x, the standard blue
										// doesn't
										// look good with red and green
										bg = ColorUtil.convertToRGB(ColorUtil
												.convertToGrayscale(comp
														.getBackground()));
									}

									color = calculateCategoryColor(category, bg);

									comp.setBackground(color);

									if (ColorUtil.isDarkColor(color)) {
										comp.setForeground(Color.WHITE);
									}

								} else {
									comp.setForeground(calculateCategoryColor(
											category, comp.getForeground()));
								}
							}

						}
					} else if (value instanceof Exam) {
						Exam exam = (Exam) value;

						valueString = exam.getTitle();

					}

					valueString += " (" + questionCount + ")";

					setText(valueString);
				}

				comp.setEnabled(isEnabled);

				if (!(isSelected && isEnabled)) {
					comp.setOpaque(false);
				} else {
					comp.setOpaque(true);
				}

				return comp;

			}
		});
		
		JScrollPane scrollPane = Factory.createStrippedScrollPane((this.list));
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);

		gradientPanel.add(scrollPane);

		setFocusJComponent(this.list);

		return kif;
	}

	protected abstract String getSelectionToolTipText();
	
	protected String getSelectionToolTiptextImpl(String key) {
		String modifierKey = Util.getModifierKeyName();
		
		String[] subst = { modifierKey, modifierKey, modifierKey };
		
		
		String toolTipText = TrainerTextResources.getStringWithSubstitution(
			key, subst);
		
		return toolTipText;
	}

	final protected void choosableQuestionsChanged() {
    list.revalidate();
    list.repaint();
	}
	
	private Set<TitledQuestionGroup> extractGroups(final List<TitledQuestionGroup> list) {

		HashSet<TitledQuestionGroup> groups = new HashSet<TitledQuestionGroup>();

		for (TitledQuestionGroup currentCat : list) {
			groups.add(currentCat);
		}

		return groups;
	}
	
  @Override
  protected QuestionPool getSelectedQuestions() {
    
    final Set<TitledQuestionGroup> groups = extractGroups(this.list.getSelectedValuesList());
    
    final int numberOfQuestionsInSelection = getChoosableQuestions().getQuestionCountForGroups(groups);
    
    final int numberOfQuestionsToDo;
    if (isMaxNumberOfQuestionsSelected()) {
      numberOfQuestionsToDo = getMaxNumberOfQuestions();
    } else {
      numberOfQuestionsToDo = numberOfQuestionsInSelection;
    }
    
    final QuestionPool selectedQuestions;
    
    if (groups.size() == 1 && getChoosableQuestions().getQuestionCountForGroups(groups) == numberOfQuestionsToDo) {
      selectedQuestions = getChoosableQuestions().getQuestionPoolForGroups(groups);
    } else {
      selectedQuestions = getChoosableQuestions().getRandomQuestionPoolForGroups(groups, numberOfQuestionsToDo);
    }
    
    return selectedQuestions;
    
  }

	@Override
	protected int getSelectedQuestionsCount() {
    if (getChoosableQuestions() != null) {
      return getChoosableQuestions().getQuestionCountForGroups(extractGroups(this.list.getSelectedValuesList()));
    } else {
      return 0;
    }
	}
	
}
