package de.kreawi.trainer.license;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.text.MaskFormatter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;

public class KreawiLicenseHandler {

	public static final String LICENSE_KEY_FORMAT_MASK = "AAAA-AAAA-AAAA-AAAA-AAAA-AAAA-AAAA-AAAA";

	private static KreawiLicenseHandler s_instance;

  private Boolean hasValidLicense = null;

  private static Logger logger;

	private KreawiLicenseHandler() {
    
    // init logger after System.setProperty("trainerAppDir", ...) in main()
    logger = LoggerFactory.getLogger(KreawiLicenseHandler.class);
    
	}
	
	public static KreawiLicenseHandler getInstance() {
		if (s_instance == null) {
			s_instance = new KreawiLicenseHandler();
		}
		return s_instance;
	}
	
  public void askForLicenseKey(final Component parent, final Runnable nextAction) {
    
    final String prefilledKey = LicenseFileHandler.getCurrentLicenceKey();

    // Alles zurücksetzen (inkl. der Registrierung, falls es eine gab)
    LicenseFileHandler.resetProperties();
    
    final String text;
    final boolean enteredKeyInvalid = (prefilledKey != null);
    if (enteredKeyInvalid) {
      text = TrainerTextResources.getStringWithSubstitution("LICENCE_TEXT_INVALID_KEY", KreawiGlobals.CURRENT_YEAR);
    } else {
      if (KreawiGlobals.dataExists()) {
        text = TrainerTextResources.getStringWithSubstitution("LICENCE_TEXT_NEW_KEY_FULL", new String[] {
            KreawiGlobals.CURRENT_YEAR, KreawiGlobals.CURRENT_YEAR });
      } else {
        text = TrainerTextResources.getStringWithSubstitution("LICENCE_TEXT_NEW_KEY_DEMO", new String[] {
            KreawiGlobals.CURRENT_YEAR, KreawiGlobals.CURRENT_YEAR });
      }
    }
    JScrollPane scrollPane = TrainerHtmlUtil.createHTMLMessageScrollPane(text);

    BorderLayout borderLayout = new BorderLayout();
    borderLayout.setVgap(15);

    final JPanel panel = new JPanel(borderLayout);
    panel.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
    panel.add(scrollPane, BorderLayout.CENTER);

    MaskFormatter licenceKeyFormatter = null;
    try {
      licenceKeyFormatter = new MaskFormatter(KreawiLicenseHandler.LICENSE_KEY_FORMAT_MASK);
      licenceKeyFormatter.setPlaceholderCharacter('.');
    } catch (ParseException e) {
      /* do nothing */
    }

    final JFormattedTextField licenceKeyField = new JFormattedTextField(
        licenceKeyFormatter);

    licenceKeyField.setFont(new Font("Monospaced", Font.PLAIN,
        licenceKeyField.getFont().getSize()));

    if (enteredKeyInvalid) {
      licenceKeyField.setText(prefilledKey);
    }

    licenceKeyField.setMaximumSize(new Dimension(licenceKeyField
        .getPreferredSize().width + 2, licenceKeyField
        .getPreferredSize().height));
    licenceKeyField.setPreferredSize(licenceKeyField.getMaximumSize());

    JPanel fieldPanel = new JPanel();
    fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.X_AXIS));
    fieldPanel.add(Box.createHorizontalGlue());
    fieldPanel.add(licenceKeyField);
    fieldPanel.add(Box.createHorizontalGlue());

    panel.add(fieldPanel, BorderLayout.SOUTH);

    licenceKeyField.addPropertyChangeListener("ancestor",
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent evt) {
            Timer timer = new Timer(300, new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                SwingUtilities.invokeLater(new Runnable() {
                  public void run() {
                    licenceKeyField.requestFocusInWindow();
                  }
                });
              }
            });
            timer.setRepeats(false);
            timer.start();
          }
        });

    FocusListener fl = new FocusAdapter() {
      public void focusGained(FocusEvent e) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            licenceKeyField.setCaretPosition(0);
          }
        });
        licenceKeyField.removeFocusListener(this);
      }
    };
    licenceKeyField.addFocusListener(fl);

    licenceKeyField.setFocusLostBehavior(JFormattedTextField.COMMIT);

    final String title = TrainerTextResources.getString("LICENCE_DIALOG_TITLE");

    final String[] options;

    if (enteredKeyInvalid) {
      options = new String[3];
      options[0] = TrainerTextResources.getString("LICENCE_BUTTON_OK");
      options[1] = TrainerTextResources.getString("LICENCE_BUTTON_DEMO");
      options[2] = TrainerTextResources.getString("LICENCE_BUTTON_RESET");
    } else {
      options = new String[2];
      options[0] = TrainerTextResources.getString("LICENCE_BUTTON_OK");
      options[1] = TrainerTextResources.getString("LICENCE_BUTTON_DEMO");
    }

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        int answer = JOptionPane.showOptionDialog(parent, panel,
            title, enteredKeyInvalid ? JOptionPane.YES_NO_CANCEL_OPTION
                : JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE,
                null, options, options[0]);

        try {
          licenceKeyField.commitEdit();
        } catch (ParseException e) {
          logger.info("FEHLER: Lizenzschlüssel konnte nicht geparst werden!");
        }
        final String enteredKey = (String) licenceKeyField.getValue();
        if (answer == JOptionPane.YES_OPTION) {
          LicenseFileHandler.setLicenseKey(enteredKey);
          if (!checkLicense()) {
            askForLicenseKey(parent, nextAction);
          } else {
            nextAction.run();
          }
        } else if (answer == JOptionPane.CANCEL_OPTION) {
          LicenseFileHandler.resetProperties();
          askForLicenseKey(parent, nextAction);
        } else {
          // Run in Demo mode (NO_OPTION)
          nextAction.run();
        }
      }
    });
  }
  
  public void deleteLicense() {
    LicenseFileHandler.resetProperties();
  }
  
  public boolean hasValidLicence() {
    
    if (this.hasValidLicense == null) {
      checkLicense();
    }
    
    return this.hasValidLicense.booleanValue();
  }

  private boolean checkLicense() {
    final String licenceKey = LicenseFileHandler.getCurrentLicenceKey();
    
    final boolean avoidAmbigiousChars = false;
    
    this.hasValidLicense = licenceKey != null && isValidLicence(licenceKey, avoidAmbigiousChars);
    
    return this.hasValidLicense;
  }
  
  private boolean isValidLicence(final String licenseKey, final boolean avoidAmbigiousChars) {
    
    if (licenseKey == null || LICENSE_KEY_FORMAT_MASK.length() != licenseKey.length()) {
      return false;
    }
    
	  final String test = "kreawiTrainer" + KreawiGlobals.CURRENT_YEAR;
//		try {
//      final File out_file = new File(KreawiGlobals.getApplicationDirectory(), test + ".csv");
//			out_file.createNewFile();
//			BufferedWriter writer = new BufferedWriter(new FileWriter(out_file));

			for (int i = 100000; i <= 129999; i++) {
				String number = i + "";
				number.substring(1);
				try {
				  String hash = hash(test + number);

				  if (!(avoidAmbigiousChars && (hash.toLowerCase().contains(
							"i") || hash.toLowerCase().contains("o")))) {

				    if (hash.equalsIgnoreCase(licenseKey)) {
				      return true;
				    }
//				    writer.write(hash);
//				    writer.newLine();
				  } else {
//						logger.debug("Dropping key " + i + ": " + hash);
					}
					
				} catch (NoSuchAlgorithmException e) {
					/* do nothing */
				} catch (IOException e) {
					/* do nothing */
				}
			}

//			writer.flush();
//			writer.close();
//		} catch (IOException e1) {
//			e1.printStackTrace();
//		}

		return false;
	}

	private String hash(String in) throws IOException, NoSuchAlgorithmException {

		MessageDigest digest = MessageDigest.getInstance("MD5");

		byte[] bytes = digest.digest(in.getBytes("UTF-16"));

		String ret = "";

		for (int i = 0; i < bytes.length; i++) {
			byte[] oneByte = new byte[1];
			oneByte[0] = bytes[i];

			String item = new BigInteger(1, oneByte).toString(36);

			if (item.length() < 2) {
				item = "0" + item;
			}

			ret += item;
			if (((i + 1) % 2) == 0 && (i + 1) < bytes.length) {
				ret += "-";
			}
		}

		ret = ret.toUpperCase(new Locale("en"));

		return ret;

	}

  public static void main(String[] args) {
    
    // For Log4j configuration
    System.setProperty("trainerAppDir", KreawiGlobals.getApplicationDirectory().getAbsolutePath());
    
    final boolean avoidAmbigiousChars = true;
    
    KreawiLicenseHandler.getInstance().isValidLicence("XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-XXXX-XXXX", avoidAmbigiousChars);
    
    logger.info("Fertig!");
    
  }
}