/**
 * 
 */
package de.kreawi.trainer.license;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.URL;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.HttpHostConnectException;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;

/**
 * @author dominik
 *
 */
public class RegistrationHandler {
  
  private static final int SERVER_REGISTRATION_STATUS_OK = 0;
  private static final int SERVER_REGISTRATION_STATUS_WARN = 1;
  private static final int SERVER_REGISTRATION_STATUS_LOCKED = 2;
  private static final int SERVER_REGISTRATION_STATUS_SUPER_LICENSE = 3;
  private static final int SERVER_REGISTRATION_STATUS_NOT_FOUND = 4;
  
  //private static final String LICENSE_SERVER_URL = "http://localhost:3000/";
  private static final String LICENSE_SERVER_URL = "http://kreawi-lizenz.de/";
	private static final String CHECK_URL = LICENSE_SERVER_URL + "registrations/check.json";
	private static final String NEW_URL = LICENSE_SERVER_URL + "registrations/new.json";
  private static final String DELETE_URL = LICENSE_SERVER_URL + "registrations/delete.json";
  private static final String SUPPORT_URL = LICENSE_SERVER_URL + "support/new_request.html";
	
  private static final String APP_ID = "desktop";

	private static Boolean onlineCheck;
	
	private static RegistrationHandler s_instance;
	
	public static RegistrationHandler getInstance() {
	  if (s_instance == null) {
	    s_instance = new RegistrationHandler();
	  }
	  return s_instance;
	}
	
	private static final Logger logger = LoggerFactory.getLogger(RegistrationHandler.class); 

	private RegistrationHandler() {
		LicenseFileHandler.possiblySetFirstStart();
	}
	
	private boolean sendRegistration(final String email, final Component parent)	{
		boolean result = false;
		
		final String uid = UUID.randomUUID().toString();
		
		String computerName = null;
		try {
			computerName = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e) {
			computerName = "";
			logger.error("Could not fetch computer name", e);
		}
		
    final String licenseKey = LicenseFileHandler.getCurrentLicenceKey();
    
    String url;
    try {
      url = NEW_URL + "?client_uid=" + URLEncoder.encode(uid, "utf-8")
          + "&email=" + URLEncoder.encode(email, "utf-8")
          + "&computer_name=" + URLEncoder.encode(computerName, "utf-8")
          + "&trainer_year=" + URLEncoder.encode(KreawiGlobals.CURRENT_YEAR, "utf-8")
          + "&license_key=" + URLEncoder.encode(licenseKey, "utf-8")
          + "&app=" + URLEncoder.encode(APP_ID, "utf-8");
		
      final String answer = executeGet(url);
		
      final JSONObject json = new JSONObject(answer);
      LicenseFileHandler.setRegistration(uid, email, computerName, json.getInt("status"));
      resetFailedChecks();
      
      result = true;
    } catch (UnsupportedEncodingException e) {
      logger.error("Konnte nicht registrieren! (" + e.toString() + ")");
    } catch (JSONException e) {
      logger.error("Konnte nicht registrieren! (" + e.toString() + ")");
    }
		
    return result;
	}
	
	public String executeGet(String urlToRead) {
	  URL url;
	  HttpURLConnection conn;
	  BufferedReader rd;
    String line;
    String result = "";
    try {
       url = new URL(urlToRead);
       conn = (HttpURLConnection) url.openConnection();
       conn.setRequestMethod("GET");
       rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
       while ((line = rd.readLine()) != null) {
          result += line;
       }
       rd.close();
    } catch (IOException e) {
       e.printStackTrace();
    } catch (Exception e) {
       e.printStackTrace();
    }
    return result;
	}

  private void showRejectedMessage(final Component parent, final String reason) {
    
    final String support_reasons = createSupportReasons();
    
    final String message;
    if (reason== null || reason.length() == 0) {
      message = TrainerTextResources.getStringWithSubstitution("LICENSE_VALIDATOR_REJECTED", support_reasons);
    } else {
      message = TrainerTextResources.getStringWithSubstitution("LICENSE_VALIDATOR_REJECTED_REASON",
          new String[] { reason, support_reasons });
    }
    
    JScrollPane scrollPane = TrainerHtmlUtil.createHTMLMessageScrollPane(message);
    scrollPane.setOpaque(false);
    scrollPane.setBorder(BorderFactory.createEmptyBorder());
    JOptionPane.showMessageDialog(parent, scrollPane, 
    		TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE"), JOptionPane.WARNING_MESSAGE);
  }

  private void showWarnMessage(final Component parent) {
    
    final String support_reasons = createSupportReasons();
    
    final String text = TrainerTextResources.getStringWithSubstitution("LICENSE_VALIDATOR_WARN", support_reasons);
    
    JScrollPane scrollPane = TrainerHtmlUtil.createHTMLMessageScrollPane(text);
    scrollPane.setOpaque(false);
    scrollPane.setBorder(BorderFactory.createEmptyBorder());
    JOptionPane.showMessageDialog(parent, scrollPane, 
        TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE"), JOptionPane.WARNING_MESSAGE);
  }
  
  private String createSupportReasons() {
    final String client_uid = LicenseFileHandler.getRegUid();
    final String support_url1 = SUPPORT_URL + "?client_uid=" + client_uid + "&reason=1";
    final String support_url2 = SUPPORT_URL + "?client_uid=" + client_uid + "&reason=2";
    final String support_url3 = SUPPORT_URL + "?client_uid=" + client_uid + "&reason=3";
    final String support_reasons = TrainerTextResources.getStringWithSubstitution("LICENSE_VALIDATOR_SUPPORT_REASONS",
        new String[] { support_url1, support_url2, support_url3 });
    return support_reasons;
  }
  
	private static void resetFailedChecks() {
	  LicenseFileHandler.setFailedCount(Integer.toString(0));
	}

	private static void registerFailedCheck() {
		String failedCount = LicenseFileHandler.getFailedCount();
		if (failedCount == null) {
			failedCount = Integer.toString(1);
		} else {
			int i = Integer.parseInt(failedCount);
			failedCount = Integer.toString(++i);
		}
		LicenseFileHandler.setFailedCount(failedCount);
	}


  private void showFailedTooOften(final Component parent) {
    JOptionPane.showMessageDialog(parent, TrainerTextResources.getString("LICENSE_VALIDATOR_FAILED_TOO_OFTEN"), 
    		TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE"), JOptionPane.ERROR_MESSAGE);
  }

  private void askForLicenseRegistration(final Component parent, final Runnable nextTask, final Runnable exitTrainer) {
    
		final boolean necessary = isRegistrationNecessary();
		
		final String text;
		if (necessary) {
      text = TrainerTextResources.getString("LICENSE_VALIDATOR_REGISTER_MESSAGE_URGENT");
		} else {
      text = TrainerTextResources.getString("LICENSE_VALIDATOR_REGISTER_MESSAGE");
		}
		
		JScrollPane scrollPane = TrainerHtmlUtil.createHTMLMessageScrollPane(text);

		BorderLayout borderLayout = new BorderLayout();
		borderLayout.setVgap(15);

		JPanel panel = new JPanel(borderLayout);
		panel.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
		panel.add(scrollPane, BorderLayout.CENTER);

		
		final JTextField emailField = new JTextField(50);

		emailField.setFont(new Font("Monospaced", Font.PLAIN,
				emailField.getFont().getSize()));

		
		emailField.setMaximumSize(new Dimension(emailField
				.getPreferredSize().width + 2, emailField
				.getPreferredSize().height));
		emailField.setPreferredSize(emailField.getMaximumSize());

		JPanel fieldPanel = new JPanel();
		fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.X_AXIS));
		fieldPanel.add(Box.createHorizontalGlue());
		fieldPanel.add(emailField);
		fieldPanel.add(Box.createHorizontalGlue());

		panel.add(fieldPanel, BorderLayout.SOUTH);

		emailField.addPropertyChangeListener("ancestor",
				new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent evt) {
						Timer timer = new Timer(300, new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										emailField.requestFocusInWindow();
									}
								});
							}
						});
						timer.setRepeats(false);
						timer.start();
					}
				});

		FocusListener fl = new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						emailField.setCaretPosition(0);
					}
				});
				emailField.removeFocusListener(this);
			}
		};
		emailField.addFocusListener(fl);

		//emailField.setFocusLostBehavior(JFormattedTextField.COMMIT);

		String title = TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE");

		String[] options;

    if (necessary) {
      options = new String[2];
      options[0] = TrainerTextResources.getString("LICENSE_VALIDATOR_BUTTON_SUBMIT");
      options[1] = TrainerTextResources.getString("LICENSE_VALIDATOR_BUTTON_DEMO");
    } else {
      options = new String[2];
      options[0] = TrainerTextResources.getString("LICENSE_VALIDATOR_BUTTON_SUBMIT");
      options[1] = TrainerTextResources.getString("LICENSE_VALIDATOR_BUTTON_LATER");
    }
		
		int answer = JOptionPane.showOptionDialog(parent, panel,
				title, necessary ? JOptionPane.YES_OPTION : JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE,
				null, options, options[0]);

		final boolean registrationSuccessful;
		if (answer == JOptionPane.YES_OPTION) {
			String email = emailField.getText();
			
			final boolean ok = sendRegistration(email, parent);
			if (ok) {
				JOptionPane.showMessageDialog(parent, TrainerTextResources.getString("LICENSE_VALIDATOR_THANKS"), 
						TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE"), JOptionPane.INFORMATION_MESSAGE);
				
				registrationSuccessful = true;
			} else {
				JOptionPane.showMessageDialog(parent, TrainerTextResources.getString("LICENSE_VALIDATOR_RETRY"), 
						TrainerTextResources.getString("LICENSE_VALIDATOR_TITLE"), JOptionPane.WARNING_MESSAGE);
				registrationSuccessful = false;
			}
		} else {
		  registrationSuccessful = false;
		}
		
		if (registrationSuccessful) {
		  // Lizenzstatus prüfen und evtl. Meldung ausgeben!
		  registerLicense(parent, nextTask, exitTrainer);
		} else {
		  nextTask.run();
		}
	}

	private static boolean isRegistrationNecessary() {
	  if (isRegisteredOnServer()) {
	    return false;
	  } else if (!KreawiGlobals.dataExists()) {
      return true;
    } else {
      return isNoRegPeriodExceeded();
    }
  }

  private static boolean isNoRegPeriodExceeded() {
    final long firstCheck = LicenseFileHandler.getFirstStart();
    final long difference = System.currentTimeMillis() - firstCheck;
    // thirty days!
    final long period = 1000l * 60 * 60 * 24 * 30;
    final boolean periodExceeded = difference > period;
    return periodExceeded;
  }
	
	public static boolean isFullVersionAllowed() {
	  if (!isRegisteredOnServer()) {
      return !isNoRegPeriodExceeded();
    } else {
      final boolean onlineCheckSuccessful = onlineCheckSuccessful();
      if (isLicenseLocked()) {
        return false;
      } else {
        if (onlineCheckSuccessful && !isLicenseLocked()) {
          return true;
        } else {
          return !hasOnlineCheckFailedTooOften();
        }
      }
    }
  }
	
	private static boolean isRegisteredOnServer() {
    return LicenseFileHandler.getRegUid() != null;
  }
	
  private static boolean onlineCheckSuccessful() {
    if (onlineCheck == null) {
      onlineCheck = checkRegistrationOnline();
    }
    return onlineCheck;
  }
  
  public static boolean isRegistrationOk() {
    return isRegisteredOnServer() && isLicenseOk() && !hasOnlineCheckFailedTooOften();
  }
  
  public static boolean isLicenseLocked() {
    return LicenseFileHandler.getRegLastServerStatus() == SERVER_REGISTRATION_STATUS_LOCKED;
  }
  
  public static boolean isLicenseWarn() {
    return LicenseFileHandler.getRegLastServerStatus() == SERVER_REGISTRATION_STATUS_WARN;
  }
  
  public static boolean isLicenseOk() {
    return LicenseFileHandler.getRegLastServerStatus() == SERVER_REGISTRATION_STATUS_OK
        || LicenseFileHandler.getRegLastServerStatus() == SERVER_REGISTRATION_STATUS_SUPER_LICENSE;
  }
  
  public static boolean hasOnlineCheckFailedTooOften() {
    String failedCount = LicenseFileHandler.getFailedCount();
    int i;
    try {
      i = Integer.parseInt(failedCount);
    } catch (NumberFormatException e) {
      return false;
    }
    return i > 20;
  }



  /**
   * Register license online or show problems with online check of existing registration
   * 
   * @param parent
   * @param checkDataExistence) {

   * @param parent
   * @return true -> Restart of application necessary
   *         false -> No Restart of application necessary
   */
  public void registerLicense(final Component parent, final Runnable nextTask, final Runnable exitTrainer) {
		if (!isRegisteredOnServer()) {
		  askForLicenseRegistration(parent, nextTask, exitTrainer);
    } else if (hasOnlineCheckFailedTooOften()) {
      showFailedTooOften(parent);
		} else if (isLicenseLocked()) {
      final String reason = LicenseFileHandler.getRegLastServerStatusReason();
      showRejectedMessage(parent, reason);
      if (!KreawiGlobals.isDemo()) {
        exitTrainer.run();
      }
    } else if (isLicenseWarn()) {
      showWarnMessage(parent);
      nextTask.run();
		} else {
		  nextTask.run();
		}
	}

  private static boolean checkRegistrationOnline() {
    final String uid = LicenseFileHandler.getRegUid();
    HttpGet check;
    HttpClient client = new DefaultHttpClient();
    try {
      check = new HttpGet(CHECK_URL + "?client_uid=" + URLEncoder.encode(uid, "utf-8"));
      HttpResponse response = client.execute(check);
      HttpEntity entity = response.getEntity();
      if (entity != null) {
        final String answer = IOUtils.toString(entity.getContent());
        JSONObject json = new JSONObject(answer);
        int status = json.getInt("status");
        String reason = "";
        try {
          reason = json.getString("reason");
        } catch (JSONException e) {
          reason = "";
        }
        LicenseFileHandler.setRegLastServerStatus(status, reason);
        resetFailedChecks();
        return true;
      }
    } catch (JSONException e) {
      logger.error("Could not parse answer from registration server! (" + e.toString() + ")");
    } catch (HttpHostConnectException e) {
      logger.error("Online Registration Check failed! (" + e.toString() + ")");
    } catch (ClientProtocolException e) {
      logger.error("Online Registration Check failed! (" + e.toString() + ")");
    } catch (UnknownHostException e) {
      logger.error("Online Registration Check failed! (" + e.toString() + ")");
    } catch (UnsupportedEncodingException e) {
      logger.error("Online Registration Check failed! (" + e.toString() + ")");
    } catch (IOException e) {
      logger.error("Online Registration Check failed! (" + e.toString() + ")");
    } finally {
      client.getConnectionManager().shutdown();
    }
    registerFailedCheck();
    return false;
  }
  
  public boolean deleteRegistration() {
    final String uid = LicenseFileHandler.getRegUid();
    HttpGet check;
    HttpClient client = new DefaultHttpClient();
    try {
      check = new HttpGet(DELETE_URL + "?client_uid=" + URLEncoder.encode(uid, "utf-8"));
      HttpResponse response = client.execute(check);
      HttpEntity entity = response.getEntity();
      if (entity != null) {
        final String answer = IOUtils.toString(entity.getContent());
        JSONObject json = new JSONObject(answer);
        boolean success = json.getBoolean("success");
        if (success) {
          return true;
        } else {
          String reason = "";
          try {
            reason = json.getString("reason");
          } catch (JSONException e) {
            reason = null;
          }
          logger.error("Server returned success:false" + (reason != null?" (Reason: '" + reason + "')":""));
          return false;
        }
      }
    } catch (JSONException e) {
      logger.error("Could not parse answer from registration server! (" + e.toString() + ")");
    } catch (HttpHostConnectException e) {
      logger.error("Deletion of online Registration failed! (" + e.toString() + ")");
    } catch (ClientProtocolException e) {
      logger.error("Deletion of online Registration failed! (" + e.toString() + ")");
    } catch (UnknownHostException e) {
      logger.error("Deletion of online Registration failed! (" + e.toString() + ")");
    } catch (UnsupportedEncodingException e) {
      logger.error("Deletion of online Registration failed! (" + e.toString() + ")");
    } catch (IOException e) {
      logger.error("Deletion of online Registration failed! (" + e.toString() + ")");
    } finally {
      client.getConnectionManager().shutdown();
    }
    return false;
  }

}
