package de.kreawi.trainer.onlinegame.entities;

import org.javalite.activejdbc.Base;
import org.javalite.activejdbc.Model;

import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonSetter;

import javafx.scene.image.Image;

public class OnlineGameUser extends Model {
  
  public static final String DB_ID = "id";
  public static final String DB_EMAIL = "email";
  public static final String DB_MOBILE = "mobile";
  public static final String DB_PROFILE_IMAGE = "profile_image";
  public static final String DB_ACTIVE = "active";
  public static final String DB_USER_NAME = "user_name";
  public static final String DB_DEVICE_TYPE = "device_type";
  public static final String DB_DEVICE_TOKEN = "device_token";
  public static final String DB_OPEN = "open";
  public static final String DB_PAID_USER = "paid_user";
  public static final String DB_FRIEND = "friend";
  public static final String DB_CREATED_AT = "created_at";
  public static final String DB_UPDATED_AT = "updated_at";
  
  private Image profileImageData;

  public OnlineGameUser() {}
  
  public OnlineGameUser(final String id, final String email, final String userName, final String profileImage,
      final String mobile, final boolean active, final String deviceType, final String deviceToken,
      final boolean open, final boolean friend) {
    setId(id);
    setEmail(email);
    setUserName(userName);
    setProfileImage(profileImage);
    setMobile(mobile);
    setActive(active);
    setDeviceType(deviceType);
    setDeviceToken(deviceToken);
    setOpen(open);
    setFriend(friend);
  }
  
  /** Don't use any functions of activejdbc.Model before creating the table! **/ 
  public static void createTableIfNotExists() {
    final String createQuery = "CREATE TABLE IF NOT EXISTS online_game_users (" +
        DB_ID + " VARCHAR(50) PRIMARY KEY, " +  
        DB_EMAIL + " VARCHAR(100) NULL, " +  
        DB_MOBILE + " VARCHAR(50) NULL, " +
        DB_PROFILE_IMAGE + " VARCHAR(255) NULL, " +  
        DB_ACTIVE + " BOOLEAN NOT NULL DEFAULT TRUE, " +  
        DB_USER_NAME + " VARCHAR(100) NULL, " +  
        DB_DEVICE_TYPE + " VARCHAR(50) NULL, " +  
        DB_DEVICE_TOKEN + " VARCHAR(255) NULL, " +  
        DB_OPEN + " BOOLEAN NOT NULL DEFAULT TRUE, " +  
        DB_PAID_USER + " BOOLEAN NOT NULL DEFAULT TRUE, " +  
        DB_FRIEND + " BOOLEAN NOT NULL DEFAULT TRUE, " +
        DB_CREATED_AT + " TIMESTAMP NULL, " +
        DB_UPDATED_AT + " TIMESTAMP NULL " +
        ");";
    Base.exec(createQuery);
  }

  @JsonGetter("id")
  public String getId() {
    return getString(DB_ID);
  }

  @JsonSetter("id")
  public void setId(final String id) {
    set(DB_ID, id);
  }

  @JsonGetter("email")
  public String getEmail() {
    return getString(DB_EMAIL);
  }

  @JsonSetter("email")
  public void setEmail(final String email) {
    set(DB_EMAIL, email);
  }

  @JsonGetter("mobile")
  public String getMobile() {
    return getString(DB_MOBILE);
  }

  @JsonSetter("mobile")
  public void setMobile(final String mobile) {
    set(DB_MOBILE, mobile);
  }

  @JsonGetter("profileImage")
  public String getProfileImage() {
    return getString(DB_PROFILE_IMAGE);
  }

  @JsonSetter("profileImage")
  public void setProfileImage(final String profileImage) {
    set(DB_PROFILE_IMAGE, profileImage);
  }
  
  /** not in DB **/
  public Image getProfileImageData() {
    return this.profileImageData;
  }

  /** not in DB **/
  public void setProfileImageData(final Image profileImageData) {
    this.profileImageData = profileImageData;
  }

  @JsonGetter("active")
  public boolean isActive() {
    return getBoolean(DB_ACTIVE);
  }

  @JsonSetter("active")
  public void setActive(final boolean active) {
    set(DB_ACTIVE, active);
  }

  @JsonGetter("userName")
  public String getUserName() {
    return getString(DB_USER_NAME);
  }

  @JsonSetter("userName")
  public void setUserName(final String userName) {
    set(DB_USER_NAME, userName);
  }

  @JsonGetter("deviceType")
  public String getDeviceType() {
    return getString(DB_DEVICE_TYPE);
  }

  @JsonSetter("deviceType")
  public void setDeviceType(final String deviceType) {
    set(DB_DEVICE_TYPE, deviceType);
  }

  @JsonGetter("deviceToken")
  public String getDeviceToken() {
    return getString(DB_DEVICE_TOKEN);
  }

  @JsonSetter("deviceToken")
  public void setDeviceToken(final String deviceToken) {
    set(DB_DEVICE_TOKEN, deviceToken);
  }

  @JsonGetter("open")
  public boolean isOpen() {
    return getBoolean(DB_OPEN);
  }

  @JsonSetter("open")
  public void setOpen(final boolean open) {
    set(DB_OPEN, open);
  }

  @JsonGetter("paidUser")
  public boolean getPaidUser() {
    return getBoolean(DB_PAID_USER);
  }

  @JsonSetter("paidUser")
  public void setPaidUser(final boolean paidUser) {
    set(DB_PAID_USER, paidUser);
  }
  
  public boolean isFriend() {
    return getBoolean(DB_FRIEND);
  }
  
  public void setFriend(final boolean friend) {
    set(DB_FRIEND, friend);
  }
  
  @Override
  protected void afterSave() {
    super.afterSave();
    // refresh to retrieve default values of columns
    this.refresh();
  }
  
  @Override
  public String toString() {
    return "OnlineGameUser [id=" + getId() + ", email=" + getEmail() + ", userName=" + getUserName() + ", mobile=" + getMobile()
        + ", profileImage=" + getProfileImage() + ", active=" + isActive() + ", deviceType=" + getDeviceType() + ", deviceToken="
        + getDeviceToken() + ", open=" + isOpen() + ", paidUser=" + getPaidUser() + ", friend=" + isFriend() + "]";
  }
  
}
