package de.kreawi.trainer.onlinegame.net;

import java.util.Arrays;
import java.util.HashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.kreawi.trainer.onlinegame.OnlineGameModel;
import de.kreawi.trainer.onlinegame.entities.OnlineGameNotifications;

public class OnlineGameNotificationCommunicator extends OnlineGameCommunicator {

  private static final Logger logger = LoggerFactory.getLogger(OnlineGameNotificationCommunicator.class); 

  public OnlineGameNotificationCommunicator(OnlineGameModel model) {
    super(model);
  }

  public void hasUnreadNotifications(final String userId, final NotificationsChecker onSuccess, final Runnable onFailure) {
    
    final HashMap<String, String> parameters = new HashMap<String, String>();
    parameters.put("userId", userId);
    
    try {

      final NotificationsNumberList list = receiveJsonData("/unreadNotificationsList", parameters, NotificationsNumberList.class);
      
      final boolean hasNewNotifications = list.getQuiz_notifications() > 0;
      onSuccess.run(hasNewNotifications);
      
    } catch (Exception e) {
    
      logger.error("Error while checking for notifications!", e);
      
      onFailure.run();
      
    }
  }
  
  public void getNotifications(final String userId, final NotificationsReceiver onSuccess, final Runnable onFailure) {
    
    final HashMap<String, String> parameters = new HashMap<String, String>();
    parameters.put("userId", userId);
    parameters.put("recordType", RECORD_TYPE_QUIZ);
    
    try {
      
      final OnlineGameNotifications result = receiveJsonData("/getUnreadNotifications", parameters, OnlineGameNotifications.class);
    
      onSuccess.run(Arrays.asList(result.getCurrent()), Arrays.asList(result.getRecent()));
    
    } catch (Exception e) {
    
      logger.error("Error while retrieving notifications!", e);
      
      onFailure.run();
      
    }
    
  }
  
  public void updateQuizNotificationStatus(String userId, String notificationId, final Runnable onSuccess,
      final Runnable onFailure) {
    
    final OnlineGameUpdateNotificationData data = new OnlineGameUpdateNotificationData(userId, notificationId);
    
    try {
      
      submitJsonData("/updateQuizNotificationStatus", (new ObjectMapper()).writeValueAsString(data), null);
      
    } catch (Exception e) {
      
      logger.error("Error while retrieving notifications!", e);
      
      onFailure.run();
      
    }
    
  }
  
  private class NotificationsNumberList {
    
    public int challenge_notifications;
    
    public int quiz_notifications;
    
    public int getChallenge_notifications() {
      return challenge_notifications;
    }
    public void setChallenge_notifications(int challenge_notifications) {
      this.challenge_notifications = challenge_notifications;
    }
    public int getQuiz_notifications() {
      return quiz_notifications;
    }
    public void setQuiz_notifications(int quiz_notifications) {
      this.quiz_notifications = quiz_notifications;
    }
    
  }
  
  private static class OnlineGameUpdateNotificationData {
    
    private String userId;
    
    private String notificationId;

    private int read;

    public String getUserId() {
      return userId;
    }

    public void setUserId(String userId) {
      this.userId = userId;
    }

    public String getNotificationId() {
      return notificationId;
    }

    public void setNotificationId(String notificationId) {
      this.notificationId = notificationId;
    }

    public int getRead() {
      return read;
    }

    public void setRead(int read) {
      this.read = read;
    }

    public OnlineGameUpdateNotificationData(final String userId, final String notificationId) {
      this.userId = userId;
      this.notificationId = notificationId;
      this.read = 1;
    }
    
  }
  
  public interface NotificationsChecker {
    public void run(final boolean hasNewNotifications);
  }

}
