package de.kreawi.trainer.tutorial;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.dialogs.MainDialogFx;
import de.kreawi.trainer.util.OSDetector;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.util.config.AbstractConfigComponent;
import de.martinknauer.util.config.Config;
import de.martinknauer.util.config.ConfigComponentHandler;

public class TutorialHandler {
  
  final private static int DIALOG_WIDTH = 900;
  final private static int DIALOG_HEIGHT = 600;
  final private static int FONT_SIZE = 16;  // Font size in pt
  
  private static List<File> tipList;
  private static Iterator<File> tipListIterator;
  
  private static final Logger logger = LoggerFactory.getLogger(TutorialHandler.class); 

  public static void showRandomTip(final Config settings, final Component parent) {
    
    final ConfigComponentHandler configCompHandler = new ConfigComponentHandler();
    final JPanel showOnStartupConfigPanel = buildShowOnStartupConfigPanel(settings, configCompHandler);

    final JSplitPane panel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

    // int width = TrainerFrame.SMALL_DIALOG_SIZE.width;
    // int height = TrainerFrame.SMALL_DIALOG_SIZE.height - 270;
    //
    panel.setPreferredSize(new Dimension(DIALOG_WIDTH, DIALOG_HEIGHT));
    panel.setMinimumSize(panel.getPreferredSize());

    panel.setBorder(BorderFactory.createEmptyBorder());
    panel.setDividerSize(0);

    final String fontFamily = (new JLabel()).getFont().getFamily();
    final String css = "body {" +
          "text-align:left;" + 
          "font: " + FONT_SIZE + "pt '" + fontFamily + "'"  + ";" +
          "padding: 15px;" + 
        "} " +
        ".hr {" +
        "width: 100%;" +
        "height: 0;" +
        "border-bottom: 1px solid #aaaaaa;" +
        "font-size: 0;" +
        "margin: 10px 0;" +
      "} " +
      "h2 {" +
        "font-size: " + (FONT_SIZE + 2) + "pt; " +
        "text-transform: uppercase" +
      "} ";
    final String tipHtml = TrainerHtmlUtil.wrapInHtml(getNextTip(), css);
    if (KreawiGlobals.isDevMode()) {
      logger.info(tipHtml);
    }
    final JScrollPane tipPane = TrainerHtmlUtil.createHTMLMessageScrollPane(tipHtml);
    
    panel.setTopComponent(tipPane);
    panel.setBottomComponent(showOnStartupConfigPanel);
    
    final String title = TrainerTextResources.getString("TUTORIAL_TITLE");
    final ImageIcon icon = new ImageIcon(TrainerMain.class
        .getResource(MainDialogFx.LOGO_PATH));
    final String buttonNext = TrainerTextResources.getString("TUTORIAL_BUTTON_NEXT");
    final String buttonClose = TrainerTextResources.getString("TUTORIAL_BUTTON_CLOSE");
    final String[] buttons = new String[] { buttonNext, buttonClose };
    
    tipPane.setPreferredSize(new Dimension(DIALOG_WIDTH - icon.getIconWidth(),
        (int) (DIALOG_HEIGHT - showOnStartupConfigPanel.getPreferredSize().getHeight())));
    
    final int choice = JOptionPane.showOptionDialog(parent, panel, title, JOptionPane.YES_NO_OPTION,
        JOptionPane.PLAIN_MESSAGE, icon, buttons, buttonNext);
    if (choice == JOptionPane.YES_OPTION) {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          showRandomTip(settings, parent);
        }
      });
    }
    configCompHandler.commit();
  }

  private static String getNextTip() {
    if (tipList == null) {
      tipList = new ArrayList<File>(FileUtils.listFiles(getBaseDir(), new String[] {"html",  "htm"}, false));
      Collections.sort(tipList);
      tipListIterator = tipList.iterator();
    }
    if (!tipListIterator.hasNext()) {
      tipListIterator = tipList.iterator();
    }
    try {
      final String nextTipp = FileUtils.readFileToString(tipListIterator.next());
      return TrainerHtmlUtil.replaceRelativePathOfImages(nextTipp, getBaseDir());
    } catch (IOException e) {
      logger.error("Fehler beim Einlesen des nächsten Tipps!", e);
      return "<html><body><h3>Fehler beim Einlesen des Tipps!</h3></body></html>";
    }
  }

  private static File getBaseDir() {
    final File workDir = new File(System.getProperty("user.dir"));
    return new File(new File(new File(workDir, KreawiGlobals.RESOURCES_DIR_NAME), KreawiGlobals.TUTORIAL_DIR_NAME), 
        OSDetector.getOs().isMac()?"mac":"win");
  }

  private static JPanel buildShowOnStartupConfigPanel(final Config settings,
      final ConfigComponentHandler configCompHandler) {
    final String settingsKey = TrainerSettingsConstants.KEY_SHOW_TUTORIAL_ON_STARTUP;
    final AbstractConfigComponent<JCheckBox> showTutorialOnStartupBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings, settingsKey) {
      @Override
      public String getComponentValue() {
        return (getComponent().isSelected() ? "true" : "false");
      }
      @Override
      protected void displayConfigValue() {
        boolean oldValue = super.config.getBool(super.key);
        getComponent().setSelected(oldValue);
      }
    };
    configCompHandler.addConfigComponent(showTutorialOnStartupBox);
    FormLayout layout = new FormLayout(
        "right:pref, 3dlu, pref,",// column
        ""); // add rows dynamically
    DefaultFormBuilder builder = new DefaultFormBuilder(layout);
    builder.setDefaultDialogBorder();
    builder.setLeftToRight(true);

    builder.append(TrainerTextResources.getString("TUTORIAL_SHOW_ON_STARTUP_LABEL"), showTutorialOnStartupBox.getComponent());
    builder.nextLine();
    final JPanel showOnStartupConfigPanel = builder.getPanel();
    return showOnStartupConfigPanel;
  }

}