package de.kreawi.trainer.util;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.File;
import java.net.MalformedURLException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.border.EtchedBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Document.OutputSettings;
import org.jsoup.safety.Whitelist;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.component.Factory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.util.swingx.DesktopApi;
import de.martinknauer.util.Util;
import edu.stanford.ejalbert.BrowserLauncher;

public class TrainerHtmlUtil {

	private static HyperlinkListener hyperLinkListener;
	
	private static final Logger logger = LoggerFactory.getLogger(TrainerHtmlUtil.class); 

  public static JScrollPane createHTMLMessageScrollPane(String text) {
    JTextPane textPane = createHtmlPane(text);
    
    JScrollPane scrollPane = Factory.createStrippedScrollPane(textPane);
    scrollPane.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
    
    if (OSDetector.getOs().isMac()) {
      /* mac os */
      scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    }
    
    textPane.setSize(450, Integer.MAX_VALUE);
    textPane.validate();
    
    textPane.setPreferredSize(new Dimension(textPane.getSize().width, textPane.getPreferredSize().height));
    textPane.validate();
    scrollPane.setPreferredSize(
        new Dimension(textPane.getSize().width + 4, Math.min(textPane.getPreferredSize().height + 4, 600)));
    
    if (scrollPane.getSize().height >= 600) {
      if (!OSDetector.getOs().isMac()) {
        /* non mac os */
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      }
    }
    
    scrollPane.setOpaque(true);
    scrollPane.setBackground(Color.WHITE);
    scrollPane.getViewport().setOpaque(false);
    
    return scrollPane;
  }
  
  public static JTextPane createHtmlPane(final String text) {
    JTextPane textPane = new JTextPane() {
      public void paint(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        try {
          g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, Util.getAATextRenderingHintValue());
        } catch (ClassNotFoundException e) {
          logger.info("Text-Anti-Aliasing is not supported!");
        }
        super.paint(g2d);
      }
      
      @Override
      public void setText(final String t) {
        final String html;
        if (!t.toLowerCase().startsWith("<html")) {
          final String css = TrainerHtmlUtil.getDefaultCss(true);
          html = TrainerHtmlUtil.wrapInHtml(t, css);
        } else {
          html = t;
        }
        if (KreawiGlobals.isDevMode()) {
          logger.info("html: " + html);
        }
        super.setText(html);
      }
    };
    
    textPane.setEditable(false);
    textPane.setContentType("text/html");
    textPane.addHyperlinkListener(TrainerHtmlUtil.getDefaultHyperlinkListener());
    
    textPane.setText(text);
    
    textPane.setCaretPosition(0);
    
    textPane.setOpaque(false);
    return textPane;
  }
  
	/**
	 * Convenience method to easily access a HyperlinkListener which opens a
	 * hyperlink in the user's default web browser using java.awt.Desktop, or,
	 * if java.awt.Desktop is not supported on the current platform,
	 * BrowserLauncher2.
	 * 
	 * @return a HyperlinkListener
	 */
	public static synchronized HyperlinkListener getDefaultHyperlinkListener() {

		if (TrainerHtmlUtil.hyperLinkListener == null) {

			TrainerHtmlUtil.hyperLinkListener = new HyperlinkListener() {
				public void hyperlinkUpdate(HyperlinkEvent e) {

					if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

						if (e.getSource() instanceof JEditorPane) {

							final JEditorPane pane = (JEditorPane) e
									.getSource();

							if (e instanceof HTMLFrameHyperlinkEvent) {
								HTMLFrameHyperlinkEvent evt = (HTMLFrameHyperlinkEvent) e;
								HTMLDocument doc = (HTMLDocument) pane.getDocument();
								doc.processHTMLFrameHyperlinkEvent(evt);
							} else {

							  final Cursor storedCursor = pane.getCursor();

								try {
								  pane.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

									 try {
									   logger.info("Öffne URL: " + e.getURL());
									   DesktopApi.browse(e.getURL().toURI());
									 } catch (Exception ex) {
                    logger.error("Desktop.browse hat nicht funktioniert! (url: " + e.getURL().toString() + ")", ex);
									   try {
									     (new BrowserLauncher()).openURLinBrowser(e.getURL().toString());
									   } catch (Exception ex2) {
									     logger.error("Konnte URL nicht öffnen! (url: " + e.getURL().toString() + ")", ex2);
									   }
									 }
								} finally {
									Timer timer = new Timer();
									timer.schedule(new TimerTask() {
										@Override
										public void run() {
											pane.setCursor(storedCursor);
										}
									}, 1000);
								}
							}
						}
					}

				}
			};

		}

		return TrainerHtmlUtil.hyperLinkListener;
	}

	public static String addLineBreaks(String text, int maxCharsInLine) {
		String[] splits = text.split(" ");
		String output = "";
		String currentLine = "";
		String temp;

		for (String current : splits) {

			if (current == "") {
				continue;
			}

			if (currentLine == "") {
				currentLine = current;
				continue;
			}

			temp = currentLine + " " + current;

			if (temp.lastIndexOf("<") <= temp.lastIndexOf(">")
					&& temp.length() - temp.lastIndexOf(">")
							- temp.lastIndexOf("<") > maxCharsInLine) {
				if (output == "") {
					output += currentLine;
				} else {
					output += "<br>" + currentLine;
				}
				currentLine = current;
			} else {
				currentLine = temp;
			}

		}

		if (output == "") {
			output += currentLine;
		} else {
			output += "<br>" + currentLine;
		}

		return output;
	}
	
	public static String removeHtmlTags(String html) {
	  Document jsoupDoc = Jsoup.parse(html);

	  //set pretty print to false, so \n is not removed
    jsoupDoc.outputSettings(new OutputSettings().prettyPrint(false));
    
    //select all <br> tags and append \n after that
    jsoupDoc.select("br").after("\\n");
    
    //select all <p> tags and prepend \n before that
    jsoupDoc.select("p").before("\\n");
            
    //get the HTML from the document, and retaining original new lines
    String str = jsoupDoc.html().replaceAll("\\\\n", "\n");
    
    return Jsoup.clean(str, "", Whitelist.none(), new OutputSettings().prettyPrint(false));
	}
	
  public static String replaceRelativePathOfImages(String text, final File baseDir) {
    final String regex = "(<img\\s+(?:[^>]*?\\s+)?src=\")(?!file:/)(file:)?([^/][^\"]*)(\"[^>]*>)";
    Matcher matcher = Pattern.compile(regex).matcher(text);
    
    while (matcher.find()) {

      try {
        final String urlPath = matcher.group(3);
        
        final File file = new File(baseDir, urlPath);
        String newUrl = null;
        if (file.exists()) {
          newUrl = file.toURI().toURL().toString();
          if (KreawiGlobals.isDevMode()) {
            logger.info("relative url '" + urlPath + "' changed to '" + newUrl + "'!");
          }
        } 
        if (!file.exists() && newUrl == null || !newUrl.startsWith("file:/")) {
          logger.info("Image not found: " + urlPath);
          newUrl = new File(new File(System.getProperty("user.dir"),
              KreawiGlobals.DEMO_DIR_NAME),
              KreawiGlobals.ERROR_IMAGE_FILE_NAME).toURI().toURL().toString();
        }
        
        final String replacement = matcher.group(1) + newUrl + matcher.group(4);
        
        logger.info("Replace image: " + matcher.group());
        logger.info("  with: " + replacement);
        
        text = matcher.replaceFirst(replacement);
        
        matcher = Pattern.compile(regex).matcher(text);
      } catch (MalformedURLException e) {
        logger.error("Fehler beim Ersetzen der Bild-URL!", e);
      }
    }
    return text;
  }
  
  public static String getDefaultCss(final boolean justify) {
    final Font font = (new JLabel()).getFont();
    return getCssWithFont(justify, font);
  }
  
  public static String getCssWithFont(final boolean justify, final Font font) {
    return "body {" + 
        "text-align:" + (justify?"justify":"left") + ";" + 
        "font: " + font.getSize() + "pt '" + font.getFamily() + "'"  + ";" +
      "} ";
  }
  
  public static String wrapInHtml(final String text, final String css) {
    final String styleTag;
    if (css != null && css.length() > 0) {
      styleTag = "<style type=\"text/css\">" + css + "</style>";
    } else {
      styleTag = "";
    }
    return "<html><head><meta charset=\"utf-8\">" + styleTag + "</head><body>" + text + "</body></html>";
  }
  
  public static String getHTMLColorString(final Color color) {
    final String red = Integer.toHexString(color.getRed());
    final String green = Integer.toHexString(color.getGreen());
    final String blue = Integer.toHexString(color.getBlue());
    
    return "#" + (red.length() == 1 ? "0" + red : red) + (green.length() == 1 ? "0" + green : green)
        + (blue.length() == 1 ? "0" + blue : blue);
  }

}
