package de.kreawi.trainer.util;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.SwingWorker;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.util.swingx.CursorToolkit;

public class TrainerLoadSplash extends JFrame {
  
  private JLayeredPane lpane = new JLayeredPane();
  private final FadeInPanel panelIcon;
  private JPanel panelLabel;
  private final Dimension SPLASH_SIZE = new Dimension(500, 350);
  private SwingWorker m_worker;
  
  private static final Logger logger = LoggerFactory.getLogger(TrainerLoadSplash.class); 

  public TrainerLoadSplash(final long initialExpectedDuration, final String trainerYear, SwingWorker worker) {
    setUndecorated(true);
    setTitle(TrainerTextResources.getStringWithSubstitution("APPLICATION_TITLE_LONG", KreawiGlobals.CURRENT_YEAR));
    
    setIcon();
    setPreferredSize(SPLASH_SIZE);
    setLayout(new BorderLayout());
    add(lpane, BorderLayout.CENTER);
    lpane.setBounds(0, 0, SPLASH_SIZE.width, SPLASH_SIZE.height);
    final URL imageUrl = getClass().getResource("/de/kreawi/trainer/resources/images/logo_big.png");
    panelLabel = getLabelPanel(trainerYear);  // MainDialog.buildAppTitlePanel(false);
    panelLabel.setBounds(0, 0, SPLASH_SIZE.width, SPLASH_SIZE.height);
    panelIcon = new FadeInPanel(imageUrl, initialExpectedDuration);
    panelIcon.setBounds(SPLASH_SIZE.width / 4, SPLASH_SIZE.height / 4, SPLASH_SIZE.width / 2, SPLASH_SIZE.height / 2);
    lpane.add(panelLabel, new Integer(0), 0);
    lpane.add(panelIcon, new Integer(1), 0);
    GraphicsDevice[] monitors = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
    Dimension dim = new Dimension(monitors[0].getDisplayMode().getWidth(), monitors[0].getDisplayMode().getHeight());
    final Rectangle monitorBounds = monitors[0].getDefaultConfiguration().getBounds();
    final int xLocation = dim.width/2-SPLASH_SIZE.width/2;
    final int yLocation = dim.height/2-SPLASH_SIZE.height/2 + monitorBounds.y;
    this.setLocation(xLocation, yLocation);
    pack();
    final TrainerLoadSplash owner = this;
    addWindowListener(new WindowAdapter() {
      public void windowOpened(WindowEvent e) {
        CursorToolkit.startWaitCursor(owner.getRootPane());
      }
      
      public void windowClosed(WindowEvent e) {
        CursorToolkit.stopWaitCursor(owner.getRootPane());
      }
    });
    if (worker != null) {
      m_worker = worker;
      initBackgroundWorker();
    }
  }

  private void initBackgroundWorker() {
    m_worker.addPropertyChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        if ("state".equals(evt.getPropertyName()) && SwingWorker.StateValue.DONE == evt.getNewValue()) {
          setVisible(false);
          dispose();
        }
      } 
    });
  }
  
  public void setVisible(final boolean visible) {
    super.setVisible(visible);
    panelIcon.setVisible(visible);
    if (visible) {
      if (m_worker != null && !m_worker.isDone()) {
        m_worker.execute();
      }
    }
  }
  
  private void setIcon() {
    String iconPath = KreawiGlobals.APP_IMAGES_DIR_NAME + "/trainer_icon.gif";
    
    URL url = TrainerMain.class.getResource(iconPath);
    setIconImage(new ImageIcon(url).getImage());
  }

  private JPanel getLabelPanel(final String trainerYear) {
    final String htmlText = "<html><body style=\"margin:10px\"><p style=\"font:20px bolder black;margin:0\">" + 
    "kreawi <span style=\"font-style:italic;\">Prüfungstrainer</span> " + trainerYear + "</p>" + 
    "<p style=\"font:10px normal black;margin:0\">&copy; 2007-" + trainerYear + " Kreativität & Wissen Verlag</p>" + 
    "</body></html>";
    final JLabel labelPanel = new JLabel(htmlText);
    labelPanel.setOpaque(false);
    final JPanel layoutPanel = new JPanel(new BorderLayout());
    layoutPanel.add(labelPanel, BorderLayout.SOUTH);
    layoutPanel.setOpaque(false);
    return layoutPanel;
  }
  
}
