package de.kreawi.trainer.util.swingx;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.jdesktop.beans.AbstractBean;
import org.jdesktop.swingx.JXTable;
import org.jdesktop.swingx.decorator.FilterPipeline;
import org.jdesktop.swingx.decorator.PatternFilter;

public abstract class AbstractStringFiltering extends AbstractBean {

	private PatternFilter searchFilter;
	private String filterString;
	protected JXTable table;

	public AbstractStringFiltering(JXTable table) {
		super();
		this.table = table;
		table.getModel().addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent e) {
				updateSearchFilter();
			}
			
		});
	}

	public boolean isFilteringByString() {
		return !isEmpty(getFilterString());
	}

	private boolean isEmpty(String filterString) {
		return filterString == null || filterString.length() == 0;
	}

	/**
	 * @param filterString
	 *            the filterString to set
	 */
	public void setFilterString(String filterString) {
		String oldValue = getFilterString(); 
	
		this.filterString = filterString;
		updateSearchFilter();

		firePropertyChange("filterString", oldValue, getFilterString());
	}

	/**
	 * @return the filterString
	 */
	public String getFilterString() {
		return filterString;
	}

	private void updateSearchFilter() {
		if ((filterString != null) && (filterString.length() > 0)) {
			searchFilter = createSearchFilter(filterString + ".*");
		} else {
			searchFilter = null;
		}
		updateFilters();
	}

	private void updateFilters() {
		if (searchFilter != null) {
			FilterPipeline pipeline = new FilterPipeline(searchFilter);
			table.setFilters(pipeline);
		} else {
			table.setFilters(null);
		}
	}
	
	public int getViewSize() {
		if (this.searchFilter != null && isFilteringByString()) {
			return this.searchFilter.getSize();
		} else {
			return this.table.getModel().getRowCount();
		}
	}
	
	protected abstract PatternFilter createSearchFilter(final String filterString);

}