package de.kreawi.trainer.util.swingx;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingWorker;
import javax.swing.SwingWorker.StateValue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.TrainerStorage;
import de.kreawi.trainer.util.TrainerTextResources;

public class PleaseWaitDialog extends JDialog {
  private static final int DIALOG_WIDTH = 300;
  private static final int DIALOG_HEIGHT = 200;

  private JLabel m_messageLabel;
  private SwingWorker m_worker;
  private JProgressBar progressBar;

  private static final Logger logger = LoggerFactory.getLogger(PleaseWaitDialog.class); 

  public PleaseWaitDialog(final JFrame owner, final String message, final SwingWorker worker, final boolean indeterminate) {
    super(owner, false);
    initWindow(owner, message, indeterminate);
    if (worker != null) {
      m_worker = worker;
      initBackgroundWorker();
    }
  }

  public PleaseWaitDialog(final JFrame owner, final String message, final SwingWorker worker) {
    this(owner, message, worker, true);
  }
  
  public PleaseWaitDialog(JFrame owner) {
    this(owner, null, null, true);
  }

  private void initBackgroundWorker() {
    m_worker.addPropertyChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent evt) {
        if ("state".equals(evt.getPropertyName()) && SwingWorker.StateValue.DONE == evt.getNewValue()) {
          setVisible(false);
          dispose();
        } else if ("progress".equals(evt.getPropertyName())) {
          if (progressBar != null) {
            if (progressBar.isIndeterminate()) {
              progressBar.setIndeterminate(false);
              progressBar.setMaximum(100);
            }
            progressBar.setValue(((Integer) evt.getNewValue()).intValue());
          }
        }
      } 
    });
  }

  private void initWindow(final JFrame owner, final String message, final boolean indeterminate) {
    setSize(DIALOG_WIDTH, DIALOG_HEIGHT);
    setTitle(TrainerTextResources.getString("APPLICATION_TITLE_SHORT"));
    progressBar = new JProgressBar();
    progressBar.setIndeterminate(indeterminate);
    if (!indeterminate) {
      progressBar.setMaximum(100);
    }
    progressBar.setSize(getWidth() - 40, 30);
    final JPanel centeringPanel = new JPanel(new GridBagLayout());
    centeringPanel.add(progressBar);
    final BorderLayout borderLayout = new BorderLayout();
    JPanel panel = new JPanel(borderLayout);
    panel.add(centeringPanel, BorderLayout.CENTER);
    final String label = (message == null) ? TrainerTextResources
        .getString("PLEASE_WAIT_DIALOG_DEFAULT_TEXT") : message;
    m_messageLabel = new JLabel(label, JLabel.CENTER);
    m_messageLabel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
    panel.add(m_messageLabel, BorderLayout.PAGE_START);
    Action cancelAction = new AbstractAction() {
      public void actionPerformed(ActionEvent e) {
        cancel();
      }
    };
    final JButton cancelButton = new JButton(cancelAction);
    cancelButton.setText(TrainerTextResources.getString("PLEASE_WAIT_DIALOG_CANCEL_BUTTON"));
    final JPanel centeringPanel2 = new JPanel(new GridBagLayout());
    centeringPanel2.add(cancelButton);
    panel.add(centeringPanel2, BorderLayout.PAGE_END);
    getRootPane().setDefaultButton(cancelButton);
    panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
    add(panel);
    setLocationRelativeTo(owner);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    setUndecorated(true);
    addWindowListener(new WindowAdapter() {
      public void windowOpened(WindowEvent e) {
        CursorToolkit.startWaitCursor(owner.getRootPane());
      }
      
      public void windowClosed(WindowEvent e) {
        CursorToolkit.stopWaitCursor(owner.getRootPane());
      }
    });
  }
  
  @Override
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    if (visible) {
      if (m_worker != null) {
        m_worker.execute();
      }
    }
  }
  
  public void cancel() {
    if (m_worker != null && m_worker.getState() != StateValue.DONE) {
      m_worker.cancel(true);
    }
  }

  public void setMessage(String message) {
    m_messageLabel.setText(message);
  }
  
}
