package de.kreawi.trainer.viewer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import javazoom.jlgui.basicplayer.BasicController;
import javazoom.jlgui.basicplayer.BasicPlayer;
import javazoom.jlgui.basicplayer.BasicPlayerEvent;
import javazoom.jlgui.basicplayer.BasicPlayerException;
import javazoom.jlgui.basicplayer.BasicPlayerListener;

public class AudioViewer extends ViewerWindow {

	private static final long serialVersionUID = -4226444549161649629L;
	private BasicPlayer player;

	private static final Logger logger = LoggerFactory.getLogger(AudioViewer.class); 

	public AudioViewer(Component owner, String title, URL url) throws ViewerException {
		super(owner, title);

		try {
			final File audioFile = new File(url.toURI());

			this.player = new BasicPlayer();

			JPanel gradientPanel = new KreawiInternalFrame("",
					new BorderLayout()).getContent();

			String playIconUrl = KreawiGlobals.APP_IMAGES_DIR_NAME + "/play12_grey.gif";
			final ImageIcon playIcon = new ImageIcon(KreawiGlobals.class
					.getResource(playIconUrl));
			
			String playIconUrlRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/play12_grey_light.gif";
			final ImageIcon playIconRollover = new ImageIcon(KreawiGlobals.class
					.getResource(playIconUrlRollover));
			
			String playIconUrlPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/play12_grey_dark.gif";
			final ImageIcon playIconPressed = new ImageIcon(KreawiGlobals.class
					.getResource(playIconUrlPressed));

			String pauseIconUrl = KreawiGlobals.APP_IMAGES_DIR_NAME + "/pause12_grey.gif";
			final ImageIcon pauseIcon = new ImageIcon(KreawiGlobals.class
					.getResource(pauseIconUrl));
			
			String pauseIconUrlRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/pause12_grey_light.gif";
			final ImageIcon pauseIconRollover = new ImageIcon(KreawiGlobals.class
					.getResource(pauseIconUrlRollover));
			
			String pauseIconUrlPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/pause12_grey_dark.gif";
			final ImageIcon pauseIconPressed = new ImageIcon(KreawiGlobals.class
					.getResource(pauseIconUrlPressed));

			final JButton playButton = createRolloverButton(pauseIcon,
					pauseIconRollover, pauseIconPressed, new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							try {
								if (player.getStatus() == BasicPlayer.PLAYING) {
									player.pause();
								} else if (player.getStatus() == BasicPlayer.PAUSED) {
									player.resume();
								} else {
									player.play();
								}
							} catch (BasicPlayerException bpe) {
                logger.error("Exception occurred during audio playback!", bpe);
							}
						}
					});

			String stopIconUrl = KreawiGlobals.APP_IMAGES_DIR_NAME + "/stop12_grey.gif";
			ImageIcon stopIcon = new ImageIcon(KreawiGlobals.class
					.getResource(stopIconUrl));
			
			String stopIconUrlRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/stop12_grey_light.gif";
			ImageIcon stopIconRollover = new ImageIcon(KreawiGlobals.class
					.getResource(stopIconUrlRollover));
			
			String stopIconUrlPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/stop12_grey_dark.gif";
			ImageIcon stopIconPressed = new ImageIcon(KreawiGlobals.class
					.getResource(stopIconUrlPressed));

			final JSlider slider = new JSlider(0, (int) audioFile.length(), 0);
			slider.setOpaque(false);
			slider.addChangeListener(new ChangeListener() {
				private boolean stopped;

				public void stateChanged(ChangeEvent e) {
					if (slider.getValueIsAdjusting()) {
						try {
							if (!(player.getStatus() == BasicPlayer.STOPPED)) {
								player.stop();
								stopped = true;
							}
						} catch (BasicPlayerException bpe) {
              logger.error("Exception occurred during audio playback!", bpe);
						}
					} else if (player.getStatus() == BasicPlayer.STOPPED) {
						try {
							player.seek(slider.getValue());
							if (stopped) {
								player.play();
								stopped = false;
							}
						} catch (BasicPlayerException bpe) {
              logger.error("Exception occurred during audio playback!", bpe);
						}
					}
				}
			});
			
			JButton stopButton = createRolloverButton(stopIcon,
					stopIconRollover, stopIconPressed, new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							try {
								player.stop();
								slider.setValue(0);
							} catch (BasicPlayerException bpe) {
                logger.error("Exception occurred during audio playback!", bpe);
							}
						}
					});
			
			
			JPanel leftGluePanel = new JPanel();
			leftGluePanel.setBorder(BorderFactory.createEmptyBorder());
			leftGluePanel.setPreferredSize(new Dimension(2, 2));
			leftGluePanel.setMaximumSize(leftGluePanel.getPreferredSize());
			leftGluePanel.setOpaque(false);
			
			JPanel rightGluePanel = new JPanel();
			rightGluePanel.setBorder(BorderFactory.createEmptyBorder());
			rightGluePanel.setPreferredSize(new Dimension(2, 2));
			rightGluePanel.setMaximumSize(rightGluePanel.getPreferredSize());
			rightGluePanel.setOpaque(false);

			JToolBar toolbar = new JToolBar();
			toolbar.setOpaque(false);
			toolbar.setBorder(BorderFactory.createEmptyBorder());
			toolbar.setFloatable(false);
			toolbar.setRollover(true);
			
			toolbar.add(leftGluePanel);
			toolbar.add(playButton);
			toolbar.add(stopButton);
			toolbar.add(slider);
			toolbar.add(rightGluePanel);

			gradientPanel.add(toolbar, BorderLayout.CENTER);

			sif.add(gradientPanel);

			addWindowListener(new WindowAdapter() {
				public void windowClosed(WindowEvent e) {
					try {
						/*
						 * pausing and stopping in invokeLater() is done, because
						 * just calling stop() crashed the whole app from time
						 * to time
						 */
					  logger.info("Pausing player...");
						player.pause();
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								try {
								  logger.info("Stopping player...");
									player.stop();
								} catch (BasicPlayerException bpe) {
                  logger.error("Exception occurred during audio playback!", bpe);
								}
							}
						});
					} catch (BasicPlayerException bpe) {
					  logger.error("Exception occurred during audio playback!", bpe);
					}
				}
			});

			player.addBasicPlayerListener(new BasicPlayerListener() {

				@SuppressWarnings("unchecked")
				public void opened(Object stream, Map properties) {
					/* do nothing */
				}

				@SuppressWarnings("unchecked")
				public void progress(int bytesread, long microseconds,
						byte[] pcmdata, Map properties) {
					slider.setValue(bytesread);
				}

				public void setController(BasicController controller) {
					/* do nothing */
				}

				public void stateUpdated(BasicPlayerEvent event) {
					
					if (KreawiGlobals.isDevMode()) {
						switch (event.getCode()) {
						case BasicPlayerEvent.UNKNOWN:
						  logger.info("UNKNOWN");
							break;
						case BasicPlayerEvent.PAUSED:
						  logger.info("PAUSED");
							break;
						case BasicPlayerEvent.PLAYING:
						  logger.info("PLAYING");
							break;
						case BasicPlayerEvent.STOPPED:
						  logger.info("STOPPED");
							break;
						case BasicPlayerEvent.SEEKED:
						  logger.info("SEEKED");
							break;
						case BasicPlayerEvent.SEEKING:
						  logger.info("SEEKING");
							break;
						case BasicPlayerEvent.OPENED:
						  logger.info("OPENED");
							break;
						case BasicPlayerEvent.OPENING:
						  logger.info("OPENING");
							break;
						case BasicPlayerEvent.RESUMED:
						  logger.info("RESUMED");
							break;
						case BasicPlayerEvent.EOM:
						  logger.info("EOM");
							break;
						default:
						  logger.info("Unknown BasicPlayerEvent: " + event.getCode());
						}
					}
					
					if (event.getCode() == BasicPlayerEvent.PLAYING
							|| event.getCode() == BasicPlayerEvent.RESUMED) {
						playButton.setIcon(pauseIcon);
						playButton.setRolloverIcon(pauseIconRollover);
						playButton.setPressedIcon(pauseIconPressed);
					} else if (event.getCode() == BasicPlayerEvent.PAUSED
							|| event.getCode() == BasicPlayerEvent.STOPPED) {
						playButton.setIcon(playIcon);
						playButton.setRolloverIcon(playIconRollover);
						playButton.setPressedIcon(playIconPressed);
					}
				}

			});

			try {
				player.open(audioFile);
				player.play();
			} catch (BasicPlayerException e) {
				String message = "Exception occurred during audio playback!";
				logger.error(message, e);
				throw new ViewerException(message);
			}

			pack();
			setLocationRelativeTo(owner);
			setVisible(true);
		} catch (URISyntaxException e) {
			String message = "Unable to open file " + url
					+ " due to an URISyntaxException";
			logger.error(message, e);
			throw new ViewerException(message);
		}
	}
	
	private JButton createRolloverButton(Icon icon,
			Icon rolloverIcon, Icon pressedIcon, ActionListener actionListener) {

		final JButton button = new JButton(icon);
		button.setOpaque(false);
		button.setFocusPainted(false);
		button.addActionListener(actionListener);
		button.setRolloverIcon(rolloverIcon);
		button.setPressedIcon(pressedIcon);
		button.setRolloverEnabled(true);
		button.setBorderPainted(false);
		button.setContentAreaFilled(false);

		return button;
	}

	public static boolean isAudioUrl(URL url) {
		if (url == null) {
			return false;
		}

		String fileName = url.getFile().toLowerCase();

		fileName = fileName.toLowerCase();
		return fileName.endsWith(".mp3");
	}

}
