package de.kreawi.trainer.viewer;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.JWindow;
import javax.swing.SwingUtilities;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.martinknauer.util.Util;

public class ViewerWindow extends JWindow {

	private static final long serialVersionUID = 4752508932914123402L;

	protected SimpleInternalFrame sif;

	private MouseEvent pressed;
	
//	private boolean  disposalScheduled = false;

	private static final Logger logger = LoggerFactory.getLogger(ViewerWindow.class); 

	public ViewerWindow(final Component owner, String title) throws ViewerException {
		super(Util.getOwnerWindow(owner));

		owner.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals("ancestor")
						|| evt.getPropertyName().equals("document")) {
//					disposalScheduled = true;
					setVisible(false);
					dispose();
				} else {
					logger.info(owner.getClass()
							+ " PropertyChangeEvent for property \""
							+ evt.getPropertyName() + "\"");
				}
			}
		});
		
		/*
		 * log window events. this is desperate try to resolve a strange issue
		 * on a clients system which could not be reproduced
		 */
		/*addWindowListener(new WindowListener() {

			public void windowClosed(WindowEvent e) {
				log("Viewer window closed");
				logEvent(e);
			}

			public void windowClosing(WindowEvent e) {
				log("Viewer window closing");
				logEvent(e);
			}

			public void windowDeactivated(WindowEvent e) {
				log("Viewer window deactivated");
				logEvent(e);
			}

			public void windowIconified(WindowEvent e) {
				log("Viewer window iconified");
				logEvent(e);
			}

			public void windowActivated(WindowEvent e) {
				log("Viewer window activated");
				logEvent(e);
			}

			public void windowDeiconified(WindowEvent e) {
				log("Viewer window deiconified");
				logEvent(e);
			}

			public void windowOpened(WindowEvent e) {
				log("Viewer window opened");
				logEvent(e);
			}

			private void logEvent(WindowEvent e) {
				log("Window event source: " + e.getSource());
				logStackTrace();
			}
		});*/

		setLayout(new GridBagLayout());

		String icon = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2.gif";
		URL url = KreawiGlobals.class.getResource(icon);

		String iconRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_light.gif";
		URL urlRollover = KreawiGlobals.class.getResource(iconRollover);
		
		String iconPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_dark.gif";
		URL urlPressed = KreawiGlobals.class.getResource(iconPressed);
		
		JToolBar toolbar = KreawiInternalFrame.createLabeledIconButtonToolBar(
				new ImageIcon(url), new ImageIcon(urlRollover), new ImageIcon(
						urlPressed), "", "", new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
							  logger.info("Viewer window is going to close!");
//								disposalScheduled = true;
								setVisible(false);
								dispose();
							}
						});
					}
				});

		this.sif = new SimpleInternalFrame(null, title == null ? " " : title,
				toolbar, new JPanel(), false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		getContentPane().add(sif, gbc);
		getContentPane().setMaximumSize(owner.getSize());

		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				pressed = e;
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {

			private Point location;

			public void mouseDragged(MouseEvent me) {
				location = getLocation(location);
				int x = location.x - pressed.getX() + me.getX();
				int y = location.y - pressed.getY() + me.getY();
				setLocation(x, y);
			}
		});
	}

	/*@Override
	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING && !this.disposalScheduled) {
			log("Unscheduled window closing event ignored.");
			logStackTrace();
		} else if (e.getID() == WindowEvent.WINDOW_ICONIFIED) {
			log("Window minimize event ignored.");
			logStackTrace();
		} else {
			super.processWindowEvent(e);
		}
	}*/

	/*private void logStackTrace() {
		String stackTrace = "";
		for (StackTraceElement ste : Thread.currentThread().getStackTrace()) {
			stackTrace += ste.toString() + "\n";
		}
		log(stackTrace);
	}*/

}